<?php
/**
 * Send a requested VIDEO, AUDIO FILE or PLAYLIST to user from a redirected 'not found' request
 * this script is used for file access from embed or external audio players
 *
 * 3 kind of audio file can be processed :
 *	- audio files (.mp3, .wav...)
 *	- existing playlist (.m3u files)
 *	- PHP playlist (used by audio explorer scripts), named *playlist*.m3u[.xml]
 *   - single mp3 playlist (used by audio explorer scripts),named *singleMP3*.mp3.xml, used to bypass flash player limitation (cannot read somme characters like "&"...)
 * if a .xml extension is added on a m3u or a mp3 file, the actual output is converted to an XML playlist file designed for being red by flash player.
 *
 * All (mime) video files can be processed. Only simple "streaming" is supported (no playlist or XML stuff...)
 *
 * URI must be formated like "/ext/sess_$passedsessionId/resId$resId/[image/][w$width/][h$height/]urlencoded($filename)
 * where "$id" is an active session id, and "urlencoded(filename)" is url-encoded requested
 * file name (must include absolute path, and not relative path)
 *
 * If audio/$bitrate/ is specified, output file is reencoded with FFMPEG encoder
 *
 * If $filename is an existing m3u file, an modified m3u is sent to user, replacing song's filenames by
 * reformated filenames (/external/sess_$id/...)
 *
 * If $fileName is *playlist*.m3u or *playlist*.m3u.xml,
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */


//cfDbgUrl();
//define('DEBUG_REMOVE_HEADER',1);

require(INCLUDE_DIR.'explorerFunctions.php');
require(INCLUDE_DIR.'mime_type.php');

define('SESSION_NAME_PREFIX','sess_');

if(1) define('W_VIDEO_REDELAY','-redelay 5 ');
else define('W_VIDEO_REDELAY',''); // For use with a non-weezo version of ffmpeg



/**
 * @desc reencode video to flv using ffmpeg, and stream result to output
 *
 * @param string $completeFilename : original video file name
 * @param array $params: optional parameters:
 * 		  w & h: width & height (pixels)
 * 		  q: output quality ("sameq","high","medium","low")
 * 		  noSound: no sound
 * 		  offset: offset from start
 */


/**
 * @desc reencode video to flv using ffmpeg, and stream result to output
 *
 * @param string $completeFilename : original video file name
 * @param array $params:
 * 			w (width),
 * 			h (height),
 * 			q (quality),
 * 			noSound(),
 * 			chunkOffset (offset in seconds from begining, used by video chunking),
 * 			chunkLength(seconds),
 * 			offset(overriden by chunkOffset),
 * 			format ("flashFlv", "flashF4v", "html5Webm", "html5Ogv")
 * @param int $height
 * @param string $outputQuality
 */
function videoGenerateReecodedVideo($completeFilename, $params=array()){
	$preExtra='';

	// Reencoding format
	if(!($format=@$params['format'])) $format=(cfFfmpegVersion()=='GPL')?'flashF4v':'flashFlv';

	// Fix mms protocol description to mms over tcp (mms over udp not supported yet). Note: mmsh is for mms over http
	if(cfCmpLeft($completeFilename,'mms://')) $completeFilename='mmsh://'.substr($completeFilename,6);

	// Detect streams, where offsets cannot be used
	if(strpos($completeFilename,'://')) $streaming=true;

	// Video size
	$outputWidth =(isset($params['w']))?$params['w']:cfRGetVar('singleVideoWidth');
	$outputHeight=(isset($params['h']))?$params['h']:cfRGetVar('singleVideoHeight');

	// Set even values for width & height
	$outputHeight+=$outputHeight%2;
	$outputWidth+=$outputWidth%2;

	// Compute video bitrate
	switch ($outputWidth) {
		case 1280:
			$br=1000;break;
		case 900:
			$br=670;break;
		case 640:
			$br=400;break;
		case 400:
			$br=300;break;
		case 160:
			$br=25;	break;
		default:
			// Original size
			if(is_numeric($outputWidth)) {
				if($outputWidth<250) $br=max(floor(25*pow($outputWidth/160,2)),25); else $br=$outputWidth*3/4;
			}
			else $br=100;
	}

	// Global quality & sound quality
	$outputQuality=(isset($params['q']))?$params['q']:cfRGetVar('flashOutputQuality');
	switch ($outputQuality) {
		case 'original':
			$outputQuality='-qscale 1';
			if($format=='html5Ogv'||$format=='flashF4v') $outputQuality='-b '.(8*$br).'k'; // Bug WA with ogg/theora ?

			$sound=' -af aresample -ar 44100 -ab 384k -ac 2';
			break;
		case 'high':
			$outputQuality='-b '.(3*$br).'k';
			$sound=' -af aresample -ar 44100 -ab 128k -ac 2';
			break;
		case 'medium':
			$outputQuality='-b '.(2*$br).'k';
			$sound=' -af aresample -ar 44100 -ab 64k -ac 2';
			break;
		default:
			$outputQuality='-b '.(1*$br).'k';
			$sound=' -af aresample -ar 22050 -ab 32k -ac 1';
			break;
	}

	// Webcam streaming: tweak filename to stream webcam
	if(cfCmpLeft($completeFilename,'webcam:')) {
		require_once(INCLUDE_DIR.'webcamFunctions.php');
		$completeFilename=webcamFfmpegCommandLineInput($completeFilename,$soundMinQuality,$soundMaxQuality);
		if($outputQuality=='original' || $outputQuality=='high') $sound=$soundMaxQuality; else $sound=$soundMinQuality;
		$preExtra=' -f dshow ';
		$streaming=true;
	}

	// Put quotes around filename
	else $completeFilename='"'.$completeFilename.'"';

	// Mute
	if(isset($params['noSound'])) $sound=' -an';

	// Video Offset
	if(@$streaming) $offset=0;
	// 1st use current chunk offset if passed by flash player
	if(isset($params['chunkOffset']) && is_numeric($params['chunkOffset'])) $offset=$params['chunkOffset'];
	// Then use initial offset passed by flash player
	elseif(isset($params['offset']) && is_numeric($params['offset'])) $offset=$params['offset'];
	// Then use resource-set offset
	elseif(cfRGetVar('flashOutputOffset') && is_numeric(cfRGetVar('flashOutputOffset'))) $offset=cfRGetVar('flashOutputOffset').' ';
	// Or start at begining of file
	else $offset=0;
	//$offset=' -ss '.(floor($offset*100)/100);
	$offset=' -ss '.(floor($offset));

	// Encoding duration: only limit if slicing video into chunks
	if(cfIsWII()) unset($params['chunkLength']);
	$duration=(@($params['chunkLength']))?' -t '.($params['chunkLength']+1):'';

	// Encoding ratge
	$rate='-re '.W_VIDEO_REDELAY;
	if(@$streaming) $rate='';

	// Ffmpeg command line
	$cl='"'.cfFfmpegExe().'" '.$preExtra.$rate.$offset.' -i '.$completeFilename.'  '.$outputQuality.$sound.' -s '.$outputWidth.'x'.$outputHeight.' '.$duration;

	/**
	 * WebM with VP8/Vorbis
	 */
	if($format=='html5Webm'){
		$cl.=' -f webm -acodec libvorbis -vcodec libvpx -threads 4';
	}

	/**
	 * Ogg video with Theora/Vorbis
	 */
	if($format=='html5Ogv'){
		$cl.=' -f ogg -acodec libvorbis -vcodec libtheora -threads 4';
	}

	/**
	 * F4V with h264
	 */
	if($format=='html5H264'){
		$cl.=' -f m4v -acodec libmp3lame -vcodec libx264 -threads 4 -flags +loop -partitions +parti4x4+partp8x8+partb8x8 -subq 5 -trellis 1 -refs 1 -coder 0 -me_range 16 -keyint_min 25 -sc_threshold 40 -i_qfactor 0.71 -bt 200k -rc_eq \'blurCplx^(1-qComp)\' -qcomp 0.6 -qmin 10 -qmax 51 -qdiff 4 -level 30 -aspect '.$outputWidth.':'.$outputHeight.' -g 30';
	}
	/**
	 * F4V with h264
	 */
	if($format=='flashF4v'){
		// F4V with h264 (GPL ffmpeg required)
		if(cfFfmpegVersion()=='GPL')
			$cl.=' -f flv -acodec libmp3lame -vcodec libx264 -threads 4 -flags +loop -partitions +parti4x4+partp8x8+partb8x8 -subq 5 -trellis 1 -refs 1 -coder 0 -me_range 16 -keyint_min 25 -sc_threshold 40 -i_qfactor 0.71 -bt 200k -rc_eq \'blurCplx^(1-qComp)\' -qcomp 0.6 -qmin 10 -qmax 51 -qdiff 4 -level 30 -aspect '.$outputWidth.':'.$outputHeight.' -g 30';

		// If ffmpeg is not GPL, fallback on VP6 flv
		else
			$format='flashFlv';
	}

	/**
	 * FLV with VP6
	 */
	if($format=='flashFlv'){
		$cl.=' -f flv -acodec libmp3lame';
	}

	$cl.=' -';


	cfFfmpegLog($cl,$format.' video encoding');

	// Close session
	wSession_write_close();

	// Set speed limit
	$speedLimit=(int)((isset($params['downloadSpeedLimit']))?$params['downloadSpeedLimit']:cfUGetVar('downloadSpeedLimit'));
	unset($_SESSION);

	// Launch ffmpeg
	cfDebugUpdateBuffer('Launch ffmpeg: '.$cl);
	cfStreamProc($cl,array('stdout'=>'stdout'),array('speedLimit'=>$speedLimit));
	cfDebugSingle('End of '.$format.' reencoded streaming');
	exit;
}

/**
 * @desc reencode video to WMV using ffmpeg, and stream result to output - to be used for Windows Mobile streaming
 *
 * @param string $completeFilename : original video file name
 * @param int $width
 * @param int $height
 * @param string $outputQuality
 */
function videoGenerateWMVVideo($completeFilename, $params=array()){
	// Video size
	$outputWidth =(isset($params['w']))?$params['w']:cfRGetVar('singleVideoWidth');
	$outputHeight=(isset($params['h']))?$params['h']:cfRGetVar('singleVideoHeight');

	// Set even values for width & height
	$outputHeight+=$outputHeight%2;
	$outputWidth+=$outputWidth%2;

	// Global quality
	$outputQuality=(isset($params['q']))?$params['q']:cfRGetVar('flashOutputQuality');
	switch ($outputQuality) {
		case 'original':
			$outputQuality='-qscale 1';
			break;
		case 'high':
			$outputQuality='-b 256k';
			break;
		case 'medium':
			$outputQuality='-b 128k';
			break;
		default:
			$outputQuality='-b 64k';
			break;
	}

	// Sound
	if(!isset($params['noSound'])) $sound=' -acodec wmav2 -ar 11025 -ac 2 -ab 24k'; else $sound=' -an';

	// Offset
	if(isset($params['offset']) && is_numeric($params['offset'])) $offset=$params['offset'];
	elseif(cfRGetVar('flashOutputOffset') && is_numeric(cfRGetVar('flashOutputOffset'))) $offset=cfRGetVar('flashOutputOffset').' ';
	else $offset=0;
	$offset=' -ss '.(floor($offset*100)/100);

	// Ffmpeg command line
	$cl='"'.cfFfmpegExe().'" -re '.W_VIDEO_REDELAY.$offset.' -i "'.$completeFilename.'" -vcodec wmv2 '.$outputQuality.$sound.' -f asf -s '.$outputWidth.'x'.$outputHeight.' - ';
	cfFfmpegLog($cl,'WMV video encoding');

	// Close session
	wSession_write_close();
	$speedLimit=cfUGetVar('downloadSpeedLimit');
	unset($_SESSION);

	// Launch ffmpeg
	cfStreamProc($cl,array('stdout'=>'stdout'),array('speedLimit'=>$speedLimit));
	exit;
}

/**
 * @desc Check published image access rights and send to output
 *
 * @param array $uri: parsed URI ("previewImg"=>1 to generate a snapshot)
 */
function videoPublished($uri){
	$id=$uri['dlToken'];
	require_once(INCLUDE_DIR.'explorerFunctions.php');
	$tokenList=efTokensRead();

	// If link is not published video or streamed direct link, exit
	if(!(@$token=$tokenList[$id])) cfLog('invalid published video id ('.$id.')',LOG_DBG);
	if($token['type']!=='publishVideo' && ($token['type']!=='directLink' || @$token['streaming']!=='video')) cfLog('invalid published video id ('.$id.')',LOG_DBG);

	// Check file's existence
	if(!is_file($token['filename'])) logError('perempted published video ('.$token['filename'].')');

	// Video screenshot
	if(isset($uri['previewImg'])){
		require_once(INCLUDE_DIR.'explorerFunctions.php');
		header('Content-type: image/jpg');
		efMakeVideoThumbnail($token['filename'],false,$token['w'],$token['h'],70,30);
		exit;
	}

	// Count one download
	$tokenList[$id]['limitNb']--;
	efTokensWrite($tokenList);

	// Set options
	$options=array('w'=>$token['w'],'h'=>$token['h'],'q'=>$token['flashOutputQuality']);
	$options+=array('chunkLength'=>(isset($uri['chunkLength']))?$uri['chunkLength']:300);
	$options+=array('downloadSpeedLimit'=>0);
	if(isset($uri['offset'])) $options+=array('offset'=>$uri['offset']);
	if(isset($uri['chunkOffset'])) $options+=array('chunkOffset'=>$uri['chunkOffset']);
	if(isset($uri['encoding'])) $options+=array('encoding'=>$uri['encoding']);

	// Show message box on server
	cfServerShowMessageForm('Streaming',cfUTF8Encode(basename($token['filename'])),$_SERVER['REMOTE_ADDR'],cfAppDocRoot().'/gfx/directLinkVideo32.gif','#aad0e0','#84b7cc');

	// Generate video
	if(cfBGetVar('name')=='IEMobile') videoGenerateWMVVideo($token['filename'],$options);
	else videoGenerateReecodedVideo($token['filename'],$options);
}

/**
 * @desc Check minimum parameters and start session with sessionId passed in URL
 *
 */
function videoStartSession(&$uri){

	// Verify minimum data presence
	if(!isset($uri['video']) || !isset($uri['sess']) || !isset($uri['resId']) || !isset($uri['file'])) cfHTTPError(403);

	cfLog('extAccess.php: file requested : '.$_SERVER['REQUEST_URI'],LOG_DBG);


	// verify that passed session ID exists
	if(!($handle = opendir(wSession_save_path()))) cfHTTPError(403);
	while ((($file = @readdir($handle))!==false) && !isset($found)){if(is_file(wSession_save_path().'/'.$file) && $uri['sess']==$file) $found=true;}
	closedir($handle);

	// If passed session not found, display 404 and exit
	if(!isset($found)) cfHTTPError(404);

	/**
	 * START SESSION, and verifie that client's IP match with session's IP
	 */
	$uri['sess']=substr($uri['sess'],strlen(SESSION_NAME_PREFIX));
	wSession_id($uri['sess']);
	wSession_start();
	if($_SERVER['REMOTE_ADDR']!=$_SESSION['accountIP'] && cfGGetVar('disableClientSessionIPControl')!='true' && !isset($_SERVER['HTTPS'])) {
		cfLog('extAccess.php: passed session id does not match with IP => 403',LOG_ERROR);
		cfHTTPError(403);
	}

	// Set active resource id
	if(!is_numeric($uri['resId']) || !isset($_SESSION['res'][$uri['resId']]) || !is_array($_SESSION['res'][$uri['resId']]))  cfHTTPError(403);
	$_SESSION['activeResourceId']=$uri['resId'];
}

/**
 * @esc Unencode filename
 *
 * @param array $uri
 * @return string
 */
function videoUnencodeFilename($uri){
	// b64 encoded
	if(isset($uri['b64'])) $completeFilename=cfBase64_url_encode($uri['file']);

	// if encoding-decoding doesn't work then string is already utf8-encoded
	elseif(cfUTF8Encode(cfUTF8Decode($uri['file'],true,false,false,2048),false,true)!=$uri['file']){
		$completeFilename=$uri['file'];
	}
	else{
		$completeFilename=urldecode(cfUTF8Decode($uri['file'],true,true,false,2048));
	}

	$completeFilename=str_replace('&amp;','&',$completeFilename);
	$completeFilename=str_replace('*weezoSharp*','#',$completeFilename);


	// Restore real path
	if(cfSharedMode('list') && strpos($completeFilename,'*resourceBasePath*')!==false) $completeFilename=cfSharedCompleteFilename($completeFilename);
	else $completeFilename=str_replace('*resourceBasePath*',cfRGetVar('path'),$completeFilename);
	$completeFilename=str_replace('*resourceDataDirBasePath*',cfAppResourceDir(),$completeFilename);

	// Remove rnd part for playlists, singleMp3 and singleVideo
	if(substr($completeFilename,-17)=='*singleVideo*.xml') $completeFilename=substr($completeFilename,strrpos($completeFilename,'*',-10));

	if($completeFilename=='*singleVideo*.xml') return $completeFilename;

	/***********************************************************************************************************************************
	 * Check file access rights
	 **********************************************************************************************************************************/

	// Resource specific file source and rights function : bypass common file's existence and access rights
	// and (may) replace given file name by real file name
	if(cfRGetVar('extAccessFunction')) {
		$extAccessFunction = create_function('$completeFilename', cfRGetVar('extAccessFunction'));

		if(!($completeFilename=$extAccessFunction($completeFilename))) cfHTTPError(404);
	}
	// Check file's existence and access rights
	else{

		// If file doesn't exist, Display 404 and exit
		if(!@file_exists($completeFilename))	cfHTTPError(404);

		// If file download not allowed, Display 404 and exit
		if(!cfFileRights($completeFilename,'download')) cfHTTPError(403);

		// If file is not a video file, Display 404 and exit
		if($mt=mimeType(cfFileExtension($completeFilename))) $type=substr($mt,0,strpos($mt,"/")); else $type=false;
		if($type!='video') cfHTTPError(403);
	}

	return $completeFilename;
}

/**
 * @desc Output a single video XML playlist
 *
 * @param array $uri
 */
function videoSingleVideoXmlPlaylist($uri){
	// HEADER
	header('Content-Type: application/xml'); // Note: setting text/xml sometimes fails under opera
	header("Pragma: public"); // Required by IE6/7 in SSL mode
	header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); // No cache

	$src=cfRGetVar('singleVideo');

	if(isset($uri['chunkOffset']) && strpos($src,'/file'))	$src=substr($src,0,strpos($src,'/file')).'/chunkOffset.'.$uri['chunkOffset'].substr($src,strpos($src,'/file'));
	if(isset($uri['chunkOffset']) && strpos($src,'/b64'))	$src=substr($src,0,strpos($src,'/b64')).'/chunkOffset.'.$uri['chunkOffset'].substr($src,strpos($src,'/b64'));
	$src=cfHostName().cfUTF8Encode($src);

	$xml='<?xml version="1.0" encoding="utf-8"?>'."\n".'<playlist volume="'.((cfUGetVar('audioVolume')!==false)?cfUGetVar('audioVolume'):'80').'">'."\n<video file=\"".$src."\"/>\n";
	// Subtitles
	if(cfRGetVar('singleVideoSubtitles')) {
		require_once(INCLUDE_DIR.'subtitlesFunctions.php');
		$xml.=sfGenerateXMLSubs(cfRGetVar('singleVideoSubtitles'));
	}
	$xml.="\n</playlist>";

	cfDebugUpdateBuffer($xml);
	die($xml);
}

/**
 * @desc Compute requested content-range from headers
 * @param boolean $acceptRange: true if range requests are allowed
 *
 * @return array 'from','to' or false if no (satisfiable) range requested
 */
function videoGetContentRange($acceptRange,$completeFilename){
	// Check if client did a range request
	list($from,$to)=WHeadersIn::getRange();

	// process range request
	if($from!==false){
		$fileSize=@cfFileSize($completeFilename);

		// From
		if($from>$fileSize) $rangeErr=true;

		// To
		if($to===false) $to = $fileSize-1;
		elseif($to>$fileSize-1 || $to<$from) $rangeErr=true;

		// 0- range request: ignore
		if($from==0 || $to==$fileSize-1) return false;

		// Reject range requests on resampled files or m3u or xml
		if(!$acceptRange){
			header('HTTP/1.1 416 Requested Range Not Satisfiable');
			$range=false;
		}

		// Incorrect range
		if(isset($rangeErr)){
			header('HTTP/1.1 416 Requested Range Not Satisfiable');
			$range=false;
			cfDebugUpdateBuffer('Incorrect range requested: '.$from.'-'.$to);
		}
		else{
			header('HTTP/1.1 206 Partial Content');
			header("Content-Range: bytes $from-$to/$fileSize");

			$range=array('from'=>$from,'to'=>$to);
			cfDebugUpdateBuffer('Range requested: '.$from.'-'.$to);
		}
	}
	else $range=false;
	return $range;
}

/**
 * @desc Take a screenshot
 *
 * @param string $completeFilename
 * @param array $uri: parameters: w, h, offset(optional)
 */
function videoScreenshot($completeFilename,$uri){
	wSession_write_close();
	// Generate and send thumbnail to output
	if(efMakeVideoThumbnail($completeFilename,'',isset($uri['w'])?$uri['w']:0,isset($uri['h'])?$uri['h']:0,0,isset($uri['offset'])?$uri['offset']:20)) exit;

	// If failure, generate
	$im = imagecreate($uri['w'], $uri['h']);
	imagefilledrectangle($im, 0, 0, $uri['w'], $uri['h'], imagecolorallocate($im, 0, 0, 0));
	imagestring($im,5,$uri['w']/2-8, $uri['h']/2-8,'?',imagecolorallocate($im, 255, 255, 255));
	imagejpeg($im);
	imagedestroy($im);
	exit;
}

/**
 * @desc Send content-type, pragma, cache-control, etag, last-modified, expire, content-disposition and content-length headers
 *
 * @param string $completeFilename
 * @param $string $reencodeFormat
 * @param array $uri
 * @param boolean $acceptRange
 * @param integer $contentLength
 */
function videoSendHeaders($completeFilename,$reencodeFormat,$uri,$range,$acceptRange){

	$headersOut=array();

	// Mime type
	if($reencodeFormat=='flashFlv') $headersOut['Content-Type']='video/x-flv';	// Flash VP6
	elseif($reencodeFormat=='flashF4v') $headersOut['Content-Type']='video/f4v';// Flash H264
	elseif($reencodeFormat=='html5Webm') $headersOut['Content-Type']=mimeType('webm');// WebM
	elseif($reencodeFormat=='html5Ogv') $headersOut['Content-Type']=mimeType('ogv');// Ogg Video
	elseif(in_array($reencodeFormat,array('wmv'))) $headersOut['Content-Type']=mimeType($reencodeFormat);	// WMV, WebM or Ogg Video
	elseif(isset($uri['image']))$headersOut['Content-Type']='image/jpg';		// Screenshot
	else $headersOut['Content-Type']=mimeType($completeFilename); 				// Not reencoded video


	// Caching
	$cacheFile=substr($completeFilename,0,13)!=='*singleVideo*' && !$reencodeFormat;

	// Cache
	if($cacheFile){
		$headersOut['Pragma']=''; // Do not remove, required by IE in SSL mode
		$headersOut['Cache-Control']='private';
		$headersOut['ETag']='"'.md5($completeFilename).'"';
		$headersOut['Last-Modified']=gmdate("D, d M Y H:i:s",@filemtime($completeFilename)).' GMT';
		$headersOut['Expires']=gmdate("D, d M Y H:i:s",time()+3600*24*100).' GMT';
	}
	// No cache
	else{
		$headersOut['Pragma']=(cfIsWII())?'no-cache':''; // Do not remove, required by IE in SSL mode
		$headersOut['Cache-Control']='no-store, no-cache, must-revalidate, post-check=0, pre-check=0';
		$headersOut['Expires']="Mon, 26 Jul 1997 05:00:00 GMT";
	}

	if(!isset($uri['image'])){

		// Set content-length if possible
		if($range) $contentLength=1+$to-$from;
		elseif(cfFileExtension($completeFilename)=='m3u' || cfFileExtension($completeFilename)=='xml') $contentLength=strlen($convertedM3UFile);
		elseif(cfIsWII()) $contentLength=999999999;
		elseif (!$reencodeFormat) $contentLength=@filesize($completeFilename);

		if(@$contentLength) $headersOut['Content-Length']=$contentLength;

		// Misc
		$headersOut['X-Powered-By:']='';
		$headersOut['Content-Disposition']='inline; filename="'.cfUTF8Encode(basename($completeFilename),false,false).'"';
		$headersOut['Accept-Ranges']=($acceptRange)?'bytes':'none';
	}
	// Send headers
	if(!defined('DEBUG_REMOVE_HEADER')) foreach ($headersOut as $k=>$v) header($k.':'.(($v)?' '.$v:''));
}

// Decode and parse URI
$uri=cfParseExtURI();

// Debug
//cfDebugSetBuffer('video',$uri);
//cfDbg($uri,1);
//if(isset($uri['encoding'])) cfDbg($uri);
//cfDbg(getallheaders());
//cfDbgUrl();

// Published video
if(isset($uri['dlToken'])) videoPublished($uri);

// Check minimum parameters and start session with sessionId passed in URL
videoStartSession($uri);


/**
 * unencode filename, check file exists and access is allowed
 */
$completeFilename=videoUnencodeFilename($uri);


/**
 * Flash single video XML playlist
 */
if($completeFilename=='*singleVideo*.xml') videoSingleVideoXmlPlaylist($uri);



// Flash video real-time reencoding
$reencodeFormat=@$uri['format'];


/**
 * iPhone-specific video processing: process through videoIPhone.php script
 */
if(in_array($reencodeFormat,array('m3u8sa','m3u8','ts'))) require(cfAppDocRoot().'/ext/videoIPhone.php');


// Check if a range request can be satified
$acceptRange=!$reencodeFormat;
$range=videoGetContentRange($acceptRange,$completeFilename);

// HTTP Headers out
videoSendHeaders($completeFilename,$reencodeFormat,$uri,$range,$acceptRange);

// Still image (screenshot) generate screenshot (and exit)
if(isset($uri['image'])) videoScreenshot($completeFilename,$uri);


// Log in events log
cfLog('/ext/video.php: file download initiated:'.$completeFilename,LOG_DBG);

// Log in viewed content history
cfLogContentAccess($completeFilename);

// Debug
cfDebugUpdateBuffer('Headers sent, requested file: '.$completeFilename);

// Real-time ffmpeg video flash reencoding (then exit)
if($reencodeFormat=='wmv') videoGenerateWMVVideo($completeFilename, $uri);

// Real-time ffmpeg video flash reencoding (then exit)
if($reencodeFormat) videoGenerateReecodedVideo($completeFilename, $uri);

// Stream raw file
cfStreamFile($completeFilename,false,cfUGetVar('downloadSpeedLimit'),$range);
?>