<?php
/**
 * Common bookmarks functions
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * Bookmark structure (stored in "bookmark" resource data)
 *  array(4) {
    ["id"]=> string: unique id of item
    ["name"]=> string: UTF8 label
    ["type"]=> string: b=bookmark, f=folder, q=end of folder
    ["url"]=>  string: URL (bookmarks only)
 *
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/**
 * @desc Initialize resource at 1st login
 *
 */
function bfInit(){
	require_once(INCLUDE_DIR.'miscFunctions.php');

	if(!cfRGetVar('resourceParametersInitialized')){
		// Size of generated thumbnails (full-size)
		cfRInitVar('thumbnailSize',640);
		cfRInitVar('thumbnailRatio',0.75);

		cfRInitVar('thumbnailSizeIcons',120);
		cfRInitVar('thumbnailSizeFull',200);

		cfRInitVar('monitorDelay',4000);

		// Display treeView panel
		cfRInitVar('treeViewPanel',true);

		// Alphabetically sort treeView
		cfRInitVar('sortTreeView',true);

		// Layout type
		cfRInitVar('bookmarksLayout','icons');
		cfRInitVar('singleBookmarkLayout','thumbnail');

		if(!cfRGetVar('displayThumbnails')) cfRSetVar('bookmarksLayout','list');
		cfRInitVar('layoutIconsWidth',100);
		cfRInitVar('layoutFullWidth',140);

		cfRSetVar('resourceParametersInitialized',true);
	}

	cfRInitUserRightsVar('editionAllowed');

	// Persistent paramters
	cfSetSavedRVar('bookmarksLayout','singleBookmarkLayout');
}

/**
 * @desc Return default bookmarks path depending on browser
 *
 * @param string $browser : "IE", "Opera" or "Custom"
 * @return string path if found
 * 			boolean false if not found and $returnErrorCaption==false
 * 			string error caption if not found and $returnErrorCaption==true
 */
function browserBookmarksLocation($browser,$returnErrorCaption=false){
	// Try to find bookmarks location
	$browser=strtolower($browser);
	// Internet Explorer
	if($browser=='ie'){
		if(!function_exists('customQueryRegistry')) return false;
		return str_replace('\\','/',customQueryRegistry("HKEY_CURRENT_USER", "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders\\", "Favorites"));
	}
	// Firefox
	elseif($browser=='firefox'){
		$folder=strtr(cfGGetVar('specialFolderAppData'),"\\","/").'/Mozilla/Firefox/Profiles';
		if(is_dir($folder)){
			$matchingFolders = cfGlob($folder.'/*.default');
			foreach ($matchingFolders as $matchingFolder) if(file_exists($matchingFolder.'/places.sqlite')) return $matchingFolder;
			foreach ($matchingFolders as $matchingFolder) if(file_exists($matchingFolder.'/bookmarks.html')) return $matchingFolder;
			return (($returnErrorCaption)?cfCaption('bookmarksSourceNotFound',cfCaption('bookmarksFF')):false);
		}
		else return (($returnErrorCaption)?cfCaption('bookmarksSourceNotFound',cfCaption('bookmarksFF')):false);
	}
	// Opera
	elseif($browser=='opera'){
		$operaPath=false;
		// First check in documents & settings profiles directory (Opera 9)
		if(file_exists(strtr(cfGGetVar('specialFolderAppData'),"\\","/").'/Opera/Opera/profile/Opera6.adr')) $operaPath=strtr(getenv('APPDATA'),"\\","/").'/Opera/Opera';
		// Then check in default directory
		if ((!$operaPath) && file_exists(strtr(getenv('programfiles'),"\\","/").'/Opera/profile/Opera6.adr')) $operaPath = strtr(getenv('programfiles'),"\\","/").'/Opera';
		// Then check in registry to find installation directory
		if ((!$operaPath) && function_exists('customQueryRegistry') && customQueryRegistry("HKEY_LOCAL_MACHINE", "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Opera.exe\\", "Path")){
			$operaPath=customQueryRegistry("HKEY_LOCAL_MACHINE", "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Opera.exe\\", "Path").'/profile';
			$operaPath = str_replace('\\','/',$operaPath);
			if(!file_exists($operaPath.'/profile/Opera6.adr')) $operaPath = false;
		}
		if ((!$operaPath) && function_exists('customQueryRegistry') && customQueryRegistry("HKEY_CLASSES_ROOT", "HTTP\\shell\\open\\command\\", "")){
			$operaPath = customQueryRegistry("HKEY_CLASSES_ROOT", "HTTP\\shell\\open\\command\\", "");
			$operaPath = str_replace('\\','/',substr($operaPath,1,strrpos($operaPath,'\\')-1));
			if(!file_exists($operaPath.'/profile/Opera6.adr')) $operaPath = false;
		}
		if ((!$operaPath) && function_exists('customQueryRegistry') && customQueryRegistry("HKEY_CURRENT_USER", "Software\\Opera Software\\", "Last Directory3")){
			$operaPath = customQueryRegistry("HKEY_CURRENT_USER", "Software\\Opera Software\\", "Last Directory3");
			$operaPath = str_replace('\\','/',$operaPath);
			if(!file_exists($operaPath.'/profile/Opera6.adr')) $operaPath = false;
		}
		// Set path
		if($operaPath) return $operaPath.'/profile'; else return (($returnErrorCaption)?cfCaption('bookmarksSourceNotFound',cfCaption('bookmarksOpera')):false);
	}
	elseif($browser=='custom'){
		return cfAppSharedDir();
	}


	if(!isset($criticalErrorMessage)) cfRSetVar('parametersLoaded',true);
}

/**
 * @return bool result
 * @param integer $index : index of bookmark to delete
 * @desc delete bookmark
*/
function deleteBookmark($index){
	// Internet Explorer 6/7
	if(cfRGetVar('bookmarksType')=='IE'){
		$file=cfRGetVar('bookmarks',$index,'path').'/'.cfRGetVar('bookmarks',$index,'name').'.url';
		if(!(@unlink($file))) {cfLog('f='.__FUNCTION__.' l='.__LINE__.' Cannot delete IE bookmark file "'.$file.'"', LOG_ERROR); return false;} // delete bookmark
		$file=cfAppResourceDir().substr(cfRGetVar('bookmarks',$index,'path'),strlen(cfRGetVar('bookmarksPath'))).'/'.cfRGetVar('bookmarks',$index,'name').'.url.jpg';
		if(file_exists($file)) unlink($file); // delete thumbnail (if exists)
		cfLog('f='.__FUNCTION__.' l='.__LINE__.' Successfully deleted IE bookmark file "'.$file.'"', LOG_DBG);
		return true;
	}

	// Opera / custom
	if(cfRGetVar('bookmarksType')=='Opera' || cfRGetVar('bookmarksType')=='Custom'){
		$id=cfRGetVar('bookmarks',$index,'id');
		// Get bookmarks file
		if(cfRGetVar('bookmarksType')=='Opera') $file=cfRGetVar('bookmarksPath').'/Opera6.adr';
		if(cfRGetVar('bookmarksType')=='Custom') {$file=cfRGetVar('bookmarksPath').'/bookmarks.adr'; if(!file_exists($file)) {$fh=fopen($file,'w'); fclose($fh);}}

		$inSearchedId=false;
		//read bookmarks file
		if(!file_exists($file) || !is_writeable($file)){cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera/Custom bookmark file "'.$file.'" for reading', LOG_ERROR); return false;}
		if(!($fh = fopen($file,'r'))) {cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera/Custom bookmark file "'.$file.'" for reading', LOG_ERROR); return false;} // open file
		$content=array();
		$currentURLContent=array();

		// note : "ID="  must be placed before other declarations...
		while ($line=fgets($fh)) {
			$lcLine=strtolower(trim($line));
			if($lcLine=='id='.$id) $inSearchedId=true;// searched id found
			if(substr($lcLine,0,1)=='#' || substr($lcLine,0,1)=='-'){// end of item
				if(!$inSearchedId) foreach ($currentURLContent as $key) $content[]=$key;// if item's ID is not searched ID, copy in output
				unset($currentURLContent);$currentURLContent=array();// clear temp lines
				$inSearchedId=false;
			}
			if(ord(substr($line,-1,1))!=13 && ord(substr($line,-1,1))!=10) $line=$line."\n";
			$currentURLContent[]=$line;
		}
		if(!$inSearchedId) foreach ($currentURLContent as $key) $content[]=$key;// if item's ID is not searched ID, copy in output
		unset($currentURLContent);// clear temp lines
		fclose($fh);

		//write bookmarks file
		if(!$fh=fopen($file,'w')){cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera/Custom bookmark file "'.$file.'" for writing (delete bookmark)', LOG_ERROR); return false;}
		foreach ($content as $key) fwrite($fh,$key);
		fclose($fh);
		unset($content);
		cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Successfully deleted bookmark from file "'.$file.'"', LOG_DBG);
		return  true;
	}

	// Firefox 3
	if(cfRGetVar('bookmarksType')=='Firefox'){
		$bookmarksFile=cfRGetVar('bookmarksPath').'/places.sqlite';
		if(!@rename($bookmarksFile,$bookmarksFile)) return false;

		// Open database
		$pdo=new PDO('sqlite:'.$bookmarksFile,'','');

		// Remove entry in bookmarks table
		$result=$pdo->query('DELETE FROM moz_bookmarks WHERE id="'.cfRGetVar('bookmarks',$index,'id').'"',PDO::FETCH_ASSOC);

		// Return result
		return ($result->rowCount==1);
	}
}

/**
 * @desc Return url's full-size thumbnail filename (even if file doesn't exist)
 *
 * @param string $url
 * @return string
 */
function bfThumbnailFile($url){
	static $ts=0,$rd=0;
	if(!$ts) {
		$ts=cfRGetVar('thumbnailSize');
		$rd=cfAppResourceDir();
	}
	return $rd.'/'.md5($url).'.thumbnail.'.$ts.'.jpg';
}

/**
 * @desc Save FF3 bookmark
 *
 * @param string $bookmarkName : utf8 decoded name of bookmark
 * @param string $url : URL of bookmark
 * @return boolean result
 */
function saveNewBookmarkFF3($bookmarkName,$url){
	$bookmarksFile=cfRGetVar('bookmarksPath').'/places.sqlite';
	$time=time().'000000';

	// If file cannot be written, abort
	if(!@rename($bookmarksFile,$bookmarksFile)) return false;

	// Open database
	$pdo=new PDO('sqlite:'.$bookmarksFile,'','');

	// Get "menu" folder, in which new bookmark must be created
	$parent=false;
	foreach ($pdo->query('SELECT folder_id FROM moz_bookmarks_roots WHERE root_name="menu"',PDO::FETCH_ASSOC) as $value) $parent=$value;
	if(!$parent) return false;
	$parent=$parent['folder_id'];

	// Get bookmark sequential position (= max position + 1 of items with same parent)
	$position=false;
	foreach ($pdo->query('SELECT max(position) AS mx FROM moz_bookmarks WHERE parent="'.$parent.'"',PDO::FETCH_ASSOC) as $value) $position=$value;
	if(!$position) return false;
	$position=$position['mx']+1;

	// Insert bookmark in moz_places table
	$result=$pdo->query('INSERT INTO moz_places (url, title, rev_host, hidden, typed, frecency) VALUES ("'.sqlite_escape_string(cfUTF8Encode($url)).'","'.sqlite_escape_string(cfUTF8Encode($bookmarkName)).'","",0,0,30)',PDO::FETCH_ASSOC);
	// Get inserted id
	$result=$pdo->query('SELECT id FROM moz_places WHERE url="'.sqlite_escape_string(cfUTF8Encode($url)).'" AND title="'.sqlite_escape_string(cfUTF8Encode($bookmarkName)).'"',PDO::FETCH_ASSOC);
	$fk=false; foreach ($result as $value) $fk=$value;
	if(!$fk) return false;
	$fk=$fk['id'];

	// Insert bookmark in moz_bookmarks table
	$result=$pdo->query('INSERT INTO moz_bookmarks (type, fk, parent, position, title, dateAdded, lastModified) VALUES ("1", "'.$fk.'","'.$parent.'","'.$position.'","'.sqlite_escape_string(cfUTF8Encode($bookmarkName)).'","'.$time.'","'.$time.'")',PDO::FETCH_ASSOC);

	// Get inserted id
	$result=$pdo->query('SELECT id FROM moz_bookmarks WHERE fk="'.$fk.'"',PDO::FETCH_ASSOC);
	$place_id=false; foreach ($result as $value) $place_id=$value;
	if(!$place_id) return false;
	$place_id=$place_id['id'];

	// Insert bookmark in moz_historyvisits table
	$result=$pdo->query('INSERT INTO moz_historyvisits (from_visit, place_id, visit_date, visit_type, session) VALUES ("0", "'.$fk.'","'.$time.'","2","0")',PDO::FETCH_ASSOC);
	return true;
}

/**
 * @return bool result
 * @param string $bookmarkName : utf8 decoded name of bookmark
 * @param string $url : URL of bookmark
 * @desc save new bookmark
*/
function saveNewBookmark($bookmarkName,$url){
	// IE
	if(cfRGetVar('bookmarksType')=='IE'){
		$file=cfRGetVar('bookmarksPath').'/'.$bookmarkName.'.url';
		if(!($fh=fopen($file,'w'))) {cfLog('f='.__FUNCTION__.' l='.__LINE__.' Cannot create IE bookmark file "'.$file.'"', LOG_ERROR); return false;}
		fwrite($fh,"[InternetShortcut]\n");
		fwrite($fh,'URL='.$url);
		fclose($fh);
		cfLog('f='.__FUNCTION__.' l='.__LINE__.' Successfully created new IE bookmark file "'.$file.'"', LOG_DBG);
		return true;
	}

	// FF 3
	if(cfRGetVar('bookmarksType')=='Firefox') return saveNewBookmarkFF3($bookmarkName,$url);

	// Opera / Custom
	if(cfRGetVar('bookmarksType')=='Opera' || cfRGetVar('bookmarksType')=='Custom'){
		// Get bookmarks file
		if(cfRGetVar('bookmarksType')=='Opera') $file=cfRGetVar('bookmarksPath').'/Opera6.adr';
		if(cfRGetVar('bookmarksType')=='Custom') {$file=cfRGetVar('bookmarksPath').'/bookmarks.adr'; if(!file_exists($file)) {$fh=fopen($file,'w'); fclose($fh);}}

		$jobDone=false;
		$content=array();
		//read bookmarks file
		if(!file_exists($file) || !is_writeable($file)){cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera bookmark/Custom file "'.$file.'" for reading', LOG_ERROR); return false;}
		if(!($fh = fopen($file,'r'))) {cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera/Custom bookmark file "'.$file.'" for reading', LOG_ERROR); return false;} // open file
			$content=array();
			while ($line=fgets($fh)){
				if(!$jobDone && substr(trim($line),0,1)=='#'){
					$id=cfRGetVar('maxID')+1;
					cfRSetVar('maxID',$id);
					$content[]="#URL\n";
					$content[]='	ID='.$id."\n";
					$content[]='	NAME='.cfUTF8Encode($bookmarkName)."\n";
					$content[]='	URL='.$url."\n";
					$content[]="\n";
					$jobDone=true;
				}
				$content[]=$line;
			}
			if(!$jobDone){
				$id=cfRGetVar('maxID')+1;
				cfRSetVar('maxID',$id);
				$content[]="#URL\n";
				$content[]='	ID='.$id."\n";
				$content[]='	NAME='.cfUTF8Encode($bookmarkName)."\n";
				$content[]='	URL='.$url."\n";
				$content[]="\n";
			}
		fclose($fh);

		//write bookmarks file
		if(!$fh=fopen($file,'w')){cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera bookmark file "'.$file.'" for writing (save new bookmark)', LOG_ERROR); return false;}
		foreach ($content as $key) fwrite($fh,$key);
		fclose($fh);
		unset($content);
		cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Successfully inserted new bookmark in file "'.$file.'"', LOG_DBG);
		return  true;
	}
}

/**
 * @return bool result
 * @param string $index : index of bookmark
 * @desc save bookmark's comments
*/
function saveComments($index){
	// IE
	if(cfRGetVar('bookmarksType')=='IE'){
		$file=cfRGetVar('bookmarks',$index,'path').'/'.cfRGetVar('bookmarks',$index,'name').'.url';
		$content=array();
		if($fh=fopen($file,'r')){
			$comments=false;
			while ($line=fgets($fh)) {
				if(!strchr($line,'=') || strtolower(trim(substr($line,0,strpos($line,'='))))!='comments') {if(ord(substr($line,-1,1))!=13 && ord(substr($line,-1,1))!=10) $line=$line."\n"; $content[]=$line;}
			}
			fclose($fh);
			$content[]='comments = "'.cfRGetVar('bookmarks',$index,'comments').'"';
			if($fh=fopen($file,'w')){
				foreach ($content as $key) fwrite($fh,$key);
				unset($content);
				fclose($fh);
				cfLog('f='.__FUNCTION__.' l='.__LINE__.' Successfully updated bookmark file "'.$file.'" comments', LOG_DBG);
				return  true;
			}
			else {cfLog('f='.__FUNCTION__.' l='.__LINE__.' Cannot open bookmark file "'.$file.'" for reading', LOG_ERROR); return false;}
		}
		else {
			cfLog('f='.__FUNCTION__.' l='.__LINE__.' Cannot open bookmark file "'.$file.'" for reading', LOG_ERROR);
			return false;
		}
	}
	// Opera or Custom
	if(cfRGetVar('bookmarksType')=='Opera' || cfRGetVar('bookmarksType')=='Custom'){
		// Get bookmarks file
		if(cfRGetVar('bookmarksType')=='Opera') $file=cfRGetVar('bookmarksPath').'/Opera6.adr';
		if(cfRGetVar('bookmarksType')=='Custom') {$file=cfRGetVar('bookmarksPath').'/bookmarks.adr'; if(!file_exists($file)) {$fh=fopen($file,'w'); fclose($fh);}}

		$id=cfRGetVar('bookmarks',$index,'id');
		$inSearchedId=false;
		//read bookmarks file
		if(!file_exists($file) || !is_writeable($file)){cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera/Custom bookmark file "'.$file.'" for reading', LOG_ERROR); return false;}
		if(!($fh = fopen($file,'r'))) {cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera/Custom bookmark file "'.$file.'" for reading', LOG_ERROR); return false;} // open file
			$comments=false;
			$content=array();

			// note : "ID="  must be placed before other declarations...
			while ($line=fgets($fh)) {
				$lcLine=strtolower(trim($line));
				if($lcLine=='id='.$id) {// searched id found
					if(ord(substr($line,-1,1))!=13 && ord(substr($line,-1,1))!=10) $line=$line."\n";
					$content[]=$line;
					$content[]='	NAME='.cfUTF8Encode(cfRGetVar('bookmarks',$index,'name'))."\n";
					$content[]='	DESCRIPTION='.cfUTF8Encode(cfRGetVar('bookmarks',$index,'comments'))."\n";
					$inSearchedId=true;
				}
				else {
					if(!($inSearchedId && (substr($lcLine,0,12)=='description=' || substr($lcLine,0,5)=='name='))){
						if($inSearchedId && (substr($lcLine,0,1)=='#' || substr($lcLine,0,1)=='-')) $inSearchedId=false;
						if(ord(substr($line,-1,1))!=13 && ord(substr($line,-1,1))!=10) $line=$line."\n";
						$content[]=$line;
					}
				}
			}
		fclose($fh);
		//write bookmarks file
		if(!$fh=fopen($file,'w')){cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera/Custom bookmark file "'.$file.'" for writing comments', LOG_ERROR); return false;}
		foreach ($content as $key) fwrite($fh,$key);
		unset($content);
		fclose($fh);
		cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Successfully updated bookmark file "'.$file.'" comments', LOG_DBG);
		return  true;
	}
}

/**
 * @return new index
 * @param string $path : path to search
 * @param string $index : current index (bookmark number)
 * @param string $level : current sub level number
 * @desc recursively generate bookmarks tree with IE bookmarks
*/
function readIEBookmarks($path, $index, $level){
	cflog('Search IE bookmarks in '.$path,LOG_DBG);

	if(($handle = opendir($path))==false) {cflog('IE bookmarks generation : cannot open '.$path,LOG_DBG);return $index;}

	if(!isset($_ENV['readIEBookmarksIndex'])) $_ENV['readIEBookmarksIndex']=0;


	// set folder header
	cfRSetVarArray('bookmarks',$index);
	cfRSetVar('bookmarks',$index,'name',substr($path,strrpos($path,'/')+1));
	cfRSetVar('bookmarks',$index,'id',$_ENV['readIEBookmarksIndex']++);
	cfRSetVar('bookmarks',$index,'type','f');
	cfRSetVar('bookmarks',$index,'level',$level);
	$index++;
	// browse $path's bookmarks
	while (($file = @readdir($handle))!==false) if(is_file($path.'/'.$file) && cfFileExtension($file)=='url'){
		cfDebugSingle(file_get_contents($path.'/'.$file));
//cfDbg('       >          '.($path.'/'.$file),'');
//cfDbg(file_get_contents($path.'/'.$file),'');
		// scan file for "URL="
		if($fh=fopen($path.'/'.$file,'r')){
			$urlFound=false;
			$comments=false;
			while ($line=fgets($fh)) {
				if(strchr($line,'=') && strtolower(trim(substr($line,0,strpos($line,'='))))=='url') {
					$url=strtolower(trim(substr($line,strpos($line,'=')+1)));
					if((substr($url,0,10))!='javascript'){
						// add to bookmarks list
						cfRSetVarArray('bookmarks',$index);
						cfRSetVar('bookmarks',$index,'name',cfFileWithoutExtension($file));
						cfRSetVar('bookmarks',$index,'id',$_ENV['readIEBookmarksIndex']++);
						cfRSetVar('bookmarks',$index,'type','b');
						cfRSetVar('bookmarks',$index,'url',strtolower(trim(substr($line,strpos($line,'=')+1))));
						cfRSetVar('bookmarks',$index,'path',$path);
						$index++;
						$urlFound=true;
					}
				}
				if(strchr($line,'=') && strtolower(trim(substr($line,0,strpos($line,'='))))=='comments') {$comments=(trim(substr($line,strpos($line,'=')+1)));$comments=substr($comments,1,strlen($comments)-2);}
			}
			fclose($fh);
			if($urlFound && $comments) cfRSetVar('bookmarks',$index-1,'comments',$comments);
		}
	}
	closedir($handle);

	// browse subfolders for bookmarks
	$handle = opendir($path);
	while (($file = @readdir($handle))!==false) if(is_dir($path.'/'.$file) && $file!='.' && $file!='..') $index=readIEBookmarks($path.'/'.$file,$index,(($level) +1));
	closedir($handle);

	// set folder queue
	cfRSetVarArray('bookmarks',$index);
	cfRSetVar('bookmarks',$index,'id',$_ENV['readIEBookmarksIndex']++);
	cfRSetVar('bookmarks',$index,'name',substr($path,strrpos($path,'/')));
	cfRSetVar('bookmarks',$index,'type','q');
	cfRSetVar('bookmarks',$index,'level',$level);
	$index++;

	return $index;
}

/**
 * @return new index
 * @param string $path : path to search
 * @param string $index : current index (bookmark number)
 * @param string $level : current sub level number
 * @desc recursively generate bookmarks tree with FF bookmarks from XML bookmark tree
*/
function readFFBookmarks($element, $index, $level){
	if(!isset($_ENV['readFFBookmarksIndex'])) $_ENV['readFFBookmarksIndex']=0;

	// set folder header
	cfRSetVarArray('bookmarks',$index);
	cfRSetVar('bookmarks',$index,'type','f');
	cfRSetVar('bookmarks',$index,'level',$level);
	cfRSetVar('bookmarks',$index,'id',$_ENV['readFFBookmarksIndex']++);
	foreach($element->childNodes as $node) {
		if($node->nodeType==XML_TEXT_NODE && strlen(trim($node->nodeValue))>0) {
			cfRSetVar('bookmarks',$index,'name',str_replace('&amp;','&',trim($node->nodeValue)));
		}
	}
	$index++;

	// Set subfolders
	foreach($element->childNodes as $node) {
		if($node->nodeName=='H3') $index=readFFBookmarks($node,$index,$level+1);
	}
	foreach($element->childNodes as $node) {
		if($node->hasAttributes() && $node->getAttribute("HREF")){
			// add to bookmarks list
			cfRSetVarArray('bookmarks',$index);
			cfRSetVar('bookmarks',$index,'id',$_ENV['readFFBookmarksIndex']++);
			foreach($node->childNodes as $subNode) {
				if($subNode->nodeType==XML_TEXT_NODE && strlen(trim($subNode->nodeValue))>0) {
					cfRSetVar('bookmarks',$index,'name',cfUTF8Decode(str_replace('&amp;','&',trim($subNode->nodeValue)),true,false,false));
				}
			}
			if(strtolower((substr($node->getAttribute("HREF"),0,10)))!='javascript'){
				cfRSetVar('bookmarks',$index,'type','b');
				cfRSetVar('bookmarks',$index,'url',$node->getAttribute("HREF"));
				$index++;
			}
		}
	}
	// set folder queue
	cfRSetVarArray('bookmarks',$index);
	cfRSetVar('bookmarks',$index,'name','');
	cfRSetVar('bookmarks',$index,'type','q');
	cfRSetVar('bookmarks',$index,'level',$level);
	cfRSetVar('bookmarks',$index,'id',$_ENV['readFFBookmarksIndex']++);
	$index++;
	return $index;

}

/**
 * @desc Generate bookmarks tree from Firefox 3 sqlite bookmarks database
 *
 * @param string $bookmarksFile: database file
 * @return number of nodes
 */
function readFF3Bookmarks($bookmarksFile){
	global $FF3folders, $FF3bookmarks, $FF3index;

	function recurseBuildFF3Bookmarks($parentFolderId, $level){
		global $FF3folders, $FF3bookmarks, $FF3index;

		// Browse folders
		foreach ($FF3folders as $folder){
			if($folder['parent']==$parentFolderId){

				// Folder start
				if($level==0) $level=1;
				$item=array('id'=>$folder['id'],
							'name'=>cfUTF8Decode(str_replace('&amp;','&',$folder['title']),true,false,false),
							'type'=>'f',
							'level'=>$level);
				cfRSetVar('bookmarks',(int)$FF3index,$item);
				$index=$FF3index;
				$FF3index++;

				// folder content
				recurseBuildFF3Bookmarks($folder['id'], $level+1);

				// If folder is empty, remove
				if($index==$FF3index-1){
					$FF3index--;
					cfRUnsetVar('bookmarks',$index);
				}
				// Else, insert folder queue
				else{
					cfRSetVarArray('bookmarks',$FF3index);
					cfRSetVar('bookmarks',$FF3index,'type','q');
					cfRSetVar('bookmarks',$FF3index,'level',$level);
					cfRSetVar('bookmarks',$FF3index,'doneBy','yes');
					$FF3index++;
				}
			}
		}
		// Browse bookmarks
		foreach ($FF3bookmarks as $bookmark){
			if($bookmark['parent']==$parentFolderId){
				if(strtolower((substr($bookmark['url'],0,10)))!='javascript'){
					cfRSetVar('bookmarks',$FF3index,'id',$bookmark['id']);
					cfRSetVar('bookmarks',$FF3index,'name',cfUTF8Decode(str_replace('&amp;','&',$bookmark['title']),true,false,false));
					cfRSetVar('bookmarks',$FF3index,'type','b');
					cfRSetVar('bookmarks',$FF3index,'url',$bookmark['url']);
					$FF3index++;
				}
			}
		}
	}

	// If file is locked (by firefox), create a copy so PDO can work on it
	if(!@rename($bookmarksFile,$bookmarksFile)){
		if(!file_exists(cfAppResourceDir().'/places.sqlite') || md5_file(cfAppResourceDir().'/places.sqlite')!=md5_file($bookmarksFile)){
			copy($bookmarksFile,cfAppResourceDir().'/places.sqlite');
		}
		// Open database
		$pdo=new PDO('sqlite:'.cfAppResourceDir().'/places.sqlite','','');
	}
	// Open database
	else $pdo=new PDO('sqlite:'.$bookmarksFile,'','');

	// Get "menu" folder, whose items must be placed directly below root
	$menu=false;
	foreach ($pdo->query('SELECT folder_id FROM moz_bookmarks_roots WHERE root_name="menu"',PDO::FETCH_ASSOC) as $value) $menu=$value;
	if(!$menu) return false;
	$menu=$menu['folder_id'];

	// Get folders
	$FF3folders=array();
	foreach ($pdo->query('SELECT * FROM moz_bookmarks WHERE type=2',PDO::FETCH_ASSOC) as $value) {
		//if($value['parent']==$menu) cfVarDump($value);
		if($value['parent']==$menu) $value['parent']=1;
		$FF3folders[]=$value;
	}

	// Get bookmarks
	$FF3bookmarks=array();
	foreach ($pdo->query('SELECT moz_bookmarks.id, moz_bookmarks.parent, moz_bookmarks.title, moz_places.url FROM moz_bookmarks INNER JOIN moz_places ON moz_bookmarks.fk=moz_places.id WHERE moz_bookmarks.type=1 AND moz_places.hidden=0',PDO::FETCH_ASSOC) as $value) {
		if($value['parent']==$menu) $value['parent']=1;
		$FF3bookmarks[]=$value;
	}

	cfRSetVar('bookmarks',array());
	recurseBuildFF3Bookmarks(0,0);
	$level=0;

	return $FF3index;
}

/**
 * @desc Transform HTML bookmarks page into readable XML
 *
 * @param string $completeFilename : HTML bookmarks file name
 * @return string XML code
 */
function preProcessFFBookmarks($completeFilename){
	if(!($fh = @fopen($completeFilename,'r'))) return false;
	$fc='';
	while ($line=fgets($fh)) {
		$line=ltrim($line);
		if(substr($line,0,4)!='<DD>') $fc.=$line;
	}
	fclose($fh);

	if(strpos($fc,'<TITLE>Bookmarks</TITLE>')===false || strpos($fc,'<DL>')===false) return false;
	$fc=substr($fc,strpos($fc,'<DL>'));
	$fc=str_ireplace('<p>','',str_ireplace('<dt>','',str_ireplace('<hr>','',str_ireplace('<dd>','',$fc))));
	$fc=str_ireplace('<p>','',str_ireplace('</dl>','</H3>',str_ireplace('<dl>','',str_ireplace('</h3>','',$fc))));
	$fc='<?xml version="1.0" encoding="UTF-8"?><H3>'.$fc;
	return str_replace('&','&amp;',$fc);
}

/**
 * @return integer number of nodes (folder start, folder end, bookmark) in generated bookmark tree
 * @desc generate bookmarks tree in resource var (cfRGetVar) 'bookmarks'
*/
function readBookmarks($bookmarksType=false){
	// If no specified bookmark type, use configuration
	if(!$bookmarksType) $bookmarksType=cfRGetVar('bookmarksType');

	// If bookmarks folder doesn't exist, create it
	if(!is_dir(cfRGetVar('bookmarksPath'))) cfMkdir(cfRGetVar('bookmarksPath'));

	// IE
	if($bookmarksType=='IE') return readIEBookmarks(cfRGetVar('bookmarksPath'),0,0);

	// Firefox
	elseif($bookmarksType=='Firefox'){
		// Verify bookmarks file's existence, and if not found, reset location
		if(!file_exists(cfRGetVar('bookmarksPath').'/bookmarks.html') && !file_exists(cfRGetVar('bookmarksPath').'/places.sqlite')) cfRSetVar('bookmarksPath',browserBookmarksLocation('Firefox'));

		// Firefox 3 bookmarks
		if(file_exists(cfRGetVar('bookmarksPath').'/places.sqlite')) return readFF3Bookmarks(cfRGetVar('bookmarksPath').'/places.sqlite');

		// Read bookmarks HTML file
		if(!($fc=preProcessFFBookmarks(cfRGetVar('bookmarksPath').'/bookmarks.html'))) return false;

		// Generate XML tree from modified HTML
		$dom = new DOMDocument();
		@$dom->loadXML($fc);
		@$root=$dom->documentElement;
		// Build bookmarks tree
		return readFFBookmarks($root,0,0);
	}

	// Opera or custom
	elseif($bookmarksType=='Opera' || $bookmarksType=='Custom'){
		// Get bookmarks file
		if($bookmarksType=='Opera') $file=cfRGetVar('bookmarksPath').'/Opera6.adr';
		if($bookmarksType=='Custom') {$file=cfRGetVar('bookmarksPath').'/bookmarks.adr'; if(!file_exists($file)) {$fh=@fopen($file,'w'); @fclose($fh);}}
		$file=str_replace('*appDir*',cfAppRootDir(),$file);

		if(!file_exists($file) || !is_readable($file)){cfLog('f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera bookmark file "'.$file.'" for reading', LOG_ERROR); return false;}
		if(!($fh = fopen($file,'r'))) {cfLog('s='.__FILE__.' f='.__FUNCTION__.' l='.__LINE__.' Cannot open Opera bookmark file "'.$file.'" for reading', LOG_ERROR); return false;} // open file

		cfRSetVar('maxID',0);
		$index=1;

		//init root folder
		cfRSetVar('bookmarks',array(0=>array('name'=>'root','type'=>'f','id'=>'0','level'=>'0')));

		$itemName=false;$url=false;$comments=false;$type=false;$id=false;$level=1;$index=1;$inTrash=false;
		while ($line = fgets($fh)){// read lines
			$line=trim($line);
			$lcLine=strtolower($line);
			if(substr($lcLine,0,12)=='trash folder') $inTrash=true;

			if($type=='folder' && !$inTrash){
				if(substr($lcLine,0,3)=='id=') $id=trim(substr($line,3));
				if(substr($lcLine,0,5)=='name=') $itemName=cfUTF8Decode(trim(substr($line,5)));
				if(substr($lcLine,0,1)=='#' || substr($lcLine,0,1)=='-'){
					if($itemName && $id){
						cfRSetVarArray('bookmarks',$index);
						cfRSetVar('bookmarks',$index,'name',$itemName);
						cfRSetVar('bookmarks',$index,'id',$id);
						cfRSetVar('bookmarks',$index,'type','f');
						cfRSetVar('bookmarks',$index,'level',$level);
						$index++; $level++;
						if($id>cfRGetVar('maxID')) cfRSetVar('maxID',$id);
					}
					$itemName=false;$url=false;$comments=false;$type=false;$id=false;
				}
			}

			if($type=='url' && !$inTrash){
				if(substr($lcLine,0,3)=='id=') $id=trim(substr($line,3));
				if(substr($lcLine,0,4)=='url=') $url=trim(substr($line,4));
				if(substr($lcLine,0,5)=='name=') $itemName=cfUTF8Decode(trim(substr($line,5)));
				if(substr($lcLine,0,12)=='description=') $comments=cfUTF8Decode(trim(substr($line,12)));
				if(substr($lcLine,0,1)=='#' || substr($lcLine,0,1)=='-'){
					if($itemName && $id && $url){
						cfRSetVarArray('bookmarks',$index);
						cfRSetVar('bookmarks',$index,'id',$id);
						cfRSetVar('bookmarks',$index,'name',$itemName);
						cfRSetVar('bookmarks',$index,'url',$url);
						if($comments) cfRSetVar('bookmarks',$index,'comments',$comments);
						cfRSetVar('bookmarks',$index,'type','b');
						cfRSetVar('bookmarks',$index,'level',$level);
						$index++;
						if($id>cfRGetVar('maxID')) cfRSetVar('maxID',$id);
					}
					$itemName=false;$url=false;$comments=false;$type=false;$id=false;
				}
			}
			if(substr($lcLine,0,7)=='#folder' && !$inTrash) $type='folder';
			if(substr($lcLine,0,4)=='#url' && !$inTrash) $type='url';
			if(substr($lcLine,0,1)=='-'){
				if($inTrash) {$inTrash=false;$itemName=false;$url=false;$comments=false;$type=false;$id=false;}
				else{
					$level--;
					cfRSetVarArray('bookmarks',$index);
					cfRSetVar('bookmarks',$index,'name','q');
					cfRSetVar('bookmarks',$index,'type','q');
					cfRSetVar('bookmarks',$index,'level',$level);
					$index++;
					$itemName=false;$url=false;$comments=false;$type=false;$id=false;
				}
			}
		}
		if($type=='url' && $itemName && $id && $url && strtolower(substr($url,0,10))!='javascript'){
			cfRSetVarArray('bookmarks',$index);
			cfRSetVar('bookmarks',$index,'id',$id);
			cfRSetVar('bookmarks',$index,'name',$itemName);
			cfRSetVar('bookmarks',$index,'url',$url);
			if($comments) cfRSetVar('bookmarks',$index,'comments',$comments);
			cfRSetVar('bookmarks',$index,'type','b');
			cfRSetVar('bookmarks',$index,'level',$level);
			$index++;
			if($id>cfRGetVar('maxID')) cfRSetVar('maxID',$id);
		}
		fclose($fh);
	}
	// set folder queue
	cfRSetVarArray('bookmarks',$index);
	cfRSetVar('bookmarks',$index,'name','q');
	cfRSetVar('bookmarks',$index,'type','q');
	cfRSetVar('bookmarks',$index,'level',$level-1);
	$index++;
	return $index;
}

/**
 * @desc Wrapper function to correct readBookmarks problems
 *
 * @param string $bookmarksType
 */
function bfReadBookmarks($bookmarksType=false){
	if(!($nb=readBookmarks($bookmarksType))) return 0;
	$b=array();
	foreach (cfRGetVar('bookmarks') as $k=>$v) {
		if($v['type']=='b' && substr(strtolower($v['url']),0,4)!=='http') {
			$touched=1;
			$nb--;
		}
		else $b[]=$v;
	}
	if(isset($touched)) cfRSetVar('bookmarks',$b);
	return $nb;
}

/**
 * @desc Recursively generate an URL=>title array from IE7 feeds
 *
 * @param string $folder : folder to explore
 * @param array $feedsArray
 * @return void
 */
function importIE7Feeds($folder, &$feedsArray){
	// Read XML .feed-ms
	foreach (cfGlob($folder.'/*.feed-ms') as $filename) {
		$txt=file_get_contents($filename);
		$pUrl=false;
		// Search for <url>FEED-URL</url></itemdata>
		while($endPos=stripos($txt,'</url></itemdata>')){
			$startPos=strripos(substr($txt,0,$endPos),'<url>');
			if($startPos){
				$url=substr($txt,$startPos+5,$endPos-$startPos-5);
				//cfDbg($url,1);
				$pUrl=@parse_url($url);
				if(isset($pUrl['host'])) break;
			}
			$txt=substr($txt,$endPos+1);
		}
		if(isset($url)) $feedsArray[$url]=cfUTF8Encode(ucfirst(cfFileWithoutExtension(str_ireplace('www.','',$pUrl['host']))));
	}
	// Subfolders
	foreach (cfGlob($folder.'/*') as $filename) if(is_dir($filename)) importIE7Feeds($filename,$feedsArray);
}

/**
 * @desc Convert bookmark flat tree to treeview formated array
 *
 * @param unknown_type $flat
 * @param unknown_type $index
 * @return unknown
 */
function convertBookmarksToTreeArray($flat,&$index=1,$sort=false){
	$bookmarks=array();
	$selectedItem=cfRGetVar('selectedItem');
	while ($index<count($flat)) {
		if(!isset($flat[$index])) {$index++;continue;}
		$item=$flat[$index];
		$index++;
		if($item['type']=='b') {
			$bookmarks[]=array(
				'ico'=>mfFaviconLink($item['url']),
				'id'=>$item['id'],
				'type'=>'b',
				'label'=>cfUTF8Encode($item['name'],false,false),
				'url'=>$item['url'],
				'com'=>isset($item['comments'])?$item['comments']:'',
				'sub'=>array()) +
				(($selectedItem==$item['id'])?array('selected'=>true):array());
		}
		elseif($item['type']=='f') {
			$bookmarks[$item['name']]=array(
				'ico'=>outIcon('folder'),
				'id'=>$item['id'],
				'type'=>'f',
				'label'=>cfUTF8Encode($item['name'],false,false),
				'sub'=>convertBookmarksToTreeArray($flat,$index)) +
				(($selectedItem==$item['id'])?array('selected'=>true):array());
		}
		elseif($item['type']=='q') {
			if(cfRGetVar('sortTreeView') && count($bookmarks)>1){
				$f=array();
				$b=array();
				foreach ($bookmarks as $v) {
					if($v['type']=='f') $f[strtolower($v['label']).$v['id']]=$v;
					else $b[strtolower($v['label']).$v['id']]=$v;
				}
				ksort($f);
				ksort($b);
				$bookmarks=array();
				foreach ($f as $item) $bookmarks[]=$item;
				foreach ($b as $item) $bookmarks[]=$item;
			}
			return $bookmarks;
		}
	}
}

/**
 * @desc Return true if user is allowed to edit bookmarks (comments, add, delete)
 *
 * @return bool
 */
function bfEditAllowed(){return cfRGetVar('editionAllowed');}

/**
 * @desc Transform a bookmark id into flat tree sequential number
 *
 * @param unknown_type $id
 * @return unknown
 */
function bfIdToSequential($id){foreach (cfRGetVar('bookmarks') as $k=>$v) if(isset($v['id']) && $v['id']==$id) return $k;}

/**
 * @desc Download favicons of bookmars list
 *
 */
function bfRetreiveFavicons(){
	require_once(INCLUDE_DIR.'miscFunctions.php');

	// Search for missing favicons
	foreach (cfRGetVar('bookmarks') as $k=>$v) if($v['type']=='b')	if(!mfFaviconFile($v['url'])) $new[]=$v['url'];
	if(!isset($new)) return;

	// Echo synchronous project stuff
	require_once(INCLUDE_DIR.'outputFunctions.php');
	outSyncProgress(false,false,count($new));

	// Retreive favicons
	mfGetFavicons($new,array('toFile'=>cfAppResourceDir()));

	// Reload page when done
	outSyncProgressUpdate('done');
}


/*
 ***************************************************************************************************************************
 *
 * Common HTML stuff...
 *
 ***************************************************************************************************************************
 */


/**
 * @return string name of command processed, or false if no command processed
 * @desc Process commands received through HTTP Post form or Post HTTPXlmRequest
*/
function bfProcessPOST(){
	global $errorMessage;
	global $errorMessageCritical;
	global $asyncResponse;

	if(!isset($_POST)) return;

	if(cfIsAsync()) cfAsyncHeader();

	// If modification POST command received and user may edit bookmarks
	if(bfEditAllowed()){
		// Delete bookmark
		if(isset($_POST['suppress'])) {
			// convert id to sequential number
			$i=bfIdToSequential($_POST['suppress']);
			// Delete bookmark from flat tree and from source if needed
			if(!deleteBookmark($i)) $errorMessage=cfCaption('genErrorSaving');
			else {
				// Unlink thumbnail and icon
				@unlink(bfThumbnailFile(cfRGetVar('bookmarks',$i,'url')));
				@unlink(mfFaviconFile(cfRGetVar('bookmarks',$i,'url')));
				// Regenerate tree
				cfRSetVar('bookmarksProcessed',false);
			}
			cfRUnsetVar('selectedItem');
		}

		// Save new title and Comments
		if(isset($_POST['editComment']) && isset($_POST['editTitle']) && isset($_POST['item'])) {
			// convert id to sequential number
			$i=bfIdToSequential($_POST['item']);

			// Process (clean) bookmark name
			//$newName=trim(ereg_replace('[\?/\\:\.\*"<>|]','',cfUTF8Decode($_POST['editTitle'],true,true,true,70)));
			$newName=trim(cfUTF8Decode($_POST['editTitle'],true,true,true,70));

			//If IE bookmark and new name, rename .url file with new name
			if(cfRGetVar('bookmarksType')=='IE' && $newName!=cfRGetVar('bookmarks',$i,'name')){
				// Remove forbidden chars
				$newName=trim(ereg_replace('[\?/\\:\*"<>|]','',cfUTF8Decode($_POST['editTitle'],true,true,true,70)));
				if(file_exists(cfRGetVar('bookmarks',$i,'path').'/'.$newName.'.url')) $errorMessage=cfCaption('genErrorSaving');
				else rename(cfRGetVar('bookmarks',$i,'path').'/'.cfRGetVar('bookmarks',$i,'name').'.url',cfRGetVar('bookmarks',$i,'path').'/'.$newName.'.url');
			}

			// process comments
			$newComments=nl2br(cfUTF8Decode($_POST['editComment'],true,true,false));

			// Save to file and in bookmarks array
			if(!isset($errorMessage)){
				cfRSetVar('bookmarks',$i,'comments',$newComments);
				cfRSetVar('bookmarks',$i,'name',$newName);
				if(!saveComments($i)) $errorMessage=cfCaption('genErrorSaving');
			}
		}

		// New bookmark
		if(isset($_POST['url']) && strlen($_POST['url'])>0){
			$url=cfUTF8Decode($_POST['url']);
			if(strtolower(substr($url,0,7))!='http://' && strtolower(substr($url,0,8))!='https://') $url='http://'.$url;
			$parsed=parse_url($url);

			if(!isset($parsed['host']) || ($parsed['host']!='localhost' && !strpos($parsed['host'],'.'))) $errorMessage=cfCaption('bookmarksInvalidURL'); // prod
			else {
				if(!isset($_POST['name']) || strlen($_POST['name'])==0) $bookmarkName=$parsed['host']; else $bookmarkName=cfUTF8Decode($_POST['name'], true, true, true, 128);
				if(!saveNewBookmark($bookmarkName,$url)) $errorMessage=cfCaption('genErrorSaving'); else cfRSetVar('bookmarksProcessed',false);
			}
		}
	}

	// Toggle tree view TD display
	if(isset($_POST['toggleTD'])){
		cfRSetVar('treeViewPanel',$_POST['treeViewPanel']);
	}

	// Item selected
	if(isset($_POST['selectItem'])){
		cfRSetVar('selectedItem',$_POST['selectItem']);
		bfUpdateDisplay($_POST['selectItem']);
	}

	// Change layout type
	if(isset($_POST['bookmarksLayoutSet'])){
		if($_POST['bookmarksLayoutSet']=='list' || (cfRGetVar('displayThumbnails') && ($_POST['bookmarksLayoutSet']=='icons'||$_POST['bookmarksLayoutSet']=='full'))){
			cfRSetVar('bookmarksLayout',$_POST['bookmarksLayoutSet']);
			bfUpdateDisplay();
		}
	}
	if(isset($_POST['singleBookmarkLayoutSet'])){
		if($_POST['singleBookmarkLayoutSet']=='web' || $_POST['singleBookmarkLayoutSet']=='thumbnail'){
			cfRSetVar('singleBookmarkLayout',$_POST['singleBookmarkLayoutSet']);
			bfUpdateDisplay();
		}
	}

	// Change treeView sorting
	if(isset($_POST['sortTreeView'])){
		cfRSetVar('sortTreeView',$_POST['sortTreeView']);
	}

	// Monitor newly created thumbnails
	if(isset($_POST['monitorTN'])) bfMonitorTN();

	if(cfIsAsync()) die(cfAsyncFooter());
}

/**
 * @desc : populate resource 'bookmarks' array with bookmarks data (function readBookmarks)
 *		and, if needed, ask application to generate (new) thumbnails
 */
function bfGenerateBookmarksTree(){
	if(!cfRGetVar('bookmarksProcessed')){ // If bookmarks tree has already been generated in this session, exit
		// Delete previous bookmarks tree
		cfRUnsetVar('bookmarks'); cfRSetVarArray('bookmarks');

		// Generate new bookmarks tree
		cfRSetVar('nbItems',$nbItems=bfReadBookmarks());

		// Download favicons
		bfRetreiveFavicons();

		// Generate / update thumbnails
		if(cfRGetVar('displayThumbnails')){
			// Seek inexisting and outdated thumbnails
			$rR=cfRGetVar('refreshRate')*3600*24;
			$gen=array();
			foreach (cfRGetVar('bookmarks') as $k=>$item) if($item['type']=='b'){
				$tnf=bfThumbnailFile($item['url']);
				if(!file_exists($tnf) || ($rR>0 && (time()-filectime($tnf))>$rR)){
					@unlink($tnf);
					$gen[basename($tnf)]=$item['url'];
				}
			}

			// If new thumbnails must be generated : save matching file and ask application to generate thumbnails
			if(count($gen)){
				cfWriteIniFile($gen,cfAppResourceDir().'/matching.txt',false);
				cfServerSendCommand('makeWebsitesThumbnails bookmarksFile="'.cfAppResourceDir().'/matching.txt" size="'.cfRGetVar('thumbnailSize').'"  jpgQuality="'.cfHGetVar('thumbnailJPGQuality').'"');
			}
		}

		// Select top node
		cfRSetVar('selectedItem',cfRGetVar('bookmarks',0,'id'));

		cfRSetVar('bookmarksProcessed',true);
		cfLog('Bookmarks (re)loaded ('.$nbItems.' items)',LOG_DBG);


	}
}

/**
 * @desc Generate treeview
 *
 * @return TreeView item
 */
function bfTreeView(){
	global $bookmarks;
	require_once(INCLUDE_DIR.'treeViewClass.php');
	$tree=array(
		'ico'=>'/res/bookmarks/resourceIconSmall.gif','path'=>'top',
		'id'=>@$bookmarks[0]['id'],
		'label'=>cfCaption('resBookmarks'),
		'sub'=>convertBookmarksToTreeArray(cfRGetVar('bookmarks'))
		);
	if(cfRGetVar('selectedItem')==$bookmarks[0]['id']) $tree['selected']=true;

	$treeView=new treeView('mainTree');
	$treeView->phpTree=$tree;
	$treeView->selectFunction='selectBM';
	return $treeView;
}

/**
 * @desc Insert HTML <head> and <body> parts for bookmark resources, with minimal javascript functions
 *
 * @param string $bodyExtraHTML : extra HTML code to include in <body> tag
 * @return void
 */
function bfInsertExplorerHeadAndBody($extraHTML=false){
	global $errorMessage;
	global $errorMessageCritical;
	global $asyncResponse;

	// If a critical error has been raised, display error page and exit
	if(isset($criticalErrorMessage)) outDisplayErrorPage($criticalErrorMessage);
	cfInsertHEAD();

	// if an error message is present, display it in a javascript alert box
	if(isset($errorMessage)) echo'<body onload="javascript:alert(\''.$errorMessage.'\')"'.(($extraHTML)?' '.$extraHTML:'').'>';
	else echo '<body'.(($extraHTML)?' '.$extraHTML:'').">\n";
	echo "<div id=\"topDiv\">\n";
}

/**
 * @desc : insert HTML code for hidden comments edition and new bookmark forms
 *
 */
function bfInsertCommentsEditionForm(){
	if(!bfEditAllowed()) return '';

	// Title / comments edition form
	echo '<div id="cePanel" style="display:none;position:absolute;top:240px;z-index:1000">';
	outShadowBefore('400px');
	echo outImage(outIcon('closeM'),false,'onclick="ce(\'hide\')" title="'.cfCaption('genClose').'"','position:absolute;right:-12px;top:-12px;cursor:pointer;z-index:100');
	echo '<div class="popup">';
	echo '<div class="popupHeader" style="text-overflow:ellipsis;overflow:hidden;white-space:nowrap">';
	echo '<img src="/res/bookmarks/resourceIconSmall.gif" class="ico" style="margin-right:0.5em" alt=""><span id="header"></span>';
	echo '</div>';
	echo '<form name="ceForm" method="post" enctype="multipart/form-data" action="'.$_SERVER['PHP_SELF'].'">';
	echo '<input type="hidden" name="resId" value="'.$_SESSION['activeResourceId'].'">';
	echo '<input type="hidden" name="item" id="item">';
	echo cfCaption('genTitle').'<br><div style="text-align:left;"><input class="textInput" type="text" name="editTitle" id="editTitleId" size="110" maxlength="110" class="normalFont" style="width:100%"></div><br><br>';
	echo cfCaption('genComments').'<br>';
	echo '<div style="text-align:center;"><textarea name="editComment" id="editCommentId" cols="110" rows="4" class="normalFont" style="width:100%"></textarea></div></form><br><center>';
	echo outButton(cfCaption('genSave'),"javascript:ce('submit',0)",outIcon('save'));
	echo '<span style="min-width:90px;"></span>';
	echo outButton(cfCaption('genCancel'),"javascript:ce('hide',0)",outIcon('cancel'))."</center>";
	echo "</div>\n";
	outShadowAfter();
	echo "</div>\n";

	// New bookmark form
	echo '<div id="nbPanel" style="display:none;position:absolute;top:240px;left:50%;z-index:1000;margin-left:-25%">';
	outShadowBefore('50em');
	echo '<div class="popup"><div class="popupHeader"><img src="'.outIcon('new').'" class="ico" style="margin-right:0.5em" alt="">'.cfCaption('bookmarksNewBookmark').'</div><br>';
	echo '<form name="newForm" method="post" enctype="multipart/form-data" action="'.$_SERVER['PHP_SELF'].'" style="white-space:nowrap"><span style="margin-right:1em;">'.cfCaption('genName').'</span><input class="textInput" type="text" size=25 name="name" id="name">';
	echo '<input type="hidden" name="resId" value="'.$_SESSION['activeResourceId'].'">';
	echo '<input type="hidden" name="nbItemId" id="nbItemId">';
	echo '<span style="margin-right:1em;margin-left:2em;">'.cfCaption('genURL').'</span><input class="textInput" type="text" size=40 name="url" id="url">';
	echo outButton(cfCaption('genAdd'),"javascript:D.newForm.submit();", outIcon('add'),false,false,'style="vertical-align:middle;"').'</form><br></div>';
	echo outImage(outIcon('closeM'),false,'onclick="newBookmark(\'hide\')" title="'.cfCaption('genClose').'"','position:absolute;right:-8px;top:-12px;cursor:pointer');
	outShadowAfter();
	echo "</div>\n";
}

/**
 * @desc Return JS object containing bookmark data
 *
 * @param bookmark $item
 * @param bool $monitor: true to return only thumnail file's existence (used for async tn creation monitoring)
 * @return string
 */
function bfJSItem($item,$monitor=false){
	$out ='{"id":"'.$item['id'].'",';
	$out.='"tnE":"'.file_exists(bfThumbnailFile($item['url'])).'"';
	if($monitor) return $out.'}';

	$out.=',"type":"b",';
	$out.='"url":"'.$item['url'].'",';
	$out.='"tn":"'.cfExtImage(bfThumbnailFile($item['url'])).'",';
	$out.='"ico":"'.mfFaviconLink($item['url']).'",';
	if(isset($item['comments'])) $item['com']=$item['comments'];
	if(isset($item['com'])) {
		$item['com']=str_replace("\n",'',str_replace(chr(13),'',$item['com']));
		$out.='"com":"'.addslashes((cfUTF8Encode($item['com'],false,false))).'",';
	}
	$out.='"label":"'.addslashes(isset($item['name'])?cfUTF8Encode($item['name']):$item['label']).'"}';
	return $out;
}

/**
 * @desc Return JS array containing bookmark data
 *
 * @param bookmark $item: folder item
 * @param bool $monitor: true to return only thumnail file's existence (used for async tn creation monitoring)
 * @return string
 */
function bfJSItemArray($item,$monitor=false){
	$bookmarks=cfRGetVar('bookmarks');
	$nb=0;
	foreach ($bookmarks as $k=>$v) {
		$nb++;
		if(isset($v['id']) && $v['id']==$item['id']){
			$out='';
			$tree=convertBookmarksToTreeArray($bookmarks,$nb);
			foreach ($tree as $v) if($v['type']=='b') $out.=','.bfJSItem($v,$monitor);
			return 'Array('.substr($out,1).')';
		}
	}
	return 'Array()';
}

/**
 * @desc Update display of thumbnails
 *
 * @param array $id: base item for display
 * @param bool $monitor: true to return only thumnail file's existence (used for async tn creation monitoring)
 */
function bfUpdateDisplay($id=false,$monitor=false){
	if($id===false) $id=cfRGetVar('selectedItem');

	// Browse bookmarks & folders to find selected item
	foreach (cfRGetVar('bookmarks') as $v) {
		if(isset($v['id']) && $v['id']==$id) {
			// Bookmark found
			if($v['type']=='b') $action='displaySingle('.bfJSItem($v,$monitor).')';
			// Folder found
			else $action='displayMultiple('.bfJSItemArray($v).')';
			if(cfIsAsync()) echo cfAsyncXMLJSaction($action); else echo $action;
			return ;
		}
	}
}

/**
 * @desc sent command for async updating display with newly created thumnails
 *
 */
function bfMonitorTN(){
	$item=cfRGetVar('selectedItem');

	// Browse bookmarks & folders to find selected item
	if(is_array(cfRGetVar('bookmarks'))) foreach (cfRGetVar('bookmarks') as $item) {
		if(isset($item['id']) && $item['id']==cfRGetVar('selectedItem')) {
			// Bookmark found
			if($item['type']=='b') $action='updateTN('.bfJSItem($item,1).')';
			else $action='updateTN('.bfJSItemArray($item,1).')';
			if(cfIsAsync()) echo cfAsyncXMLJSaction($action); else echo $action;
			return ;
		}
	}
}
?>