<?php
/**
 * Set of functions used by menu.php and background.php scripts
 *
 * Display XP-style menu, clock...
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.8
 */


/**
 * @desc Return true if user may configure his account
 *
 * @return boolean
 */
function fConfigEnabled(){return cfUGetVar('accountType')==='singleUser';}

/**
 * @desc Return javascript link opening user configuration window
 *
 * @return string
 */
function fConfigLink(){return "openResource('/res/administration/config/main.php','-1','".cfCaption('mainConfigButton')."','".outIcon('config3')."',{'dWW':340,'dWH':200})";}

/**
 * @desc Insert HTML & JS code for displaying server time
 *
 * @param boolean $showSeconds : set to true to show hour:min:sec, false to chow only hour:min
 */
function fInsertClock($showSeconds=false){
?><span id="timeSpan" style="cursor:pointer" onclick="toggleTime()"><?php echo @date('H:i:s')?></span>
<script type="text/javascript">
var menuWinOpened=new Array();
var dte=new Date();
var timeDelta=<?php echo time()*1000;?> - dte.getTime();
var timeConnected=<?php echo cfGGetVar('connectionTime')*1000;?>;
var timeDisplayed='time';
function twoD(number){return ((number<10)?'0'+number:number);}
function showTime(showSecs){
	if(!dgi("timeSpan")) return;
	var d=new Date();
	if(timeDisplayed=='time') d.setTime(d.getTime()+timeDelta); else d.setTime(d.getTime()+timeDelta-timeConnected);
	dgi("timeSpan").innerHTML=twoD(d.getHours())+':'+twoD(d.getMinutes())+((showSecs)?':'+twoD(d.getSeconds()):'');
}
function toggleTime(){
	timeDisplayed=((timeDisplayed=='time')?'connection':'time');
	showTime(<?php echo (($showSeconds)?'true':'false'); ?>);
}
setInterval("showTime(<?php echo (($showSeconds)?'true':'false'); ?>)", 900);
</script>
<?php
}

/**
 * @desc Insert javascript code required for windows bar management
 *
 */
function fInsertWindowsBar($menuWinWidth){
	// Empty button
	echo '<span id="winOpenedBlank" style="display:none">';

	// Prepare button for javascript replacements
	$bt=outBt('CAPTION',false,'ACTION','WINID',false,false,'up',str_replace('px','',$menuWinWidth),'left');
	echo str_replace('<img alt="" src="/gfx/v.gif">','IMG',str_ireplace('<s style="', '<s style="padding-left:0.4em;',str_ireplace('<s>', '<s style="">',$bt)));
	echo "</span>\n";
	?>
<span id="winOpened">&nbsp;</span>
<script type="text/javascript">
<?php // Function  called on win action, through background winMonitor() function ?>
function menuWin(action, id, icon, caption){
	if(action=='open') menuWinOpen(id,icon,caption);
	if(action=='close') menuWinClose(id,icon,caption);
	if(action=='activate') menuWinActivate(id,icon,caption);
	if(action=='setIcon') menuWinSetIcon(id,icon,caption);
	if(action=='setCaption') menuWinSetCaption(id,icon,caption);
}
<?php // On window open, create menuWin item if needed ?>
function menuWinOpen(id,icon,caption){
	if(!menuWinOpened[id]) {
		menuWinOpened[id]="notActive";
		var bt=dgi('winOpenedBlank').innerHTML.replace(/CAPTION/gi, caption);
		bt=bt.replace(/WINID/g, 'menu'+id);
		bt=bt.replace(/href="ACTION/, "href=\"javascript:menuWinClick('"+id+"')");
		bt=bt.replace(/ACTION/, "");
		if(navigator.appName == "Microsoft Internet Explorer" && navigator.appVersion.indexOf('MSIE 6.0')>0)
			icon='<img src='+icon.substring(icon.indexOf('src=')+4,icon.indexOf(', sizing'))+' width=16 height=16 style="border:0px;width:16px;height:16px">';
		bt=bt.replace(/>IMG/g, '>'+icon.replace(/100%/gi,'16px'));
		dgi('winOpened').innerHTML+=bt;
		menuWinBarResized();
	}
	return menuWinActivate(id,icon,caption);
}
function menuWinActivate(id,icon,caption){
	var dbg='';
	for(var key in menuWinOpened) {
		if(key==id) {
			if(menuWinOpened[key] && menuWinOpened[key]!='Active'){
				menuWinOpened[key]='Active';
				btActivate('menu'+key);
			}
		}
		else{
			if(menuWinOpened[key] && menuWinOpened[key]!='notActive'){
				menuWinOpened[key]='notActive';
				btEnable('menu'+key);
			}
		}
		if(menuWinOpened[key]) dbg+=key+'>'+menuWinOpened[key]+' - ';
	}
}
function menuWinClose(id,icon,caption){
	if(menuWinOpened[id]){
		var n;
		if(n=dgi('menu'+id)){
			dgi('winOpened').removeChild(n);
			menuWinBarResized();
		}
		menuWinOpened[id]=null;
	}
}
function menuWinClick(id){
	if(btState('menu'+id)=='up') return menuWinRequestActivation(id);
	if(btState('menu'+id)=='on') {
		if(win(id).state().substr(0,3)!='min') win(id).minimize();
		else menuWinRequestActivation(id);
	}
}
function menuWinRequestActivation(id){
	win(id).activate();
}
function menuWinSetIcon(id,icon,caption){
	if(!dgi('menu'+id)) return;
	if(navigator.appName == "Microsoft Internet Explorer" && navigator.appVersion.indexOf('MSIE 6.0')>=0)
		icon='<img src='+icon.substring(icon.indexOf('src=')+4,icon.indexOf(', sizing'))+' width=16 height=16 style="border:0px;width:16px;height:16px">';
	var bt=dgi('menu'+id).innerHTML;
	var btl=bt.substr(0,bt.toLowerCase().indexOf("><b>")+4)
	var btr=bt.substr(bt.toLowerCase().indexOf("<s style"));
	dgi('menu'+id).innerHTML=btl+icon.replace(/100%/gi,'16px;')+btr;
}
function menuWinSetCaption(id,icon,caption){
	if(!dgi('menu'+id)) return;
	btSetCaption('menu'+id,caption)
	menuWinBarResized();
}
function menuWinBarResized(){
	var wO=dgi('winOpened');
	var nbWin=wO.getElementsByTagName('fieldset').length;

	if(!nbWin) return;
	var r=Math.min((actualOffsetLeft(dgi('winOpenedOuter').nextSibling))<=0?9999:actualOffsetLeft(dgi('winOpenedOuter').nextSibling), D.body.offsetWidth-dgi('winOpenedOuter').nextSibling.offsetWidth) - 35;
	var w=Math.max(16,Math.min((r - dgi('winOpenedOuter').previousSibling.offsetWidth)/nbWin ,150));

	for(var i=0;i<wO.getElementsByTagName('s').length;i++){
		if(wO.getElementsByTagName('s')[i].style.width) wO.getElementsByTagName('s')[i].style.width=(w-16)+"px";
	}
	for(i=0;i<wO.getElementsByTagName('div').length;i++){
		if(wO.getElementsByTagName('div')[i].style.width) wO.getElementsByTagName('div')[i].style.width=(w)+"px";
	}

	for(i=0;i<dgi('winOpened').getElementsByTagName('fieldset').length;i++){
		btAdjustText(dgi('winOpened').getElementsByTagName('fieldset')[i].id)
	}
}
</script>
<?php
}

/**
 * @desc insert background mask used for XP-Style menu click-out detection and icons click-protection
 * 		 and menu management javascript
 *
 */
function fInsertMenuStuff($fadeSteps=0, $maskOpacity=1){
?>
<script type="text/javascript">
function menuFadeInComplete(){
//	dgi('menuMask').style.display="block";
}
function menuFadeOutComplete(){
	dgi('menu').style.display="none";
	wl.mask.hide();
//	dgi('menuMask').style.display="none";
}
function menuShow(){
	var m=dgi('menu'),s=m.style;
	m.className="menu";
	s.left=winPadding.left;
	s.bottom=winPadding.bottom;

	wl.mask.show(0,0,0,menuMaskClick);
	s.visibility='hidden';
	s.display='block';

	wl.mask.moveAbove(m)

	if(dgi('menuBt')) dgi('menuBt').style.zIndex=s.zIndex*1+1;
	menuShown=1;
	wl.anim({'item':m,'duration':600,'extraParams':{'destH':m.offsetHeight,'destY':m.offsetTop},'callbackComplete':menuFadeInComplete,'function':
		function(p,anim){
			var f=wl.animFunctionUnfold2(p,anim),e=anim.extraParams;
			return{'opacity':Math.min(2*p,1),'visibility':'visible','top':e.destY+(1-f)*e.destH}
		}});
	wl.setKeycodeListener(D,menuHide)
}
function menuMaskClick(){menuHide()}
function menuHide(kc){
	if(kc!==undefined && kc!=27) return;
	wl.removeKeycodeListener(D)

	var m=dgi('menu'),b=dgi('winBarOuter');

	wl.anim({'item':m,'duration':400,'extraParams':{'startH':m.offsetHeight,'startY':m.offsetTop},'callbackComplete':menuFadeOutComplete,'function':
		function(p,anim){
			var f=wl.animFunctionFold2(p,anim),e=anim.extraParams;
			if(p==1) return {'opacity':0,'top':e.startY}
			return {'opacity':f,'top':e.startY+(1-f)*e.startH}
		}});
	menuShown=0;
	wl.mask.hide();
}
function menuToggle(){if(menuShown) menuHide(); else menuShow();}
function menuBtHover(){
	changeImgSrc('menuBt','<?php echo cfAppThemeDir(true);?>/menuBtH.png');
	changeImgSrc('menuBt','<?php echo cfAppThemeDir(true);?>/menuBtH.png');
}
function menuBtOut(){
	changeImgSrc('menuBt','<?php echo cfAppThemeDir(true);?>/menuBt.png');
	changeImgSrc('menuBt','<?php echo cfAppThemeDir(true);?>/menuBt.png');
}
</script>
<?php
}
?>