<?php
/**
 * MySQL database creation / connection
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.1
 */

define('MYSQL_PASSWORDS_FILE',cfAppDataDir().'/MySQL.ini');

/**
 * @desc Initialize MySQL database for a Weezo resource
 * 		 If needed, Generate database, generate, set and save root password
 * @param string $databaseName: name of database to create. Resource ID will be automatically appened
 *
 * @return mixed: result, true if database was created and all is OK, database name if database newly created, false if error
 */

function mySQLInit($databaseName){
	// Add resource id to database name (except for root)
	$databaseName=mySQLDatabase($databaseName);
	// Check MySQL existence and databasename
	if($databaseName=='root' || !mySQLExe() || !mySQLValidName($databaseName)) return false;

	// If passwords file doesn't exist, init database
	if(!file_exists(MYSQL_PASSWORDS_FILE)){if(!mySQLInitRoot()) return false;}

	// Load stored password
	$passwords=parse_ini_file(MYSQL_PASSWORDS_FILE);

	// If password set, db (should be) OK
	if(isset($passwords[$databaseName])) {
		// See if database existance has been checked once since server start (used not to access database every time)
		$checked=cfMGetVar('weezoMySQLDatabasesChecked');
		if(isset($checked[$databaseName]) && $checked[$databaseName]>time()+60) return true;

		// Check that database exists
		$res=mySQLConnectDB('root');

		$result=mysql_query('SHOW DATABASES');
		while ($row = mysql_fetch_assoc($result)) {
			if(@$row['Database']==$databaseName) {
				// Set as checked (cache result)
				$checked[$databaseName]=time();
				cfMSetVar('weezoMySQLDatabasesChecked',$checked);
				return true;
			}
		}
	}

	// Generate password
	$password=cfGenerateID(16);
	$passwords[$databaseName]=$password;

	// Connect as root user
	mySQLConnectDB('root');

	// Create database and user
	mysql_select_db('mysql');
	mysql_query("CREATE DATABASE IF NOT EXISTS ".$databaseName." ;");
	mysql_query("CREATE USER ".$databaseName."@127.0.0.1 IDENTIFIED BY '".$password."';");
	mysql_query("GRANT USAGE ON * . * TO ".$databaseName."@127.0.0.1 IDENTIFIED BY '".$password."' WITH MAX_QUERIES_PER_HOUR 0 MAX_CONNECTIONS_PER_HOUR 0 MAX_UPDATES_PER_HOUR 0 MAX_USER_CONNECTIONS 0 ;");
	mysql_query("GRANT ALL PRIVILEGES ON ".$databaseName.".* TO ".$databaseName."@127.0.0.1 WITH GRANT OPTION");
	mysql_query('FLUSH PRIVILEGES');
	mysql_close();
	// Save password to file
	mySQLWritePasswords($passwords);

	return $databaseName;
}

/**
 * @desc Display MySQL table
 *
 * @param string $table: table name
 */
function mySQLDisplayTable($table){
	if(is_array($table)){
		$result=$table;
	}
	else{
		$query='SELECT * FROM '.$table;
		$result=mysql_query($query);
		echo "<b>".$table.'</b><br>';
	}

	if(!$result || mysql_num_rows($result)==0) echo 'Table vide<br/><br/>';
	else{
		echo '<table style="border:1px solid black; color:#550; font-family:arial; font-size:10px" cellspacing=0 cellpadding=0><tr>';

		$rowNb=0;

		echo '</tr>';
		while ($row = mysql_fetch_assoc($result)) {
			echo '<tr>';
			if($rowNb==0) {
				foreach ($row as $key=>$value) echo '<td style="font-weight:bold; padding:3px; background:#550; color:white; font-weight:bold; font-family:arial; font-size:10px">'.$key.'</td>';
				echo '</tr>';
			}
			foreach ($row as $k2=>$v2) {
				echo '<td style="border:1px dashed #AAA; padding:3px">';
				 if($k2=='ts') echo date('d/m/y h:i:s',$v2); else echo $v2;
				echo '</td>';
			}
			echo "</tr>\n";
			$rowNb++;
		}
		echo "</table>\n";
	}
}

/**
 * @desc Destroy a MySQL database and user
 *
 * @param string $resId : unique resource id
 * @return boolean result
 */
function mySQLDestroyResource($resId){
	mySQLConnectDB('root');
	mysql_select_db('mysql');

	$dbIds=explode("\n",@file_get_contents(MYSQL_PASSWORDS_FILE));
	foreach ($dbIds as $nb=>$line) if(strpos($line,$resId)){
		unset($dbIds[$nb]);
		$dbId=substr($line,0,strpos($line,'='));
		// Destroy user
		mysql_query("DROP USER '".$dbId."'@'127.0.0.1'");

		// Destroy database
		mysql_query("DROP DATABASE IF EXISTS `".$dbId."`");
		// Clean passwords file
		file_put_contents(MYSQL_PASSWORDS_FILE,join("\n",$dbIds));
		mysql_close();
		return true;
	}
	mysql_close();
	return false;
}

/**
 * @desc return resource unique Id :	if resource env is set, return cfRGetVar('id'),
 * 										else parse resource config data and return id entry
 *
 * @return string : resource unique Id
 */
function mySQLUniqueResourceId(){
	// Return resource ID
	if(isset($_SESSION['activeResourceId']) && strtolower($_SESSION['activeResourceId'])!='fooid') return cfRGetVar('id');
	// for extensions that use their own session, use stored weezo session to retreive id
	elseif(isset($_ENV['weezoSession']['res'][$_ENV['weezoSession']['activeResourceId']]['id'])) return $_ENV['weezoSession']['res'][$_ENV['weezoSession']['activeResourceId']]['id'];
	elseif(cfGGetVar('configResourceUniqueId')) return cfGGetVar('configResourceUniqueId');
	elseif(@$_ENV['resourceConfigFilename'] || cfGGetVar('resourceConfigFilename')){
		$resourceConfigFilename=((isset($_ENV['resourceConfigFilename']))?$_ENV['resourceConfigFilename']:cfGGetVar('resourceConfigFilename'));
		if(!($resData=cfParse_ini_file(cfAppDataDir().'/'.$resourceConfigFilename))) return false;
		if(!isset($resData['id'])) return false;
		else {
			cfGSetVar('configResourceUniqueId',$resData['id']);
			return $resData['id'];
		}
	}
	else return false;
}

/**
 * @desc Called on MySQL server connection error
 * 		Display error page
 *
 */
function connectionError($err=false){
	global $w_lng;
	require_once(INCLUDE_DIR.'outputFunctions.php');
	outDisplayErrorPage('MySQL connection error: '.$err,false,true);
}

/**
 * @desc write mysql users/passwords in MYSQL_PASSWORDS_FILE file
 *
 * @param array $passwords
 * @return boolean result
 */
function mySQLWritePasswords($passwords){
	if (!$fp = fopen(MYSQL_PASSWORDS_FILE, 'w')) return false;
	foreach ($passwords as $key=>$value) fwrite($fp,$key.'='.$value."\n");
	fclose($fp);
}
function mySQLValidName($name){return true;}

/**
* Return path to mysqld executable
*/
function mySQLExe(){
    if(file_exists($exe=cfAppDataRootDir().'/MySQL/bin/mysqld.exe')) return $exe;
    if(file_exists($exe=cfAppDataRootDir().'/mysql/bin/mysqld')) return $exe;
    return false;
}

/**
 * @desc Initialize MySQL for Weezo
 * 		Generate, set and save root password
 * @param boolean $changePassword : set to true to renew with randmo password or to new password value
 *
 * @return boolean : result : true if MySQL present and correctly configured
 */
function mySQLInitRoot($changePassword=false){

	// Check MySQL existence
    if(!mySQLExe()) return false;

	// Load stored password
	if(file_exists(MYSQL_PASSWORDS_FILE)) $passwords=parse_ini_file(MYSQL_PASSWORDS_FILE);
	else $passwords=false;

	// Check stored password
	if(isset($passwords['root']) && strlen($passwords['root'])){
		if(!$socket=@fsockopen('127.0.0.1',mySQLPort(),$errno, $errstr, 3)) connectionError('Connection failed (1) - Cannot open connection to MySQL on port '.mySQLPort().': '.$errstr); else fclose($socket);

		if(mysql_connect(mySQLHost(),'root',$passwords['root'])){
			mysql_close();
			if(!$changePassword) return true;
			$password=$passwords['root'];
		}
		else $password='';
	}
	else $password='';

	// Ensure something is listening at the mysql TCP port
	if(!$socket=@fsockopen('127.0.0.1',mySQLPort(),$errno, $errstr, 3)) connectionError('Connection failed (2) - Cannot open socket connection to MySQL on port '.mySQLPort().': '.$errstr); else fclose($socket);

	// Try to connect to MySQL server with blank password
	if(!mysql_connect(mySQLHost($database),'root',$password)) connectionError('Connection failed (4) - Cannot connect as root: '.cfUTF8Encode(mysql_error()));

	// Connect mysql database
	if(!mysql_select_db('mysql')) connectionError('Error selecting mysql configuration table: '.mysql_error());

	// Set new password
	if(!$password || $changePassword===true) $password=cfGenerateID(16); else $password=$changePassword;

	// Save and set password
	$passwords['root']=$password;
	if(mysql_query('SET PASSWORD FOR root@localhost=PASSWORD("'.$password.'")')){
		mySQLWritePasswords($passwords);
		mysql_query('FLUSH PRIVILEGES');
	}
	else
		connectionError('Error setting root password: '.mysql_error());

	mysql_query("DROP USER ``@`%`");
	mysql_close();
	return true;
}


/**
 * @desc return host for MySQL connection
 *
 * @return string : host name
 */
function mySQLHost($database=false){
	return '127.0.0.1:'.mySQLPort();
}

/**
 * @desc return port for MySQL database
 *
 * @return string : port number
 */
function mySQLPort($database=false){return cfGGetVar('mysqlPort');}

/**
 * @desc return user name for MySQL database
 *
 * @return string : user name
 */
function mySQLUser($database){
	if($database=='root') return $database;

	// Get resource unique Id
	if(!($ruid=mySQLUniqueResourceId())) return false;
	return strtolower(substr($database.$ruid,0,16));
}

/**
 * @desc return database name for MySQL connection
 *
 * @return string : user name
 */
function mySQLDatabase($database){return mySQLUser($database);}

/**
 * @desc return user password for MySQL connection
 *
 * @return string : password
 */
function mySQLPassword($database){
	if(!$database) return false;
	if(!file_exists(MYSQL_PASSWORDS_FILE)) {
		require_once(INCLUDE_DIR.'outputFunctions.php');
		outDisplayErrorPage(cfCaption('extMySQLRequired'));
	}
	$passwords=parse_ini_file(MYSQL_PASSWORDS_FILE);
	if(isset($passwords[mySQLDatabase($database)])) return $passwords[mySQLDatabase($database)];
	return false;
}

/**
 * @desc Connect and select a database
 *
 * @return boolean
 */
function mySQLConnectDB($database){
	if(!$socket=@fsockopen('127.0.0.1',mySQLPort(),$errno, $errstr, 3)) connectionError('connection failed (3)'); else fclose($socket);
	$host=mySQLHost($database);

	if(!mysql_connect(mySQLHost($database),$user=mySQLUser($database),mySQLPassword($database))) return false;

	if($database=='root') return true;
	if(!mysql_select_db(mySQLDatabase($database))){
		mysql_close();
		return false;
	}
	return true;
}

/**
 * @desc Import a .sql file
 *
 * @param string $file: path to .sql file
 */
function mySQLImportDump2($database,$file) {
	mySQLConnectDB('root');

	if(!mysql_select_db(mySQLDatabase($database))) return false;

	$query = "";
	if(!($handle = @fopen($file, "r"))) return false;
	while(!feof($handle)) {
		$sql_line = fgets($handle);
		$tsl = trim($sql_line);
		if (($sql_line != "") && (substr($tsl, 0, 2) != "--") && (substr($tsl, 0, 1) != "#")) {
			$query .= $sql_line;

			if(preg_match("/;\s*$/", $sql_line)) {
				$query=str_replace("\n",'',str_replace("\r",'',$query));
				if(substr($query,-1)==';') $query = substr($query,0,strlen($query)-1);
				$result=mysql_query($query);
				$query = '';
			}
		}
	}
	fclose($handle);
	return true;
}
?>