<?php
/**
 * Tree View Class
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.9
 */

/**
 * @desc Generic treeView class
 *
 */
class treeView{
	public	$name; 						// Tree name/id
	public	$phpTree=false; 			// Server-side tree
	public	$syncSelectPath=false; 		// Set to true to use synchronous refreshed treeView / page
	public	$draggableNodes=false; 		// Set to true to have draggable nodes
	public	$receiverOnlyNodes=false; 	// Set to true to have receiver-only nodes
	public  $selectFunction='';			// Javascript function called when selecting a node
	public  $toggleFoldFunction=''; 	// Javascript function called when toggling node folding
	public  $dblClickFunction=''; 		// Javascript function called when double clicking a node
	public  $rightClickFunction=''; 	// Javascript function called when right-clicking a node. Can be overridden at node level
	public	$sortBy=false; 				// nodes sort by 'name', 'date' or false (unsorted)
	public	$sortOrder=false; 			// nodes sort order 'asc' or 'desc')
	public	$sortReset=false; 			// true to send a reset command to JS class, used to reconstruct tree after a change in sorting
	public	$sync='auto'; 				// 'sync', 'async', 'auto'
	public	$noATag=false; 				// Set to true to replace <A> nodes by

	protected $localBrowser=false;		// Set to true to browse all server's files/dirs, out of a resource context

	function __construct($name){
		$this->name=$name;
	}

	/**
	 * @desc Debug
	 */
	function __toString(){
		return $this->nodeToString($this->phpTree,0);
	}

	private function nodeToString($tree,$level=0){
		$output='';
		foreach ($tree as $key=>$value){
			$output.=str_pad('',$level*3,' ');
			if(!is_array($value)) $output.='['.$key.'] => '.$value."\n";
			else $output.='['.$key.'] => '."\n".$this->nodeToString($value,$level+1);
		}
		return $output;
	}

	function save(){
		if($this->localBrowser) cfGSetVar('treeView_'.$this->name,$this);
		else cfRSetVar('treeView_'.$this->name,$this);
	}

	/**
	 * @desc Return current directory
	 *
	 * @return string
	 */
	function currentDirectory(){
		if($this->localBrowser) return cfGGetVar('localBrowserDirectory');
		return cfRGetVar('efCurrentDirectory');
	}

	/**
	 * @desc Update server-side folders tree, and generate JS code needed to update client-side tree
	 *
	 * PHP Tree structure:
	 * 'folder basename'=>Array(
	 * 			(optional) 'ico'=>icon's name, or '' for standard folder icon,
	 * 			'sub'=>array of subfolders trees, false if not processed, empty array if no subfolders
	 * 			(optional) 'path'=>resource base path relative full pathname (only if top element)
	 *
	 * JS Tree structure:
	 * {"folder basename":{
	 * 		(optional) "ico": icon's name or "" for standard folder icon,
	 * 		"sub": subfolders tree, false if not processed, {} if no subfolders
	 * 		},
	 * ...
	 * }
	 *
	 * @param string $bottomFolder: selected folder (real path, without *resourceBasePath*)
	 * @param boolean $resetTree: true to reset tree and return tree from base folder
	 * @return string: JS code or false if $folder not allowed
	 */
	function update(){
		return $this->TVArrayToJS($this->phpTree,'');
	}

	/**
	 * @desc Generate JS formated tree from PHP tree
	 *
	 * @param array $tree: PHP formated tree
	 * @return string: JS formated tree
	 */
	public function TVArrayToJS($tree){
		if(!is_array($tree)) return '""';
		$output='{';

		if(isset($tree['id'])) 		$output.='"id":"'.$tree['id'].'",';
		if(isset($tree['label'])) 	$output.='"label":"'.addslashes($tree['label']).'",';
		if(isset($tree['tooltip'])) $output.='"tooltip":"'.addslashes($tree['tooltip']).'",';
		if(isset($tree['ico']))   	$output.='"ico":"'.$tree['ico'].'",';
		if(isset($tree['rightClickFunction']))   	$output.='"rcf":"'.$tree['rightClickFunction'].'",'; // oncontextmenu action. Note: this may also be set at a global
		if(isset($tree['path']))  	$output.='"path":"'.cfUTF8Encode($tree['path'],false,false).'",';
		if(isset($tree['bottom']))  $output.='"bottom":1,';
		if(isset($tree['tag']))  	$output.='"tag":"'.addslashes($tree['tag']).'",';
		if(@($tree['selected']))$output.='"selected":1,';
		if(@($tree['unfolded']))$output.='"unfolded":1,';
		if(@($tree['faded']))  	$output.='"faded":1,';
		if(@($tree['disabled']))$output.='"disabled":1,';
		if(@($tree['noClick']))  	$output.='"noClick":1,';
		if(isset($tree['outerLabel']))$output.='"outerLabel":"'.addslashes($tree['outerLabel']).'",';
		if(isset($tree['action']))  $output.='"action":"'.addslashes($tree['action']).'",';

		$output.='"sub":';
		// No possible subtree
		if(@$tree['sub']==='no') return $output.'0}';

		// Empty tree
		if(!isset($tree['sub']) || !is_array($tree['sub'])) return $output.'false}';

		$output.='{';
		$nb=0;
		// Browse subdirs
		foreach ($tree['sub'] as $dir=>$subTree) {
			if($nb>0) $output.=',';
			$output.='"'.cfUTF8Encode($dir,false,false).'":'.$this->TVArrayToJS($subTree);
			$nb++;
		}
		return $output.'}}';
	}

	public function node($path){return 'dgi("'.$this->name.'_'.cfResourceRelativePath($path).'")';}

	/**
	 * @desc Insert Javascript needed for tree view functions
	 */
	function commonScripts(){
		if(isset($_ENV[APPLICATION_NAME]['treeViewScriptsSet'])) return ''; else $_ENV[APPLICATION_NAME]['treeViewScriptsSet']=true;
		$output='<script type="text/javascript" language="javascript">';
		$output.="var treeIconPlus=\"".outIconNPNG('treePlus')."\";\n";
		$output.="var treeIconPlusOver=\"".outIconNPNG('treePlusOver','')."\";\n";
		$output.="var treeIconMinus=\"".outIconNPNG('treeMinus')."\";\n";
		$output.="var treeIconMinusOver=\"".outIconNPNG('treeMinusOver','')."\";\n";
		$output.="var treeIconLoading=\"".outIconNPNG('treeLoading')."\";\n";
		$output.="var treeIconError=\"".outIconNPNG('treeError')."\";\n";
		$output.="var treeIconFolder=\"".outIconNPNG('fi/folder')."\";\n";
		$output.="var treeIconFolderO=\"".outIconNPNG('fi/folderO')."\";\n";
		$output.="var treeUnfoldOnSelect=".((cfBGetVar('screenSize')=='monitor')?0:1).";\n";
		$output.="var treeOpenTimeout=".(cfHGetVar('treeOpenTimeout',1000)).";\n";
		return $output."</script>\n".cfScriptLink('treeView.js');
	}

	/**
	 * @desc Set whole tree data on synchronous refresh
	 *
	 * @param bool $insertScriptTag: true to embed script into <script></script>
	 */
	function createTreeView($insertScriptTag=true){
		$output = ($insertScriptTag)?'<script type="text/javascript">':'';

		$output .= 'treeViews["'.$this->name.'"]=new treeView("'.$this->name.'",'.$this->update($this->currentDirectory(),true,true).');';
		$output.='with(treeViews["'.$this->name.'"]){';
		if($this->syncSelectPath) $output.='setSelectFunction(treeSyncSelectPath);';
		if($this->selectFunction) $output.='setSelectFunction('.$this->selectFunction.');';
		if($this->toggleFoldFunction) $output.='setToggleFoldFunction ('.$this->toggleFoldFunction.');';
		if($this->dblClickFunction) $output.='setDblClickFunction('.$this->dblClickFunction.');';
		if($this->noATag) $output.='setNoATag();';
		if($this->rightClickFunction) $output.='setRightClickFunction('.$this->rightClickFunction.');';
		if($this->draggableNodes) $output .= 'draggableNodes=1;';
		$output.='sync="'.$this->sync.'";';

		if($this->receiverOnlyNodes) $output .= 'receiverOnlyNodes=1;';
		$output .= 'init();}';
		return $output.(($insertScriptTag)?'</script>':'');
	}

	/**
	 * @desc Return HTML code for tree view
	 *
	 * @param string $class: CSS class of outer <div>
	 * @param mixed $resizable: false->not resizeable, 'h'->horizontaly resizeable, 'v'->verticaly resizeable
	 * @param boolean $closable
	 * @param string $extraStyle
	 * @param string $extraHTML
	 * @param string $header: HTML code of header, leave empty for no header
	 * @return string HTML code
	 */
	function HTML($class=false, $resizable=false, $closable=false, $extraStyle=false, $extraHTML=false, $header=false){
		// Add common treeView JS
		$output=$this->commonScripts();

		// Insert HTML stuff
		$output.='<div id="'.$this->HTMLDivID().'"'.(($class)?' class="'.$class.'"':'').' style="margin:0;overflow:auto;'.$extraStyle.'"'.(($extraHTML)?' '.$extraHTML:'').'>';

		if($header){
			if($closable) $output.=outFrameHeaderTable((($class)?$class.'Header':''),$header,outButtonSmall(false,'javascript:treeToggleFrame()',outIcon('close'),cfCaption('genClose')),(($resizable)?' onmouseover="setMFResizable()" onmouseout="setMFUnresizable()"':''),(($resizable)?'cursor:e-resize':''));
			else $output.='<div '.(($class)?'class="'.$class.'Header" ':' ').(($resizable)?' onmouseover="setMFResizable()" onmouseout="setMFUnresizable()" style="cursor:e-resize"':'').'>'.$header.'</div>';
		}
		$output.='<div id="'.$this->name.'" style="border:0px;padding:0px;height:100%"></div></div>';

		// Add treeView data
		$output.=$this->createTreeView();

		// Add drag item if resizable
		if($resizable) cfDragAddItem($this->HTMLDivID(),'+RESIZABLE+RESIZABLE_ONLY+FROZEN');

		return $output;
	}

	/**
	 * @desc Return HTML code for async update of whole treeview
	 *
	 * @return string
	 */
	function HTMLUpdate(){
		$output='dgi("'.$this->name.'").innerHTML="";';
		$output.=$this->createTreeView(false);
		return $output;
	}

	/**
	 * @desc return treeView outer div id
	 *
	 * @return string id
	 */
	function HTMLDivID(){return $this->name.'_outer';}

	/**
	 * @desc return JS item;
	 *
	 */
	function JSItem(){
		return 'treeViews["'.$this->name.'"]';
	}

	/**
	 * @desc return JS code for unfolding a node
	 *
	 * @param string $completeFilename : bottom folder
	 */
	function updateScript($completeFilename) {
		$jsTree=$this->update($completeFilename,false,true);

		if($this->sortReset)	return 'treeViews["'.$this->name.'"].reset ('.$jsTree.')';
		else 					return 'treeViews["'.$this->name.'"].update('.$jsTree.',false)';
	}

	/**
	 * @desc return JS code for selecting a node
	 *
	 * @param string $completeFilename : bottom folder
	 */
	function updateAndSelectScript($completeFilename) {
		$jsTree=$this->update($completeFilename,false,false);
		if($this->sortReset)	return 'treeViews["'.$this->name.'"].reset ('.$jsTree.')';
		else 					return 'treeViews["'.$this->name.'"].update('.$jsTree.',1)';
	}
}




/**
 * @desc Folder-dedicated treeView class
 *
 */
class treeViewFolder extends treeView{
	private $filesShown=false;

	/**
	 * @desc Return true if directory access is allowed
	 *
	 * @return string
	 */

	/**
	 * @desc Return true if directory access is allowed
	 *
	 * @param string $dir
	 * @param boolean $unsafeUpperDir: set to false if upper dir access is known allowed
	 * @return boolean
	 */
	protected function isAllowedDir($dir,$unsafeUpperDir=true){
		if($this->localBrowser) return true;
		return cfFileRights($dir,'state',$unsafeUpperDir)==='authorized';
	}

	/**
	 * @desc Update server-side folders tree, and generate JS code needed to update client-side tree
	 *
	 * PHP Tree structure:
	 * 'folder basename'=>Array(
	 * 			(optional) 'ico'=>icon's name, or '' for standard folder icon,
	 * 			'sub'=>array of subfolders trees, false if not processed, empty array if no subfolders
	 * 			(optional) 'path'=>resource base path relative full pathname (only if top element)
	 *
	 * JS Tree structure:
	 * {"folder basename":{
	 * 		(optional) "ico": icon's name or "" for standard folder icon,
	 * 		"sub": subfolders tree, false if not processed, {} if no subfolders
	 * 		},
	 * ...
	 * }
	 *
	 * @param string $bottomFolder: selected folder (real path, without *resourceBasePath*,
	 * 			except for shared items type where topmost folder is *resourceBasePath*)
	 * @param boolean $resetTree: true to reset tree and return tree from base folder
	 * @return string: JS code or false if $folder not allowed
	 */
	function update($bottomFolder=false, $resetTree=false, $unfoldBottomNode=true){
		$dbg=false;
		$sortReset=false;

		if(!$resetTree) $phpTree=$this->phpTree; else $phpTree=false;

		// If sorting has changed, reset PHP tree
		if(!$this->localBrowser && ($this->sortBy!=cfRGetVar('efSortBy') || $this->sortOrder!=cfRGetVar('efSortOrder'))) {
			$phpTree=false;
			$this->sortBy=cfRGetVar('efSortBy');
			$this->sortOrder=cfRGetVar('efSortOrder');
			$sortReset=true;
		}

		// Init tree
		if(!$phpTree) {
			$phpTree=array();
			$treeBranch=&$phpTree;
		}

		/**
		 * Init folders
		 */

		// $baseFolder: topmost tree folder (cfRGetVar('path') or '*resourceBasePath*' or computerRoot for local browser)
		if($this->localBrowser) $baseFolder='computerRoot';
		else if(cfSharedMode('list')) $baseFolder='*resourceBasePath*';
		else $baseFolder=cfRGetVar('path');

		// $bottomFolder: lowest, seeked folder
		if(!$bottomFolder) $bottomFolder=$this->currentDirectory();
		if(!$this->localBrowser && cfSharedMode('list')) $bottomFolder=cfResourceRelativePath($bottomFolder); // shared list mode: convert back to relative path

		// $topFolder: folder from which tree will be returned, i.e. lowest tree folder including bottom folder,
		// and already processed. Found by climbing up from bottomFolder
		$topFolder=$bottomFolder;
		if($topFolder!="/" && substr($topFolder,-1)=='/') $topFolder=substr($topFolder,0,strlen($topFolder)-1); // Remove trailing /

		// treeView not initialized: set 'path' = baseFolder, and set topFolder to baseFolder
		// Also reset if sorting has changed
		if(!count($phpTree)) {
			$phpTree=array('path'=>cfResourceRelativePath($baseFolder));
			$topFolder=$baseFolder;
		}
		// Else, seek lowest processed folder matching with bottomFolder pattern
		elseif($topFolder!='computerRoot' && $topFolder!='*resourceBasePath*'){
			$treeBranch=&$phpTree;

			if($baseFolder=='computerRoot'){
				$donePath='';
			}
			else{
				$donePath=$baseFolder;
				if(substr($topFolder,0,strlen($baseFolder)+1)==$baseFolder.'/'){
					$topFolder=substr($topFolder,strlen($donePath)+1);
				}
				else $topFolder=$baseFolder;
			}
			if(cfSharedMode('list')) {
				//cfDbg('top:'.$topFolder.'    bottom:'.$bottomFolder.'    donePath:'.$donePath);
				//$topFolder=cfResourceRelativePath($topFolder);
			}

			$tmpFolder=explode('/',$topFolder);

			for ($i=0;$i<count($tmpFolder);$i++){
				if(!(isset($treeBranch['path']) && $tmpFolder[$i]==$treeBranch['path']) && !isset($treeBranch['sub'][$tmpFolder[$i]])){
					//cfDbg($i.')'.$tmpFolder[$i].' NOK');
					break;
				}
				//cfDbg($i.')'.$tmpFolder[$i]);
				if(isset($treeBranch['sub'][$tmpFolder[$i]])) $treeBranch=&$treeBranch['sub'][$tmpFolder[$i]];
				$donePath.='/'.$tmpFolder[$i];
			}
cfDebugSingle($donePath);
			if($donePath[0]=='/' && substr($donePath,0,2)!='//' && WEnv::host()->isWindows()) $topFolder=substr($donePath,1); else $topFolder=$donePath;
		}

		//else $topFolder='computerRoot';
cfDebugSingle($topFolder.' -> '.$bottomFolder);
		$processedFolder=$topFolder;
		// Bottom folder
		if($bottomFolder!="/" && substr($bottomFolder,-1)=='/' && WEnv::host()->isWindows()) $bottomFolder=substr($bottomFolder,0,strlen($bottomFolder)-1);

		//cfDbg($baseFolder.' - '.$topFolder.' - '.$bottomFolder,1);

		$processedFolder=$topFolder;
		if($bottomFolder=='computerRoot' || $bottomFolder=='/')	$folderArray=array($bottomFolder);
		else{
			if($topFolder=='computerRoot' || $topFolder=='/') $folderArray=array_merge(array($topFolder),explode('/',$bottomFolder));
			else {
				if(strpos($topFolder,'/')===false) {
					$processedFolder='';
					$folderArray=explode('/',$bottomFolder);
				}
				else {
					$processedFolder=substr($topFolder,0,strrpos($topFolder,'/'));
					$folderArray=explode('/',substr($bottomFolder,strlen($processedFolder)+1));
				}
			}
		}

		// Linux: when top folder is /, remove empty nodes as exploded "/path" starts by an empty value
		foreach ($folderArray as $k=>$v) if($v=="") unset($folderArray[$k]);
		$returnTree=array();
		$processedTreeView=&$returnTree;

		// Selected directory
		if($this->localBrowser) $selDir=$this->currentDirectory();
		else if(cfSharedMode('list')) $selDir=cfResourceRelativePath(cfRGetVar('efCurrentDirectory'));
		else $selDir=cfRGetVar('efCurrentDirectory');


		$level=0; $maxLevel=count($folderArray);
		$sortByDate=($this->sortBy=='date');
		foreach ($folderArray as $folderName) {
			$level++;
			$processedFolder=cfJoinPathFile($processedFolder,$folderName);

			// Uppercase drives paths
			if(strlen($folderName)==2 && $folderName[1]==':') $folderName=strtoupper($folderName);
			if(isset($processedTreeView['sub'][$folderName])) $processedTreeView=&$processedTreeView['sub'][$folderName];
			elseif (isset($processedTreeView['sub']) && is_array($processedTreeView['sub'])){
				if($dbg) echo 'Failed on '.$folderName.' -------------<br>';
				if($dbg) cfVarDump($processedTreeView);
				return false;
			}
			if(!$unfoldBottomNode && $level==$maxLevel) $processedTreeView['bottom']='true';

			// Check files rights of processed folder
			if(!$this->isAllowedDir($processedFolder)) break;

			// shared list mode, *resourceBasePath*: list and sort dirs
			if($processedFolder=='*resourceBasePath*'){
				$processedTreeView['ico']='../home.png';
				$processedTreeView['path']='*resourceBasePath*';
				$processedTreeView['label']=cfUTF8Encode(cfRGetVar('name'));
				$dirs=array();

				// Get shared directories
				foreach (cfSharedItems() as $cfn=>$type) if($type=='d' && $this->isAllowedDir($cfn)){
					if($sortByDate)	$dirs[basename($cfn)]=@filemtime($cfn);
					else	$dirs[]=basename($cfn);
				}

				// Sort them
				switch ($this->sortBy){
					case 'name':
						natcasesort($dirs);
						if($this->sortOrder=='desc') $dirs=array_reverse($dirs,true);
						break;
					case 'date':
						if($this->sortOrder=='desc') arsort($dirs);
						else asort($dirs);
						foreach ($dirs as $n=>$time) $dirs[$n]=$n;
				}

				foreach ($dirs as $dir)	{
					$processedTreeView['sub'][$dir]=array('sub'=>false);
					if(strpos($dir,':')==1) {
						$d=efDriveInfo($dir);
						$processedTreeView['sub'][$dir]['ico']=basename($d['icon']);
					}
				}
			}
			// Computer root : browse drives
			elseif($processedFolder=='computerRoot'){
				$processedTreeView['ico']='computerRoot.png';
				$processedTreeView['path']='computerRoot';
				$processedTreeView['label']=cfCaption('explorerComputerRoot');
				// Browse drives, from a to z
				for ($i=66;$i<91;$i++) {
					$driveName=chr($i).":/";
					if (is_dir($driveName) && $this->isAllowedDir($driveName)){
						$d=efDriveInfo($driveName);
						$processedTreeView['sub'][($driveName[0]).':']=array('sub'=>false,'ico'=>basename($d['icon']));
						$processedTreeView['sub'][($driveName[0]).':']['label']='('.$driveName.') '.cfUTF8Encode($d['volumeName']);
					}
				}
			}
			// Drive / Folder : browse subfolders
			else{
				if(!$this->localBrowser && $level==1 && strtolower($processedFolder)==strtolower(cfRGetVar('path'))) $processedTreeView['label']=cfUTF8Encode(cfRGetVar('name'));
				// Drive: set specific icon
				if(isset($folderName[1]) && $folderName[1]==':' && $level==0 && strlen($folderName)==2){ // Set extra controls on drive name as linux filenames may include ":"
					$d=efDriveInfo($processedFolder);
					$processedTreeView['ico']=basename($d['icon']);
					$processedTreeView['label']='('.$processedFolder.'/) '.cfUTF8Encode($d['volumeName']);
				}
				// List subfolders
				$processedTreeView['sub']=array();
				$dirs=array();
				$files=array();

				$realProcessedFolder=((cfSharedMode('list'))?cfSharedCompleteFilename($processedFolder):$processedFolder);

				if($handle = @opendir($realProcessedFolder)){
					while (($file = @readdir($handle))!==false) {
						$cfn=$realProcessedFolder.'/'.$file;
						if($file!='.' && $file!='..' && $this->isAllowedDir($cfn)) {
							// Add dir
							if(is_dir($cfn)){
								if($sortByDate)	$dirs[$file]=filemtime($cfn);
								else	$dirs[]=$file;
							}
							// Add file
							elseif($this->filesShown){
								if($sortByDate)	$files[$file]=filemtime($cfn);
								else	$files[]=$file;
							}
						}
					}
					closedir($handle);
				}
				// Sort dirs
				switch ($this->sortBy){
					case 'name':
						natcasesort($dirs);
						if($this->sortOrder=='desc') $dirs=array_reverse($dirs,true);
						break;
					case 'date':
						if($this->sortOrder=='desc') arsort($dirs);
						else asort($dirs);
						foreach ($dirs as $n=>$time) $dirs[$n]=$n;
				}

				// Sort files
				if(count($files)){
					switch ($this->sortBy){
						case 'name':
							natcasesort($files);
							if($this->sortOrder=='desc') $files=array_reverse($files,true);
							break;
						case 'date':
							if($this->sortOrder=='desc') arsort($files);
							else asort($files);
							foreach ($files as $n=>$time) $files[$n]=$n;
					}
				}

				// Add dirs & files to tree structure
				foreach ($dirs as $dir)	$processedTreeView['sub'][basename($dir)]=array('sub'=>false);
				foreach ($files as $file)	$processedTreeView['sub'][basename($file)]=array('ico'=>efIcon($file),'sub'=>'no');
			}

			// Set as selected if current directory
			if($processedFolder==$selDir) {
				$processedTreeView['selected']='true';
				$processedTreeView['unfolded']='true';
			}
		}

		// Set 'path' for topFolder
		$treeBranch=$returnTree;

		// Update server-side treeview
		$this->phpTree=$phpTree;
		$this->save();

		$returnTree['path']=cfResourceRelativePath($topFolder,false,(($this->localBrowser)?'computerRoot':false));

		// Inform
		if($sortReset) $this->sortReset=true; else $this->sortReset=false;

		return $this->TVArrayToJS($returnTree,'');
	}


	/**
	 * @desc Return HTML code for tree view
	 *
	 * @param string $class
	 * @param mixed $resizable: false->not resizeable, 'h'->horizontaly resizeable, 'v'->verticaly resizeable
	 * @param boolean $closable
	 * @param string $extraStyle
	 * @param string $extraHTML
	 * @return string HTML code
	 */
	function HTML($class=false, $resizable=false, $closable=false, $extraStyle=false, $extraHTML=false, $header=false){
		// Add common treeView JS
		$output=$this->commonScripts();

		// Insert HTML stuff
		$output.='<div id="'.$this->HTMLDivID().'" class="treeViewDiv'.(($class)?' '.$class:'').'" style="margin:0;overflow:hidden;'.$extraStyle.'"'.(($extraHTML)?' '.$extraHTML:'').'>';
		if(!cfTGetVar('avoidTitleBar')){
			if($closable) $output.=outFrameHeaderTable((($class)?$class.'Header':''),outImage(outIcon('browse.png'),false,false,'vertical-align:middle;margin-right:0.5em').cfCaption('genDirectories'),outButtonSmall(false,'javascript:treeToggleFrame()',outIcon('close'),cfCaption('genClose')),(($resizable)?' onmouseover="setMFResizable()" onmouseout="setMFUnresizable()"':''),(($resizable)?'cursor:e-resize':''));
			else $output.='<div '.(($class)?'class="'.$class.'Header" ':' ').(($resizable)?' onmouseover="setMFResizable()" onmouseout="setMFUnresizable()" style="cursor:e-resize"':'').'>'.outImage(outIcon('browse.png'),false,false,'vertical-align:middle;margin-right:0.5em').cfCaption('genDirectories').'</div>';

		}
		// Wrapper div over treeview
		if(cfGetBrowser()=='gecko') $output.='<div id="'.$this->name.'" style="position:absolute;bottom:0;top:2.5em;width:95%;overflow:auto;border:0px;padding:0;'.((cfGetBrowser()=='ie')?';width:95%':'').'"></div></div>'; // Note: width must be set under IE for overflow to work
		else $output.='<div id="'.$this->name.'" style="overflow:auto;border:0px;padding:0;height:100%'.((cfGetBrowser()=='ie')?';width:95%':'').'"></div></div>'; // Note: width must be set under IE for overflow to work

		// Add dblClick function
		//$this->dblClickFunction='treeFolderDblClickSelect';

		// Add treeView data
		$output.=$this->createTreeView();
		// Add drag item if resizable
		if($resizable) cfDragAddItem($this->HTMLDivID(),'+RESIZABLE+RESIZABLE_ONLY+FROZEN');

		return $output;
	}

	/**
	 * @desc Show files and folders
	 *
	 * @param string $show
	 */
	function showFiles($show=true){
		$this->filesShown=$show;
	}
	/**
	 * @desc Set browser as local browser
	 *
	 * @param string $show
	 */
	function setLocalBrowser($local=true){
		$this->localBrowser=$local;
	}
/**
	 * @desc Set sort data
	 *
	 */
	function setSortBy($sortBy){
		if($sortBy=='date' || $sortBy=='name') $this->sortBy=$sortBy;
	}
/**
	 * @desc Set sort order (desc/asc)
	 *
	 */
	function setSortOrder($sortOrder){
		if($sortOrder=='desc' || $sortOrder=='asc') $this->sortOrder=$sortOrder;
	}
/**
	 * @desc Set current directory (local browser only)
	 * @param string $directory
	 */
	function setCurrentDirectory($directory){
		if(!$this->localBrowser) return ;
		cfGSetVar('localBrowserDirectory',$directory);
	}

	/**
	 * @desc Process POST commands
	 *
	 */
	function processCommands(){
		if(!cfIsAsync()) return ;

		require_once(INCLUDE_DIR.'outputFunctions.php');

		cfAsyncHeader();

		list($completeFilename,$passedDir,$passedFile)=efGetPassedFileComponents();

		// Unfold folder node
		if($_POST['data3']=='treeUnfoldTo'){
			if(is_dir($completeFilename) && ($this->localBrowser || cfFileRights($completeFilename,'read'))){
				echo cfAsyncXMLJSaction($this->updateScript($completeFilename));
			}
			else{
				echo cfAsyncXMLJSaction('treeTmp = new treeNode("'.@$_POST['data5'].'");treeTmp.setError()');
			}

		}
		echo cfAsyncXMLJSaction('treeProcessing=0;');
		die(cfAsyncFooter());
	}
}




/**
 * @desc load tree from session data or create a new one if inexistant
 *
 * @param string $name tree name
 * @param boolean $forLocalBrowser: set to true to retrieve tree from GVar instead of RVar
 * @return treeView object
 */
function treeFolderLoad($name,$forLocalBrowser=false){
	if($forLocalBrowser && cfGGetVar('treeView_'.$name)) return cfGGetVar('treeView_'.$name);
	elseif(!$forLocalBrowser && cfRGetVar('treeView_'.$name)) return cfRGetVar('treeView_'.$name);
	else return new treeViewFolder($name);
}
?>