/**
 * HTML5 audio player scripts
 */


var html5AP={
	'files':{},
	'track':-1,
	'state':'stopped', // Player state : playing, stopped, paused
	'controls':['play','pause','back','next'], // list of ids (prepend "html5AP_") of controls
	'player':0, // Audio player object

	'setSrc':function(file){
		var e,p=$('playerAudioHtml5');
		// Set audio player source URL
		if(p.canPlayType('audio/mpeg; codecs=mp3')) p.src=file.url+'.convert.mp3';
		else p.src=file.url+'.convert.ogg';
	},

	/**
	 * Show or hide html5 audio player
	 * @param show: true to show, false to hide
	 */
	'show':function(show){
		try{dgi('playerAudio').style.display=(show)?'none':'block'} catch(e){} // If HTML5 player must be used, hide flash player (if present)
		dgi('playerAudioHtml5Div').style.display=(show)?'block':'none';
		if(!show) this.pause();
		if(show && !this.initialized) this.init(undefined,1,-1)
	},

	/**
	 * Initialise HTML5 player with list of songs to play
	 * If flash player is detected, abort, as flash is prefered
	 * @param array files: array of {url, artist, album, track} infos
	 * @param bool force: true to activate HTML5 even if flash is supported
	 * @param int volume: if !=-1, set default volume (0-100)
	 */
	'init':function(files, force, volume){
		if(files!=undefined) this.files=files;	// Store files now into object so if it's initializeed later, files are not lost
		if(volume!=-1) this.volSet(volume);		// Init volume
		if(wl.hasFlash() && !force) return; // Flash is supported: abort

		try{dgi('playerAudio').style.display='none'} catch(e){} // If HTML5 player must be used, hide flash player (if present)
		$('playerAudioHtml5Div').style.display='block';


		this.player=dgi('playerAudioHtml5');
		this.setState('stopped'); this.track=-1;

		this.player.onended=html5AP.trackEnded;
		//this.player.onloadeddata=html5AP.loadProgress;
		//this.player.ontimeupdate=html5AP.progress;
		setInterval(html5AP.loadProgress,300);
		setInterval(html5AP.progress,500);
		this.player.pause();
		try{this.player.currentTime=0;}catch(e){} // try catch as FF fails if no audio loaded

		dgi('html5AP_npos').style.display=dgi('html5AP_ntot').style.display='none';

		this.initialized=1;

		if(count(files)>0) this.play(0);
	},

	'enableBt':function(btId,on){
		if(!on) {$('html5AP_'+btId).e.setAlpha(0.2); $('html5AP_'+btId).e.className().add('html5APDisabled')}
		else {$('html5AP_'+btId).e.setAlpha(1); $('html5AP_'+btId).e.className().remove('html5APDisabled')}
	},

	'setState':function(state){
		var i=0,c=this.controls,n=count(this.files);

		if(n==0) for(;i<c.duration;i++) {this.enableBt(c[i],0)}
		else{
			this.enableBt('back',1);
			this.enableBt('next',this.track+1<n);
			this.enableBt('play',1);
			this.enableBt('pause',1);
		}
		dgi('html5AP_play').style.display=(state=='playing')?'none':'';
		dgi('html5AP_pause').style.display=(state=='playing')?'':'none';
		dgi('html5AP_npos').style.display=(state=='stopped')?'none':'';
		dgi('html5AP_ntot').style.display=(state=='stopped')?'none':'';
		$('html5AP_pbo').e.setAlpha((n==0)?0.4:1)
		$('html5AP_ppos').e.setAlpha(0.6)
		$('html5AP_pload').e.setAlpha(0.2)
		this.state=state;
//alert('state: '+state+' nb files:'+n+' current track:'+this.track)
	},

	/**
	 * Start playing track
	 * @param trackNb: track number or undefined to resume play
	 */
	'play':function(trackNb){
		if(trackNb==undefined && this.track==-1) trackNb=0;
		if(trackNb!=undefined && this.track!=trackNb){
			this.track=trackNb;
			this.setSrc(this.files[trackNb]);
			this.player.pause();

			if(W.inlineUpdatePlayerInfo) inlineUpdatePlayerInfo(this.files[this.track].externalRef)
		}

		dgi('html5AP_ntot').innerHTML=this.fTime(this.files[this.track].duration);

		this.setState('playing');
		this.player.play();
	},

	/**
	 * Pause player
	 */
	'pause':function(){
		if(this.state!='playing') return;
		this.player.pause();
		this.setState('paused');
	},

	/**
	 * Stop player
	 */
	'stop':function(){
		this.player.pause();
		this.setState('stopped');
		this.player.currentTime=0;
	},

	/**
	 * Rewind file or jump to previous file
	 */
	'back':function(){
		var cs=this.state;
		if(this.track==0) this.player.currentTime=0;
		else if(cs=='playing') {
			if(this.player.currentTime>5) this.player.currentTime=0;
			else this.play(this.track-1);
		}
		else{
			this.play(this.track-1);
			if(cs=='paused') this.pause(); else this.stop();
		}
	},

	/**
	 * Jump to next file
	 */
	'next':function(){
		var cs=this.state;
		if(this.track+1>=count(this.files)) return;
		this.play(this.track+1);
		if(cs=='paused') this.pause();
		else if(cs=='stopped') this.stop();
	},

	/**
	 * Track finished playing
	 */
	'trackEnded':function(){html5AP.next()},

	/**
	 * Click on progress bar
	 */
	'pbClick':function(e){
		if(!count(html5AP.files)) return;
		if(!e) e=W.event;
		var p,l;
		if(e.offsetX) l=e.offsetX; else l=((e.x)?e.x:e.pageX)-$('html5AP_pbo').e.actualOffsetLeft();
		l++;
		if(l<0) l=0;
		var p=l/($('html5AP_pbo').offsetWidth-2);
		html5AP.player.currentTime=html5AP.files[html5AP.track].duration*p;
	},

	/**
	 * Load progress indicator
	 */
	'loadProgress':function(e){
		// Workaround for mobile chrome that can't start from init function...=>set player as paused to let user start
		if(html5AP.player.paused && html5AP.state=='playing') html5AP.pause();
/*
		var p,pl=html5AP.player;
		if(pl.buffered.duration<=0) return;
		var p=100*pl.buffered.end(0)/html5AP.files[html5AP.track].duration;
		dgi('html5AP_pload').style.width=p+'%';
*/
	},

	/**
	 * Format text time output
	 */
	'fTime':function(t){
		t=Math.floor(t);
		var h=Math.floor(t/3600),m=Math.floor((t-h*3600)/60),s=t%60;
		if(m<10 && h>0) m='0'+m;
		if(s<10) s='0'+s;
		return ((h)?h+':':'')+m+':'+s;
	},

	/**
	 * Jump to next file
	 */
	'progress':function(){
		if(!count(html5AP.files)) return;

		dgi('html5AP_npos').innerHTML=html5AP.fTime(html5AP.player.currentTime);
		if(html5AP.files[html5AP.track].duration) {
			var perc=100*Math.min(1,html5AP.player.currentTime/html5AP.files[html5AP.track].duration);
			dgi('html5AP_ppos').style.width=perc+'%';
		}
	},

	/**
	 * Show / Hide volume control, called by onmouseover & onmouseout
	 */
	'volShow':function(show){
		if(html5AP.volShowTimer) clearTimeout(html5AP.volShowTimer);
		html5AP.volShowTimer=0;
		// Called by timeout: hide
		if(show==-1) {
			dgi('html5AP_vol').style.height='80%';
			dgi('html5AP_vol_po').style.display='none';
		}
		// Mouse in and not already shown: show
		else if(show && !html5AP.volShown) {
			dgi('html5AP_vol').style.height='400%';
			dgi('html5AP_vol_po').style.display='block';
			html5AP.volShown=1;
		}
		// Mouseout: set timer to show
		else if(!show && html5AP.volShown) {
			html5AP.volShowTimer=setTimeout("html5AP.volShow(-1)",800);
			html5AP.volShown=0;
		}
	},

	/**
	 * Click on volume control (#html5AP_vol_po)
	 */
	'volClick':function(e){
		var p,l;
		if(!e) e=W.event;
		l=1+mouseY-$('html5AP_vol_po').e.actualOffsetTop();;
		var p=l/($('html5AP_vol_po').offsetHeight-2);
		p=100*Math.max(0,Math.min(1,1-p));
		html5AP.volSet(p)
		stopPropagation(e);
	},

	/**
	 * Click on volume icon: toggle sound on/off
	 */
	'volToggle':function(){
		if(this.vol) this.volPrev=this.vol;
		if(!this.volPrev) this.volPrev=70;
		this.volSet((this.vol)?0:this.volPrev)
	},

	/**
	 * Set volume
	 */
	'volSet':function(v){
		if(!v && !$('html5AP_vol').e.className().has('no')) $('html5AP_vol').e.className().add('no'); else $('html5AP_vol').e.className().remove('no');
		try{html5AP.player.volume=v/100} catch(e){}
		dgi('html5AP_vol_progress').style.height=Math.floor(v)+'%';
		if(html5AP.vol!=undefined && html5AP.vol!=v) wl.asr.send({},'/flvVolume/'+v);
		html5AP.vol=v;
	}
}
