<?php
/**
 * Accounts and resources configuration
 *
 * Configuration of accounts (activate/deactivate, add/remove resource, create resource, configure resource
 * this script is included by /res/administration/std/resourceConfig.php
 * this script should never be directly called.
 *
 * resourceConfig.php sets 3 variables :
 * $resourceData (array) which contains resource parameters
 * $describerData (array) which contains resource's describer file (describer.ini) parameters.
 * 			   This array is only present if describer.ini is found
 * $_ENV['configurationEnvironment'] which indicates if resourceConfig.php has been called from browser ('browser')
 * or from application's embeded browser ('application')
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
*
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

require_once(INCLUDE_DIR.'resourceConfigFunctions.php');

/**
 * @return string array :
 * @desc return an array containing for a given resource type (identified by type / subType) :
 * "type" : type of resource
 * "subtype" : subtype of resource
 * "baseFile" : index file name
 * "configFile" : configuration file name (optional)
 * "iconFile" : icon file name
 * "name" : explicit name of resource (in current language if found)
*/

function getResourceTypeData($type, $subType){
	$resTypeData=array(); // Array used for return result
	$indexFileName=false;
	if(file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/describer.ini')){
		$describer=cfParse_ini_file(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/describer.ini');
		if(isset($describer['baseFile']) && file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/'.$describer['baseFile'])) $indexFileName=$describer['baseFile'];
	}
	if ((!$indexFileName) && file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/index.php')) $indexFileName='index.php';
	if($indexFileName!==false){
		$resTypeData['type']=$type;
		$resTypeData['subType']=$subType;
		$resTypeData['baseFile']=$indexFileName;
		// icon file
		$resTypeData['iconFile']=outResourceIcon($type, $subType);

		// config file
		if(isset($describer['configFile']) && file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/'.$describer['configFile'])) $resTypeData['configFile']=$describer['configFile'];
		elseif (file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/config.php')) $resTypeData['configFile']='config.php';
		// resource name
		if(isset($describer[cfGGetVar('language')]['name'])) $resTypeData['name']=$describer[cfGGetVar('language')]['name'];
		elseif (isset($describer['en']['name'])) $resTypeData['name']=$describer['en']['name'];
		else $resTypeData['name']=$subType;
	}
	else return false;
	if(isset($describer)) unset($describer);
	return $resTypeData;
}

/**
 * @return string array :
 * @desc return an array of all resources types, containing :
 * "type" : type of resource
 * "subtype" : subtype of resource
 * "baseFile" : index file name
 * "configFile" : configuration file name (optional)
 * "iconFile" : icon file name
 * "name" : explicit name of resource (in current language if found)
*/
function getResourceTypes($excludeAdministration=false){
	if(!file_exists(cfAppDocRoot().'/res')) return false;
	if(!($handle = opendir(cfAppDocRoot().'/res'))) return false;
	$resType=array(); $i=0;
	while (($folder = @readdir($handle))!==false) if($folder != '.' && $folder != '..' && is_dir(cfAppDocRoot().'/res/'.$folder) && !($excludeAdministration && strtolower($folder)=='administration')){
		if($handle2 = opendir(cfAppDocRoot().'/res/'.$folder)){
			while (($folder2 = @readdir($handle2))!==false){
				if($folder2 != '.' && $folder2!= '..'){
					$resType[$i]=getResourceTypeData($folder,$folder2);
					if($resType[$i]!==false) $i++;
				}
			}
		}
	}
	closedir($handle);
	return $resType;
}

/*
 * Verify access rights
*/
if($_SERVER['REMOTE_ADDR']!='127.0.0.1'){
	if(cfUGetVar('administrator')!=true ||  !cfGGetVar('remoteAdministrationAuthorized')) outDisplayErrorPage(cfcaption('errorRemoteAdministrationNotAuthorized'));
}


//#############################################################################################################
//
// Process GET / POST commands
//
//#############################################################################################################

// Configure a resource (note : resource configuration should be processed before other GET commands
// in order custom passed GET / POST parameters doesn't interfere with accounts.php GET parameters (userNb, resName, userDeactivate...)
if(cfRGetVar('configuringResource') || (isset($_GET['config']) && file_exists(cfAppDataDir().'/'.$_GET['config'])) || (isset($_POST['config']) && file_exists(cfAppDataDir().'/'.$_POST['config']))){
	if(isset($_POST['config'])) cfRSetVar('configuringResource',$_POST['config']);
	if(isset($_GET['config'])) cfRSetVar('configuringResource',$_GET['config']);
	require_once('resourceConfig.php');
	echo '</body></html>';
	exit();
}

// Add newly created resource to user
if(isset($_GET['userNb']) && isset($_GET['resName']) && isset($_GET['resTypeNb']) && cfRGetVar('advancedAdministration')){
	$resTypesList=getResourceTypes(true);
	$file=cfGGetVar('user'.$_GET['userNb'],'file');
	if($tmpUser=rcParseUserFile($file)){// Parse user file
	// Modify user file
		// Count resources
		$i=0; while(isset($tmpUser['resource'.$i]) && is_array($tmpUser['resource'.$i])) $i++;
		// Add created resource to temp user array
		$tmpUser['resource'.$i]=array();
		$tmpUser['resource'.$i]['type']=$resTypesList[$_GET['resTypeNb']]['type'];
		$tmpUser['resource'.$i]['subType']=$resTypesList[$_GET['resTypeNb']]['subType'];
		// Generate resource filename
		$u=0; while(file_exists(cfAppDataDir().'/res/'.$tmpUser['resource'.$i]['type'].$tmpUser['resource'.$i]['subType'].$u.'.res')) $u++;
		$tmpUser['resource'.$i]['file']=$tmpUser['resource'.$i]['type'].$tmpUser['resource'.$i]['subType'].$u.'.res';
		// Save user file
		rcWriteUserFile($tmpUser,$file);

	// Generate resource file
		$tmpResource=array();
		$tmpResource['name']=cfUTF8Decode($_GET['resName'],true,true,false,50);
		$tmpResource['type']=$tmpUser['resource'.$i]['type'];
		$tmpResource['subType']=$tmpUser['resource'.$i]['subType'];
		// Get default parameters if present
		if(file_exists(cfAppDocRoot().'/res/'.$tmpUser['resource'.$i]['type'].'/'.$tmpUser['resource'.$i]['subType'].'/defaultConfig.php')){
			$defaultConfig=cfParse_ini_file(cfAppDocRoot().'/res/'.$tmpUser['resource'.$i]['type'].'/'.$tmpUser['resource'.$i]['subType'].'/defaultConfig.php');
			foreach ($defaultConfig as $key=>$value) if($key!='name' && $key!='type' && $key!='subType') $tmpResource[$key]=$value;
			unset($defaultConfig);
		}
		// Write resource file
		rcWriteResourceFile($tmpResource,$tmpUser['resource'.$i]['file']);

		// Save user file (note : must be saved after resource file in as Weezo need to have valid resource files when (re)loading user
		rcWriteUserFile($tmpUser,$file);

		cfLog('administration : new resource added to '.$tmpUser['resource'.$i]['file'].' by '.$_SERVER['REMOTE_ADDR'], LOG_DBG);
		unset($tmpUser);
		unset($tmpResource);
	}
}

// Activate / Deactivate user
if(isset($_GET['userDeactivate']) || (isset($_GET['userActivate']) && cfRGetVar('advancedAdministration'))){
	if(isset($_GET['userDeactivate'])) $userNumber=$_GET['userDeactivate']; else $userNumber=$_GET['userActivate'];
	$file=cfGGetVar('user'.$userNumber,'file');
	if($tmpUser=rcParseUserFile($file)){// Parse user file
		if(isset($_GET['userDeactivate'])) $tmpUser['activated']=(bool) false; else $tmpUser['activated']=(bool) true;
		rcWriteUserFile($tmpUser, $file);
		unset($tmpUser);
		cfLog('administration : '.$file.' user activation toggled by '.$_SERVER['REMOTE_ADDR'], LOG_DBG);
	}
}

// Remove resource
if(isset($_GET['resourceRemove']) && isset($_GET['user'])){
	$file=cfGGetVar('user'.$_GET['user'],'file');
	if($tmpUser=rcParseUserFile($file)){// Parse user file
		// remove requested resource from user file
		if(isset($tmpUser['resource'.$_GET['resourceRemove']]) && is_array($tmpUser['resource'.$_GET['resourceRemove']])){
			unset($tmpUser['resource'.$_GET['resourceRemove']]);
			$i=$_GET['resourceRemove']+1;
			while(isset($tmpUser['resource'.$i]) && is_array($tmpUser['resource'.$i])){
				$tmpUser['resource'.($i-1)]=$tmpUser['resource'.$i];
				unset($tmpUser['resource'.$i]);
				$i++;
			}
		}
		rcWriteUserFile($tmpUser, $file);
		unset($tmpUser);
		cfLog('administration : resource removed from '.$file.' user by '.$_SERVER['REMOTE_ADDR'], LOG_DBG);
		// Note : resource file is not deleted, as it still can be used by other users (now or later)
	}
}

// Add a resource to an user
if(isset($_GET['resourceNew'])) $_GET['resourceAdd']=$_GET['resourceNew'];
if(isset($_GET['resourceAdd']) && cfRGetVar('advancedAdministration')){ // $_GET['resourceAdd'] is number (sequential folder read) of user
	if($handle = opendir(cfAppDataDir())){
		$existingResources=array();
		$i=0;
		while (($file = @readdir($handle))!==false) if(cfFileExtension($file)=='res'){
			if($tmpResource=cfParse_ini_file(cfAppDataDir().'/'.$file,true)){// Parse resource file
				if(isset($tmpResource['type']) && isset($tmpResource['subType']) && isset($tmpResource['name'])){
					$existingResources[$i]=array();
					// Get resource name, file name, type, subtype and resource icon
					$existingResources[$i]['name']=$tmpResource['name'];
					$existingResources[$i]['file']=$file;
					$existingResources[$i]['type']=$tmpResource['type'];
					$existingResources[$i]['subType']=$tmpResource['subType'];
					$existingResources[$i]['icon']=outResourceIcon($tmpResource['type'],$tmpResource['subType']);
					unset($tmpResource);
					$i++;
				}
			}
		}
	}
	// if resource has been selected by user
	if(isset($_GET['resourceNb'])){ // $_GET['resourceNb'] is  number (sequential folder read) of resource to add
		if($_GET['resourceNb']>=0 && $_GET['resourceNb']<$i){
			$file=cfGGetVar('user'.$_GET['resourceAdd'],'file');
			if($tmpUser=rcParseUserFile($file)){// Parse user file
				// Count resources
				$i=0;
				while(isset($tmpUser['resource'.$i]) && is_array($tmpUser['resource'.$i])) $i++;
				// Add resource
				$tmpUser['resource'.$i]=array();
				$tmpUser['resource'.$i]['type']=$existingResources[$_GET['resourceNb']]['type'];
				$tmpUser['resource'.$i]['subType']=$existingResources[$_GET['resourceNb']]['subType'];
				$tmpUser['resource'.$i]['file']=$existingResources[$_GET['resourceNb']]['file'];
				// Save user file
				rcWriteUserFile($tmpUser, $file);
			}
		}
		unset($_GET['resourceAdd']); unset($_GET['resourceNb']); //unset GET variable so resource list is no longer displayed
	}
}

/*
 * End of GET and POSTcommands process
*/



/*
 * Parse user files
 */
$userList=array(); // Array listing user's resources
$tmpUser=array(); // used to store analyzed user data
$nb=0;
// Read user and resource data from config files
while(cfGGetVar('user'.$nb,'file')) { // Browse user files
	// Read user file
	$file=cfAppDataDir().'/'.cfGGetVar('user'.$nb,'file');
	if(file_exists($file) && $tmpUser=cfParse_ini_file($file, true)){
		$userList[$nb]['completeFileName']=$file;
		if(isset($tmpUser['name'])) $userList[$nb]['name']=$tmpUser['name']; else $userList[$nb]['name']='???'; //Get name
		if(isset($tmpUser['activated'])) $userList[$nb]['activated']=$tmpUser['activated']; else $userList[$nb]['activated']=false;//Get activation
		if(isset($tmpUser['icon'])) $userList[$nb]['icon']=$tmpUser['icon']; else $userList[$nb]['icon']='default.jpg'; //Get user icon
		$nbResources=0;
		$nbResourcesOK=0;
		// Browse/add user's resources
		while(isset($tmpUser['resource'.$nbResources]['file'])){
			if(($tmpResource=cfParse_ini_file(cfAppDataDir().'/'.$tmpUser['resource'.$nbResources]['file'])) &&  isset($tmpResource['name']) && isset($tmpResource['type']) && isset($tmpResource['subType']) && file_exists(cfAppDocRoot().'/res/'.$tmpResource['type'].'/'.$tmpResource['subType'])){
				$userList[$nb][$nbResourcesOK]['name']=$tmpResource['name']; // resource name
				$userList[$nb][$nbResourcesOK]['file']=$tmpUser['resource'.$nbResources]['file']; // resource filename

				$tmpResourceData = getResourceTypeData($tmpResource['type'],$tmpResource['subType']);
				// config file
				if(isset($tmpResourceData['configFile']) && ($tmpResource['type']!='administration' || $tmpResource['subType']!='std')) $userList[$nb][$nbResourcesOK]['config'] = true;
				// resource icon
				if(isset($tmpResourceData['iconFile'])) $userList[$nb][$nbResourcesOK]['icon'] =$tmpResourceData['iconFile'];

				$nbResourcesOK++;
			}
			$nbResources++;
			unset($tmpResource);
		}
		$userList[$nb]['nbResources']=$nbResourcesOK;
		unset($tmpUser);
	}
	$nb++;
}




/*
 ***************************************************************************************************************************
 * Display accounts, account's resources, and (if needed, resource add menus)
 ***************************************************************************************************************************
 */
echo outDivFrame('frame1');
echo '<div class="frame1header"><img src="'.outIcon('admin').'" alt="" style="vertical-align:middle;margin-right:1em;">'.cfCaption('adminAccountsRes').'</div>';



//For each user
$uniqueId=0;
for ($i=0;$i<$nb;$i++){
	echo outDivFrame('frame2');
	echo '<div class="frame2header" style="vertical-align:middle;">';
	// User icon & name
	echo '<img alt="" src="/gfx/icons/'.$userList[$i]['icon'].'" width="32" height="32" style="vertical-align:middle; margin-right:1em;border: 1px solid #A0A0A0">'.cfUTF8Encode($userList[$i]['name']);
	echo "</div>\n";

	// if a resource add is requested for this user, display availables resources
	if(isset($_GET['resourceAdd']) && $_GET['resourceAdd']==$i && cfRGetVar('advancedAdministration')){
		echo outDivFrame('frame3');
		echo outFrameHeaderTable("frame3Header", cfCaption('adminResourceAdd'),((!isset($_GET['resourceNew']))?outButton(cfCaption('resNewResource'),'?resourceNew='.$i,outIcon('new'),false,false,false,true):''));
		// if a resource creation is requested
		if(isset($_GET['resourceNew'])) {
			// Display resource type selection form
			echo outDivFrame('frame2');
			// Title
			echo '<div class="frame2Header">'.cfCaption('resNewResource').'</div>';
			echo '<form name="resCreateForm" action="'.$_SERVER['PHP_SELF'].'" method="GET" enctype="multipart/form-data" style="margin:0px;">';
			echo '<input type="hidden" name="resId" value="'.$_SESSION['activeResourceId'].'">';
			echo '<span style="margin-right:1em;">'.cfCaption('resName').'</span><input class="textInput" type="text" name="resName" size="20" maxlength="25"/>';

			// Create new resource button
			echo outButton(cfCaption('adminResourceCreate'),'javascript:if(document.forms[\'resCreateForm\'].elements[\'resName\'].value!=\'\') document.resCreateForm.submit(); else alert(\''.cfCaptionJS('adminResourcePlzFill').'\');',outIcon('new'),cfCaption('adminResourceCreate'));

			echo '<input type="hidden" name="userNb" value="'.$i.'"/><br/>';

			$resTypesList=getResourceTypes(true); // Get an array containing each available resource type, excluding administration resource(s)
			$didCheck=false;
			// radio button list
			$o=0;
			foreach ($resTypesList as $resValue) {
				if($resValue){
					echo '<input type="radio" name="resTypeNb" value="'.$o.'"';
					if(!$didCheck) {echo " checked"; $didCheck=true;}
					echo '/><img alt="" src="'.$resValue['iconFile'].'" width="16" height="16"><span class="eTC" style="margin-right:2em;">'.cfCaption('res'.ucfirst($resValue['type'])).' / '.($resValue['name']).'</span><br>';
					$o++;
				}
			}
			unset($resTypesList);
			echo '</form>';
			echo '<div style="text-align:right;">';
			echo '</div><div class="frame2Footer"></div></div>';
		}
		// Display not-linked resources
		echo outTableTransparent('frame3')."\n";
		foreach ($existingResources as $key=>$value) {
			// detect if resource is already linked to user
			$resourceFileName=$value['file'];
			$resourceAlreadyLinked=false;
			foreach ($userList[$i] as $value2) {if(isset($value2['file']) && $value2['file']==$resourceFileName) $resourceAlreadyLinked=true;}
			if(!$resourceAlreadyLinked){
				echo '<tr>';
				echo '<td width="80%"><img alt="" src="'.$value['icon'].'" width="32" height="32" style="vertical-align:middle; margin-right:1em;margin-left:5em:">'.cfUTF8Encode($value['name']).'</td>';
				echo '<td width="20%" style="text-align:right;">'.outButton(cfCaption('genAdd'),'?resourceAdd='.$i.'&amp;resourceNb='.$key,outIcon('add')).'</td></tr>'."\n";
			}
		}
		echo "</table>\n";
		echo '<br/></div><br/>'."\n";
	}

	//Display user's resources
	if($userList[$i]['nbResources']>0){ // if at least one resource is bound to user
		echo outTableTransparent('frame2');

		// For each resource
		for($o=0;$o<$userList[$i]['nbResources'];$o++){
			// Resource name
			echo '<tr><td width="*"><img alt="" src="'.$userList[$i][$o]['icon'].'" width="16" height="16" style="vertical-align:middle; margin-right:1em;">'.cfUTF8Encode($userList[$i][$o]['name']);
			// Configure resource button
			if(isset($userList[$i][$o]['config']) && cfRGetVar('advancedAdministration')){
				echo '<td width="20%">'.outButton(cfCaption('genConfigure'),'?config='.urlencode($userList[$i][$o]['file']),outIcon('config3')).'</td>';}
			else
				{echo '<td width="20%">'.outButtonDisabled(cfCaption('genConfigure'),false,outIcon('config3'));echo '</td>';}
			// Remove resource button
			echo '<td width="20%">'.outButton(cfCaption('genRemove'),'?resourceRemove='.$o.'&amp;user='.$i,outIcon('cancel')).'</td>';
			echo "</tr>\n";
		}
		echo '</table>';
	}

	echo '<div class="frame2Footer">';
	// Deactivate user button
	if(strtolower($userList[$i]['activated'])=='true' || $userList[$i]['activated']=='1' || $userList[$i]['activated']===true) echo outButton(cfCaption('genActivateFalse'),'?userDeactivate='.$i,outIcon('off'));
	// Activate user button
	elseif(cfRGetVar('advancedAdministration')) echo outButton(cfCaption('genActivateTrue'),'?userActivate='.$i,outIcon('on'));
	else echo outButtonDisabled(cfCaption('genActivateTrue'),false,outIcon('on'));
	// Add a resource to user button
	if(cfRGetVar('advancedAdministration')){
		echo '<div style="position:absolute;right:0;bottom:0">';
		if(!isset($_GET['resourceAdd']) || (isset($_GET['resourceAdd']) && $_GET['resourceAdd']!=$i))
			echo outButton(cfCaption('adminResourceAdd'),'?resourceAdd='.$i,outIcon('add'));
		if(isset($_GET['resourceAdd']) && $_GET['resourceAdd']==$i)
			echo outButton(cfCaption('adminResourceCancelAdd'),'?cancel',outIcon('sup'));
		echo "</div>";
	}
	echo "</div></div>\n";
}
echo '<div class="frame1Footer"></div></div>'."\n";
?>
