<?php
/**
 * RSS feed viewer
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */


define('FEEDS_FILENAME','feeds.xml');
define('FEEDS_SERIALIZED_FILENAME','feeds');

/**
 * @desc tabs class
 *
 */
class tab{
	public $id; // Tab's unique ID
	public $name; // Tab's caption
	public $iconURL; // URL of icon. Set to 0 for default, set to false for no icon
	public $selected; // True if tab is selected
	public $static=false; // True if tab cannot be moved
	public $onclickFunction=false; // Special onclick function
	public $tooltipCaption=false; // tooltip text

    public $imported; // "imported from" feeds. False if not imported

	/**
     * @desc class constructor
     *
     * @param $id (optional) tab id. If omited, random id is generated
     */
	function __construct($id=false){
		if($id) $this->id=$id; else $this->id='tab_'.rand(0,99999999);
		$this->name=false;
		$this->iconURL=0;
		$this->selected=false;
		$this->index=99999;
		$this->imported=false;
	}

	/**
	 * @desc generate tab's opml DOM node from tab
	 *
	 * @param DOM node $dom
	 * @return DOM node
	 */
	function getNode(&$dom){
		$node=$dom->createElement('outline');
		$node->setAttribute('id',$this->id);
		$node->setAttribute('text',$this->name);
		if($this->selected) $node->setAttribute('selected',true);
        if($this->iconURL)  $node->setAttribute('icon',$this->iconURL);
        if($this->imported) $node->setAttribute('imported',$this->imported);
        $node->nodeValue="\n";
		return $node;
	}


	public function icon(){if($this->iconURL===0) return outIcon('rss'); else return $this->iconURL;}
	public function setIcon($iconURL){$this->iconURL=$iconURL;}
	public function setStatic(){$this->static=true;}
	public function onclick($function){$this->onclickFunction=$function;}
	public function tooltip($caption){$this->tooltipCaption=$caption;}
}

/**
 * @desc Single RSS feed & box class
 *
 */
class RSSFeed{
    public $id; // feed ID
    public $url; // xml file URL
    public $channelInfo; // Parsed rss feed channel info
    public $redirectedUrl; // Actual URL of feed if ->url is HTTP redirected
    public $title; // title
    public $icon; // feed icon URL (deprecated)
    public $favicon; // feed cached & png-converted favicon filename

	public $lastUpdate; // timestamp of last update
	public $updateFrequency; // XML update frequency (seconds)

    //public $xmlValue; // last retreived XML file
    public $items; // array of parsed items: id=>{pubdate, title, description, link}

    public $tab; // id of tab
    public $folded; // true for folded view, false for unfolded view
    public $maxItems; // max displayed items (if false, use default value)

    public $col; // column index
    public $yIndex; // y-axis index in columnt (0 = top)

    private $encoding; // XML encoding charset

    /**
     * @desc class constructor
     *
     * @param string $url : RSS XML URL
     */
    function __construct($url=false){
        $this->id=rand(0,99999999);
        $this->url=$url;
        $this->redirectedUrl=false;
        $this->title=false;
        $this->icon=false;
        $this->favicon=false;

		$this->lastUpdate=0;
		$this->updateFrequency=false;

        $this->items=array();

        $this->tab=false;
        $this->folded=false;
        $this->compactDisplay=false;
        $this->maxItems=false;

        $this->col=0;
        $this->yIndex=1000;
    }

    /**
     * @desc Return most probable image (if any) from <description> tag
     *
     * @param string $desc: HTML to be parsed
     * @param boolean $breakOnFirst: true to return when 1st satisfying image is found
     */
    function getDescriptionImage($item){
		if(isset($item['enclosures'])){
			foreach ($item['enclosures'] as $enclosure) {
				if(isset($enclosure['url']) && isset($enclosure['type']) && substr($enclosure['type'],0,5)=='image') return $enclosure['url'];
			}
		}
		if(!isset($item['description'])) return false;

		$desc=$item['description'];
		$imgPos=0;$imageLink=false;
		while ($imgPos!==false) {
			if(($imgPos=stripos($desc,'<img',$imgPos))!==false) {
				$gtPos=strpos($desc,'>',$imgPos);
				$srcPos=stripos($desc,'src="',$imgPos);
				$quotePos=strpos($desc,'"',$srcPos+6);
				$widthPos=strpos($desc,'width="1"',$imgPos);
				if($srcPos!==false && $gtPos && $quotePos && $quotePos<$gtPos && (!$widthPos || $widthPos>$gtPos)){

					$tmpLink=substr($desc,$srcPos+5,$quotePos-$srcPos-5);
					if(!strpos($tmpLink,'feedburner.com')){
						if(strripos($tmpLink,'.jpg') || strripos($tmpLink,'.jpeg')){
							return $tmpLink;
						}
						if(strripos($tmpLink,'.png')) $imageLink=$tmpLink;
						elseif (!$imageLink || !stripos($imageLink,'.png')) $imageLink=$tmpLink;
					}
				}
				else break;
				$imgPos=$gtPos;
			}
		}
		return $imageLink;
    }

    /**
     * @desc return HTML code tooltip
     * @param integer $feedItemId: sequential id of feed item
     * @param array $options: rawText: remove HTML wrapping
     *
     * @return string : html code
     */
    public function tooltipArray($feedItemId,$options=array()){
    	// Parse feed
        $info=array();
        $nb=0;
        foreach ($this->items as $id=>$item) {
        	if($nb==$feedItemId){
        		// Title
        		$title=cfFixedDate($item['pubdate'],cfCaption('_FILE_DATE_FORMAT')).cfCaption('genSeparator');
        		$title.=isset($item['title'])?$item['title']:'';
        		$info['header']['c']=str_replace(' ','&nbsp;',$title);

        		// Thumbnail - image
        		if(0 && $imageURL=$this->getDescriptionImage($item)){
					$info['thumbnailURL']='code:';
					$info['thumbnailURL'].='<div class="RSSImage" style="float:none;margin:0" width="'.cfRGetVar('linkImageWidth').'" height="'.cfRGetVar('linkImageHeight').'"><img src="'.$imageURL.'" onload="posImg(this.id)" id="thumbnailURLImg"></div>';
        		}

        		// Full-text
        		$info['extra']=array();
        		$info['extra']['extraTitle']=array('c'=>' ');
        		if(isset($item['description'])) {
        			if(@$options['rawText'])	$info['extra']['desc']=array('chtml'=>$item['description']);
        			else						$info['extra']['desc']=array('chtml'=>'<div style="display:block;width:50em">'.$item['description'].'</div><br>');
        		}

        		// Footer
        		if(isset($item['link'])) {
        			if(@$options['rawText'])	$info['footer']=array('c'=>cfUTF8Decode(cfCaption('genLink').cfCaption('genSeparator')),'vhtml'=>$item['link']);
        			else						$info['footer']=array('c'=>cfUTF8Decode(cfCaption('genLink').cfCaption('genSeparator')),'vhtml'=>'<a>'.$item['link'].'</a>');
        		}

        		unset($rss);
        		return $info;
        	}
        	$nb++;
        }
        unset($rss);
    }

    /**
     * @desc Modify texted XML before parsing as modifying XML_RSS is complicated ;(
     *
     * @param string $body
     */
    function prepareXMLTxt(&$body){
    	$pos=0;
        $body=str_replace("\r",' ',str_replace("\n",' ',substr($body,$pos)));

        // Replace "entry" nodes by "item" and "feed" nodes by "channel" (as RSS_XML doesn't seems to support this tags
        if(stripos($body,'<entry')) $body=str_ireplace('<entry','<item',str_ireplace('</entry','</item',$body));
        if(stripos($body,'<feed')) $body=str_ireplace('<feed','<channel',str_ireplace('</feed','</channel',$body));

        // <!DOCTYPE> node if present (issues with some feeds...)
		if($doctypePos=stripos($body,'<!doctype')){
			if($inf1=strpos($body,'[',$doctypePos+1))
				if($inf2=strpos($body,']',$inf1+1))
					if(strpos($body,'>',$inf2)<strpos($body,'<',$inf2))
						for($i=$inf1;$i<=$inf2;$i++) $body[$i]=' ';
		}

		// Fix link (and possibly image) tags with empty content but href => put href in content
		$pos=0;
		while ($p=strpos($body,'/>',$pos)){
			$fixedTags=array('link'=>1,'image'=>1);

			$startTag=$p;
			while ($startTag>0 && substr($body,$startTag,1)!='<') $startTag--;
			$in=substr($body,$startTag,$p-$startTag+2);
			$tag=substr($in,1,strpos($in,' ')-1);

			if(($hrefPos=stripos($in,'href="')) && isset($fixedTags[strtolower($tag)])){
				$l=strlen($in);

				$href=substr($in,$hrefPos+6,strpos($in,'"',$hrefPos+6)-$hrefPos-6);

				$body=substr($body,0,$p).'>'.$href.'</'.$tag.'>'.substr($body,$p+2);
				$pos=$p+strlen($href)+strlen($tag)+2;

			}
			else $pos=$p+1;
		}
	}

    /**
     * @desc return HTML code for feed element
     *
     * @return string : html code
     */
    public function HTML($includeOuterHTML=true, $extraStyle=false){
    	// Parse feed
        $channelInfo=@$this->channelInfo;

		if(is_array($this->items)) uasort($this->items,'rrsSortFunction');

        /*
         * Items
         */
		$itemsHTML='';
        $itemsHTML.=outTableTransparent('frame2', 'text-align:left',outEHoverFrameAttr())/*.'<colgroup><col width="1"><col width="*"></colgroup>'*/;
        $hasTooltip=cfHGetVar('tooltipFileInfo');

		// Initialize items number / date filter
        rssItemsFilter('init', $this->maxItems);
        $displayed=0;
        $index=-1;

        $today=mktime(0,0,1,date('n'),date('j'),date('Y'));

        // Icon and website link
        if(isset($channelInfo['link'])) $siteLink=$channelInfo['link'];
        else{
        	$siteLink=parse_url($this->url);
        	$siteLink=((isset($siteLink['scheme'])?$siteLink['scheme']:'http')).'://'.$siteLink['host'].((isset($siteLink['port'])?':'.$siteLink['port']:'')).'/';
        }

        // Generate HTML for items
        if(is_array($this->items)) foreach ($this->items as $item ){
        	$index++;

        	if(!($filter=rssItemsFilter($item))) continue;

        	if(!isset($item['link'])) $item['link']=$siteLink;
        	$item['link']=str_replace('&nbsp;',' ',$item['link']);

	        // Feed title
	        if($this->title) $title=$this->title;
	        else {
	        	$title=parse_url($this->url);
	        	$title=str_replace('www.','',$title['host']);
	        }
        	$protectedTitle=str_replace("'","\\'",str_replace('"','\\"',$title));
        	$link='ol(\''.linkSanitize($item['link'])."', '".$protectedTitle."')";

        	$itemsHTML.='<tr id="tr'.$this->id.'_'.(int)$index.'" class="eTC '.(($displayed%2)?'odd':'even').'" eH=1 onclick="'.$link.'"'.(($hasTooltip)?' onmouseover="tooltip(this)"':'').'>';

        	// Detailed display
        	if(!$this->compactDisplay){
        		$itemsHTML.='<td class="RSSItem">';
	        	if(@$item['pubdate']) $itemsHTML.='<div style="float:right">'.cfFixedDate($item['pubdate'],(($item['pubdate']>=$today)?'H:i':cfCaption('_DATE_FORMAT'))).'</div>';
	        	$itemsHTML.='<img class="ico" src="'.(($filter==='new')?'new.png':'old.png').'">';

        		// Get image URL
        		$imageURL=$this->getDescriptionImage($item);

        		// Display image
        		if($imageURL) {
        			$itemsHTML.='<div class="frame3 RSSImage" style="width:'.cfRGetVar('linkImageWidth').'px; height:'.cfRGetVar('linkImageHeight').'px">';
        			$itemsHTML.='<img name="img" alt="'.$imageURL.'" onload="posImg(this.id)" id="feedItem'.$this->id.'_img_'.(int)$index.'"/></div>';
        		}
        		// No image
        		//else $itemsHTML.='<ul><li>';

        		// Title
        		$itemsHTML.='<a href="#">'.((isset($item['title']))?$item['title']:'RSS')."</a><br/>\n";
        		// Detail
            	$itemsHTML.=((isset($item['description']))?$this->formatDescription($item['description']):'');

            	//if(!$imageURL) $itemsHTML.='</li></ul>';
            	$itemsHTML.='</td>';
        	}

        	// Compact display
        	elseif($this->compactDisplay){
				$itemsHTML.='<td class="eTCLeft" style="width:10px;vertical-align:top;padding-left:0.5em;'.(($this->compactDisplay || !isset($item['description']))?'':'padding:0.5em').'">';
				$itemsHTML.='<img class="ico" src="'.(($filter==='new')?'new.png':'old.png').'">';
            	$itemsHTML.='</td>';
            	$itemsHTML.='<td class="eTCRight"><a href="#"><b>'.@$item['title']."</b></td>\n";
        	}
        	$itemsHTML.="</tr>\n";
        	$displayed++;
        }
        $itemsHTML.='</table>';

        if(!is_array($this->items) || count($this->items)==0) $itemsHTML=cfCaption('blogNoPost');



        cfDragAddItem("feedItem".$this->id,'+INERTIA+PICKDELAY+500');

        // Outer div
        if($includeOuterHTML) $output='<div name="fI" id="feedItem'.$this->id.'" class="frame2" style="position:absolute;cursor:move;width:'.floor(100/cfRGetVar('nbColumns') - 3).'%;'.(($extraStyle)?$extraStyle:'').'">';
        else $output='';

        // Feed title
        if($this->title) $title=$this->title;
        else {
        	$title=parse_url($this->url);
        	$title=str_replace('www.','',$title['host']);
        }
        $protectedTitle=str_replace("'","\\'",str_replace('"','\\"',$title));

        // Icon and website link
        if(isset($channelInfo['link'])) $siteLink=$channelInfo['link'];
        else{
        	$siteLink=parse_url($this->url);
        	$siteLink=((isset($siteLink['scheme'])?$siteLink['scheme']:'http')).'://'.$siteLink['host'].((isset($siteLink['port'])?':'.$siteLink['port']:'')).'/';
        }
        $cil=' onclick="ol(\''.linkSanitize($siteLink).'\',\''.$protectedTitle.'\')" style="cursor:pointer"';

        // favicon
        $l='<img name="img" alt="" src="'.((substr($this->favicon,0,1)=='/')?$this->favicon:mfFaviconLink($this->favicon)).'" id="feedItem'.$this->id.'_icon" class="ico"'.$cil.'">';

        // Title
        $c='<span id="feedItem'.$this->id.'_title" onmouseup="editTitle(this.id.substr(0,this.id.length-6))" style="vertical-align:middle">'.$title.'</span><b>&nbsp;&nbsp;'.' ('.$displayed.')</b>';

        // Header

        // Options
        $r=outButtonSmall(false,'#',outIcon('gears'),false,false,'onclick="dispOpt(\''.$this->id.'\')" style="margin-right:1em"');

        // Toggle collapse button
        $r.=outButtonToggleCollapse($this->id.'_cDiv',false,!$this->folded,"'feedCollapse'");

        $output.='<div class="frame2Header"><table class="feedHeaderT"><tr><td style="width:2em">'.$l.'</td><td style="overflow:hidden;white-space:nowrap">'.$c.'</td><td style="width:5em;text-align:right">'.$r.'</td></tr></table></div>';

        // Collapsable frame
        $output.=outCollapsableFrame($this->id.'_cDiv',false,false,$this->folded);

        // Hidden data
        $output.='<div style="display:none">';
        // Time before update
        $output.='<div id="'.$this->id.'_nu">'.$this->nextUpdate().'</div>';
        // Max items
        $output.='<div id="'.$this->id.'_mi">'.((int)$this->maxItems).'</div>';
        // Compact display
        $output.='<div id="'.$this->id.'_cd">'.(int)$this->compactDisplay.'</div>';
        // URL
        $output.='<div id="'.$this->id.'_url">'.$this->url.'</div>';
        $output.='</div>';

        // ITEMS
        $output.=$itemsHTML;

        $output.='</div>';

        if($includeOuterHTML) $output.="</div>\n\n\n\n";

        return $output;
    }

    /**
     * @desc return HTML code for feed element
     *
     * @return string : html code
     */
    public function js(){
    	// Parse feed
        $channelInfo=$this->channelInfo;

        $js=array(
        	'feed'=>array(
        		'id'=>$this->id,						// feed id
        		'url'=>$this->url,						// feed's url
        		'ico'=>((substr($this->favicon,0,1)=='/')?$this->favicon:mfFaviconLink($this->favicon)),	// feed icon url
        		'fold'=>(int)$this->folded,				// 1 if feed is folded
        		'nu'=>(int)$this->nextUpdate(),			// time before next update
        		'mi'=>(int)$this->maxItems,				// max nb of items
        		'tt'=>(int)cfHGetVar('tooltipFileInfo'),// 1 if tooltip should be displayed
        		'comp'=>(int)$this->compactDisplay,		// 1 for compact display, 0 for extended
        		'imgw'=>cfRGetVar('linkImageWidth'),	// Detailed image width
        		'imgh'=>cfRGetVar('linkImageHeight')	// Detailed image height
        		),
        	'items'=>array()
        );

        // Feed title
        if($this->title) $title=$this->title;
        else {
        	$title=parse_url($this->url);
        	$title=str_replace('www.','',$title['host']);
        }
        $js['feed']['title']=$title;

        // Icon and website link
        if(isset($channelInfo['link'])) $js['feed']['link']=$channelInfo['link'];
        else{
        	$siteLink=parse_url($this->url);
        	$js['feed']['link']=((isset($siteLink['scheme'])?$siteLink['scheme']:'http')).'://'.$siteLink['host'].((isset($siteLink['port'])?':'.$siteLink['port']:'')).'/';
        }

        /*
         * Items
         */
        $items=array();

		// Initialize items number / date filter
        rssItemsFilter('init', $this->maxItems);
        $displayed=0;
        $index=-1;
        $today=mktime(0,0,1,date('n'),date('j'),date('Y'));

        // Generate HTML for items
        foreach ($this->items as $item){
        	$index++;

        	if(!($filter=rssItemsFilter($item))) continue;

        	$jsItem=array(); // Item sub id

        	if(!isset($item['link'])) $item['link']=$siteLink;

        	$jsItem['l']=linkSanitize(str_replace('&nbsp;',' ',$item['link'])); // Link
        	$jsItem['t']=$item['link']; 										// Title
        	if(!$this->compactDisplay && isset($item['description'])) $jsItem['desc']=$item['description'];	// Detailed description
        	$jsItem['date']=cfFixedDate($item['pubdate'],(($item['pubdate']>=$today)?'H:i':cfCaption('_DATE_FORMAT')));	// Formated publication date
        	$jsItem['n']=($filter=='new')?1:0;									// 1 for new items, 0 for already-displayed ones

        	// Detailed display
        	if(!$this->compactDisplay){
        		// Display image
        		if($imageURL=$this->getDescriptionImage($item)) $item['img']=$imageURL;
        	}

        	// Add item to feed
        	$js['items'][$displayed]=$jsItem;

        	$displayed++;
        }

        return json_encode($js);
    }

    /**
     * @desc return HTML code for feed element
     *
     * @return string : html code
     */
    public function mobileHTML(){
    	// Parse feed
        $channelInfo=@$this->channelInfo;

        // Icon
        $icon='<img src="'.((substr($this->favicon,0,1)=='/')?$this->favicon:mfFaviconLink($this->favicon)).'" class="frW ico16" alt="ico">';

        /**
         * Feed header
         */

        // Feed title
        if($this->title) $title=$this->title;
        else {
        	$title=parse_url($this->url);
        	$title=str_replace('www.','',$title['host']);
        }

        // Website link
        if(isset($channelInfo['link'])) $siteLink=$channelInfo['link'];
        else{
        	$siteLink=parse_url($this->url);
        	$siteLink=((isset($siteLink['scheme'])?$siteLink['scheme']:'http')).'://'.$siteLink['host'].((isset($siteLink['port'])?':'.$siteLink['port']:'')).'/';
        }
        $host=parse_url($siteLink);$host=$host['host'];

        $items='<div class="frame2">';
        $items.=mfListTable(false,'h1').mfListItem('onclick="ol(\''.$siteLink.'\')"',false,false,'<div class="lits">'.$title.'</div>'.$host,false,$icon).'</table>';


        /*
         * Items
         */
        $items.=mfListTable();

        // Length of displayed detail text
        $truncateLength=floor(pow(max(cfGGetVar('userScreenWidth'),cfGGetVar('userScreenHeight'))/30,1.5));

        // Initialize items number / date filter
        rssItemsFilter('init', $this->maxItems);
        $displayed=0;
        $index=-1;
        $today=mktime(0,0,1,date('n'),date('j'),date('Y'));


        // Generate HTML
        foreach ($this->items as $item) {
        	$index++;
        	if(!($filter=rssItemsFilter($item))) continue;

        	$id=$this->id.'_'.(int)$index;

        	// Link
        	if(!isset($item['link'])) $item['link']=$siteLink;
        	$item['link']=str_replace('&nbsp;',' ',$item['link']);
        	$link="id=\"go".$id."\" onclick=\"ol('".$item['link']."')\"";

        	// Unfold link
        	$ufLink2="onclick=\"ufl('".$id."')\"";

        	// Title
        	$title='';
			if($item['pubdate'])  $title='<div style="float:right;font-size:65%">'.cfFixedDate($item['pubdate'],(($item['pubdate']>=$today)?'H:i':cfCaption('_DATE_FORMAT'))).'</div>';
			$title.=((isset($item['title']))?$item['title']:'RSS');


    		// Get image URL
    		$imageURL=$this->getDescriptionImage($item);

			// Detail
			$item['description']=(isset($item['description']))?trim($item['description']):'';
			if($item['description']){
				$detail=cfStrTruncate($this->formatDescription($item['description']),$truncateLength);
				$image=outImage(outIcon('plus'),'','id="ico'.$id.'" class="frLi16"');
				$ufLink1="onclick=\"ufl('".$id."',1)\"";
			}
			// No detail available for this feed
			else{
				$detail='&nbsp;';
				$image=outImage(outIcon('v'),'','id="ico'.$id.'" class="nfrLi32"');
				$ufLink1=false;
			}

    		// Display image
    		$image='';

    		if($imageURL) $image='<img src="?tn='.$id.'" class="rssImg ani" style="width:1px;height:1px;opacity:0;margin:16px 24px 15px 23px" onload="s=this.style;s.width=s.height=s.margin=\'\';s.opacity=1">';
    		//if($imageURL) $image='<img src="?tn='.$id.'" class="rssImg ani" style="width:1px;height:1px" onload="this.style.width=this.style.height=\'\'">';

			$items.=mfListItem($ufLink1,false,false,$image.'<div class="lit">'.$title.'</div><div id="det'.$id.'">'.$detail.'</div>',false,outImage(outIcon('go')),$link);

			$displayed++;
        }
        $items.='</table></div>';

        /**
         * Formated output
         */

        return $items;
    }

    /**
     * @desc Async refresh displayed feed
     * @param $commitImages: true: send JS command for updating images src from alt values
     *
     */
    function asyncUpdateHTML($commitImages=false){
    	echo cfAsyncXMLInnerHTMLbyId($this->HTML(false),'feedItem'.$this->id);
    	echo cfAsyncXMLJSaction('dd.elements["feedItem'.$this->id.'"].resizeTo(colWidth-hspacing,"auto");dd.recalcItem("feedItem'.$this->id.'");resetPos(false,true)');
    	if($commitImages) echo cfAsyncXMLJSaction('setImages()');
    }

    /**
     * @desc download feed and update content (or use passed XML if provided)
     *
     * @param string $newXML: use passed XML instead of retreiving from url
     * @param string $body: put in this value actual feed's body. Used when first parsing a feed that is actually an HTML page including
     * 				some <link rel="alternate" type="application/rss+xml" to feeds (so there is no need for another request)
     * @return string result :
     * 			'timeout' if server couldn't be contacted
     * 			'invalidXML' if XML file is not OK
     * 			'OK' if file successfully red and parsed
     */
    function updateXML($newXML=false,&$body=false){
    	$pageFound=false;
    	$url=$this->url;

		// Update last update time
        $this->lastUpdate=time();

    	// Browse redirections
    	while($newXML===false && !$pageFound){
    		// Read XML
    		if(!(cfParseHTTPResponse(cfSocketHTTPRequest($url,cfRGetVar('updateTimeout'),false,$errstr,$ccnx),$status,$body,$headers))){
    			if($errstr=='Invalid URL') return 'invalidXML'; else return 'timeout';
    		}

			// Check redirections
			if(($status==301 || $status==302 || $status==307) && isset($headers['location'])){
				// Update URL if moved permanently
				if($status==301) $permanent=true; else $permanent=false;
				$url=$headers['location'];
			}
			else $pageFound=true;
    	}
    	if($newXML!==false) $body=$newXML;

    	if($url!==$this->url) $this->redirectedUrl=$url;

    	// Find XML or RSS tag
		if(($pos=stripos($body,'<?xml'))===false && ($pos=stripos($body,'<rss'))===false) {
			// If page is HTML, return info so it can be parsed to find rss feed(s) link(s)
			if(strpos($body,'<html')!==false || strpos($body,'<!DOCTYPE html')!==false) return 'HTML';
			return 'invalidXML';
		}

		// Prepare XML's text before parsing
		$this->prepareXMLTxt($body);

        if(!$body) return 'invalidXML';

        // Check format
		$rss=new XML_RSS($body);
		$rss->parse();

		$this->channelInfo=$rss->getChannelInfo();
		if(count($rss->getItems())==0 || count($this->channelInfo)==0) {
			unset($rss);
			return 'invalidXML';
		}

		// Build an already loaded feeds list ids to accelerate processing
		$ids=array(); if(is_array($this->items)) foreach ($this->items as $item) $ids[$item['id']]=1;


		// Add feeds to list
		$items=$rss->getItems();
		krsort($items); // Invert items order (reverse key sort on an index) so latest articles are processed last, so if there is no pubdate, generated pubdate will be in correct order
		$updated=false;
		foreach ($items as $item){
			// Clean attribute names
			foreach ($item as $k=>$v) if(strpos($k,':')!==false) $item[substr($k,strpos($k,':')+1)]=$v;

			// Publication date
			$pubdate=0;

			if(@$item['pubdate']) $pubdate=$item['pubdate'];
			else if(@$item['pubDate']) $pubdate=$item['pubDate'];
			else if(@$item['published']) $pubdate=$item['published'];
			else if(@$item['updated']) $pubdate=$item['updated'];
			else if(@$item['date']) $pubdate=$item['date'];
			else $pubdate=microtime(true);

			if(!is_numeric($pubdate)) $pubdate=strtotime($pubdate);

			$item=array(
				'title'=>@$item['title'],
				'description'=>@$item['description'],
				'pubdate'=>$pubdate,
				'link'=>@$item['link'],
				'importTime'=>microtime(true)
			);

			// Compute an id based on item content
			$item['id']=md5('a'.@$item['time'].@$item['link']);

			// Skip this feed if already imported
			if(isset($ids[$item['id']])) continue;

			$this->items[]=$item;
			$updated=true;
		}

		// Remove extra feeds
		if($updated){
			krsort($this->items,SORT_NUMERIC);
			while(count($this->items)>cfConstant('RSS_MAX_STORED_ITEMS')) array_pop($this->items);
		}

        // Set title if empty
		if(!$this->title) $title=@$this->channelInfo['title'];

        unset($rss);
        return 'OK';
    }

	/**
	 * @desc generate feed's opml DOM node from feed
	 *
	 * @param DOM node $dom
	 * @return DOM node
	 */
    public function getNode(&$dom){
		$node=$dom->createElement('outline');
		$node->setAttribute('id',$this->id);
		$node->setAttribute('text',$this->title);
		$node->setAttribute('type','link');
		$node->setAttribute('xmlUrl',$this->url);

		if($this->icon) $node->setAttribute('icon',$this->icon);
		$node->setAttribute('lastUpdate',$this->lastUpdate);
		$node->setAttribute('updateFrequency',$this->updateFrequency);
		$node->setAttribute('folded',(($this->folded)?'true':'false'));
		$node->setAttribute('compactDisplay',(($this->compactDisplay)?'true':'false'));
		$node->setAttribute('maxItems',(int)$this->maxItems);
		$node->setAttribute('col',$this->col);
		$node->setAttribute('yIndex',$this->yIndex);
		$node->setAttribute('xmlValue',base64_encode($this->xmlValue)); if(0) cfDbg();
		$node->nodeValue="\n";
		return $node;
    }

    /**
     * @desc return number of seconds before next update is required
     *
     * @return integer : result
     *
     */
	public function nextUpdate(){
		return (max(0,(($this->updateFrequency)?$this->updateFrequency:cfRGetVar('updateFrequency'))-(time()-$this->lastUpdate)));
	}


	/**
	 * @desc Try to find and download feed's favicon
	 *
	 */
	public function getFavicon(){
		$urls=array($this->url);
		if($this->redirectedUrl) $urls[]=$this->redirectedUrl;
		mfGetFavicons($urls,array('toFile'=>cfAppResourceDir()));
		if($this->redirectedUrl && mfFaviconFile($this->redirectedUrl)) $this->favicon=$this->redirectedUrl;
		elseif (mfFaviconFile($this->url) && mfFaviconFile($this->url)!=='noIcon') $this->favicon=$this->url;
		else $this->favicon=outIcon('rss');

	}

	/**
	 * @desc Format description field for output
	 *
	 * @param string $desc
	 * @return string
	 */
	function formatDescription($desc){
		$desc=strip_tags($desc);
		$pos=cfRGetVar('descriptionLength');
		if(!$pos || mb_strlen($desc,'UTF-8')<$pos) {
			return wordwrap($desc,40," ",true);
		}
		$pos= mb_strlen(mb_substr($desc,0,$pos,'UTF-8'),'UTF-8');
		$sep=' ;.:,?_-*+/\\)([]\'"'."\n\r";
		while(isset($desc[$pos]) && strpos($sep,$desc[$pos])===false && $pos>cfRGetVar('descriptionLength')-20) $pos--;
		$pos=substr($desc,0,$pos).'...';
		return wordwrap($pos,40," ",true);;
	}
}

/**
 ***************************************************************************************************************************
 * Process POST
 * $_POST['data3']: action:
 *		updateLayout,
 *		addFeed, deleteFeed, checkUpdates,
 *		changeTitle, changeFolding,changeMaxItems, setCompact, changeURL, getFeedInfo
 *		tabSort, tabSelect, tabCreate, tabDel, tabEditCaption, tabChangeIcon, addImportedFeeds
 ***************************************************************************************************************************
 **/
function rssProcessPost(){
	global $rssFeeds;
	global $tabs;
	global $editAllowed;
	global $cols;

	if(!isset($_POST['data3']) || !isset($_POST['data2'])) return;

	$needSave=false;
	$action=$_POST['data3'];


	// Header
	cfAsyncHeader();

	/**
	 * FEEDS
	 */

	// Update feeds layout
	if($action=='updateLayout'){
		foreach (explode('/',$_POST['data2']) as $colNb=>$colValue){
			foreach (explode(';',$colValue) as $yIndex=>$feedId){
				if(isset($rssFeeds[$feedId])) {
					$rssFeeds[$feedId]->yIndex=$yIndex;
					$rssFeeds[$feedId]->col=$colNb;
				}
			}
		}
		$needSave=true;
	}

	// Add new feed
	elseif($action=='addFeed' && $editAllowed){
	    $url=$_POST['data2'];
	    if($url=='a') $url='http://foo/foo.xml';
	    $comp=parse_url($url);

	    // Malformed URL
	    if(!isset($comp['host'])) {
	    	echo cfAsyncXMLJSaction('setNewFeedError("'.cfCaptionJS('bookmarksInvalidURL').'")');
	    }
	    else {
	    	// Check that this feed is not already in feeds list
	    	foreach ($rssFeeds as $feed) if(strtolower($feed->url)==strtolower($url)) {
	    		echo cfAsyncXMLJSaction('setNewFeedError("'.cfCaptionJS('rssExistingFeed').'")');
			    die(cfAsyncFooter());
	    	}

	    	// Download and parse XML
	    	$feed=new RSSFeed($url);
	    	$feed->tab=cfRGetVar('selectedTab');
	    	$res=$feed->updateXML(false,$html);

	    	// If it's actually an HTML, search for RSS feeds into link nodes with type application set to
	    	if($res=='HTML'){
	    		$dom = new DOMDocument();
				@$dom->loadHTML($html);
				$xpath = new DOMXPath($dom);
				$xpath_query = "//link[@type='application/rss+xml']";
				$result_rows = $xpath->query($xpath_query);

				// Put RSS links into an array
				$links=array();$nb=0;
				foreach ($result_rows as $result_object){
					$href=@$result_object->getAttribute('href');
					$title=@$result_object->getAttribute('title');
					if($href){
						$links[]=array('href'=>$href,'title'=>$title);
						if($title) $nb++;
					}
				}

				// Only one link: directly use it (or if multiple links found but none have a title)
				if(count($links)==1 || (count($links)>0 && $nb==0)){
			    	// Download and parse XML
			    	$feed=new RSSFeed($links[0]['href']);
			    	$feed->tab=cfRGetVar('selectedTab');
			    	$res=$feed->updateXML();
				}
				// Multiple links
				elseif($nb>0){
					$multipleLinks=true;
					// Remove links without title
					foreach ($links as $k=>$link) if(!$link['title']) unset($links[$k]);
					// Display client-side links
					echo cfAsyncJSaction('setNewFeedLinks('.json_encode($links).')');
				}
				// No link found
				else{
					$res='invalidXML';
				}
	    	}

	    	// Timeout
	    	if($res=='timeout')	echo cfAsyncXMLJSaction('setNewFeedError("'.cfCaptionJS('rssServerTimeout').'")');
	    	// Invalid XML
	    	elseif($res=='invalidXML') echo cfAsyncXMLJSaction('setNewFeedError("'.cfCaptionJS('rssInvalidFeed').'")');
	    	// XML OK (except for multiple links)
	    	elseif(!@$multipleLinks){
	    		$rssFeeds[]=$feed;
	    		// Put it in most empty column
	    		$min=100; $colNb=0; if(is_array($cols)) foreach ($cols as $key=>$value) if(count($value)<$min) {$min=count($value);$colNb=$key;}
	    		$feed->col=$colNb;
				$needSave=true;
	    		echo cfAsyncXMLInnerHTMLbyId($feed->HTML(true),'feedsDiv',true);
	    		echo cfAsyncXMLJSaction('insertNewFeed("'.$feed->id.'",'.$feed->col.')');
	    	}
	    }
	}

	// Delete feed
	elseif($action=='deleteFeed' && $editAllowed){
		if(isset($rssFeeds[$_POST['data2']])){
			unset($rssFeeds[$_POST['data2']]);
			$needSave=true;
			echo cfAsyncXMLJSaction('removeDeletedFeed("feedItem'.$_POST['data2'].'")');
		}
		else echo cfAsyncXMLJSaction('hideDeleteFeedForm()');
	}

	// Check which feeds are outdated and update them
	elseif($action=='deleteCache'){
    	foreach ($rssFeeds as $feed) {
			$feed->items=array();
    	}
	}

	// Check which feeds are outdated and update them
	elseif($action=='checkUpdates'){
		$needSave=false;
		// update single feed
		if(isset($rssFeeds[$_POST['data2']])){
			if($rssFeeds[$_POST['data2']]->updateXML()=='OK') {
				$rssFeeds[$_POST['data2']]->asyncUpdateHTML(true);
				$needSave=true;
			}
			else{
				echo cfAsyncXMLJSaction('setIcon("feedItem'.$_POST['data2'].'", "e")');
			}
		}
		// Update all feeds (of selected tab)
		else{
			if(isset($_POST['data4']) && $_POST['data4']) $updateAll=true; else $updateAll=false;

			// Get URLs of feeds to update
			$urls=array();
			foreach ($rssFeeds as $id=>$feed) {
				if($feed->tab==cfRGetVar('selectedTab') && ($feed->nextUpdate()==0 || $updateAll))	$urls[$id]=$feed->url;
			}

			// Retreive XML
			$xmls=cfCURLMultiRequests($urls);

			// Apply retreived XML to feeds
			foreach ($rssFeeds as $id=>$feed){
				foreach ($xmls as $k=>$v) if($k==$id){
					if($feed->updateXML($v)=='OK') {
						$feed->asyncUpdateHTML(false);
						$rssFeeds[$id]=$feed;
						$needSave=true;
					}
					else{
						echo cfAsyncXMLJSaction('setIcon("feedItem'.$feed->id.'", "e")');
					}
				}
			}
			echo cfAsyncXMLJSaction('setImages()');

			// Update client-side timestamp
			//echo cfAsyncXMLJSaction("rssUpdateCookie('".cfRGetVar('selectedTab')."',".time().")");
		}
	}

	// Change feed title
	elseif($action=='changeTitle' && isset($_POST['data4']) && $editAllowed){
		if(isset($rssFeeds[$_POST['data2']])){
			$newTitle=$_POST['data4'];
		    if($newTitle!=$rssFeeds[$_POST['data2']]->title){
		    	$rssFeeds[$_POST['data2']]->title=$newTitle;
		    	$needSave=true;
		    }
		}
	}

	// Change feed folding
	elseif($action=='changeFolding' && isset($_POST['data4'])){
		if($_POST['data2']=='all'){
			foreach ($rssFeeds as $key=>$feed) if($feed->tab==cfRGetVar('selectedTab')){
				if($_POST['data4']) $rssFeeds[$key]->folded=true; else $rssFeeds[$key]->folded=false;
			}
	    	$needSave=true;
		}
		elseif(isset($rssFeeds[$_POST['data2']])){
			if($_POST['data4']) $rssFeeds[$_POST['data2']]->folded=true; else $rssFeeds[$_POST['data2']]->folded=false;
    		$needSave=true;
		}
	}


	// Toggle compact/detailed display
	elseif($action=='setCompact' && isset($_POST['data4'])){
		if($_POST['data2']=='all'){
			foreach ($rssFeeds as $key=>$feed) if($feed->tab==cfRGetVar('selectedTab')){
				$rssFeeds[$key]->compactDisplay=($_POST['data4'])?true:false;
			}
	    	$needSave=true;
			foreach ($rssFeeds as $id=>$feed) if($feed->tab==cfRGetVar('selectedTab')) $feed->asyncUpdateHTML(false);
			echo cfAsyncXMLJSaction('setImages()');
		}
		elseif(isset($rssFeeds[$_POST['data2']])){
			if($_POST['data4']=='true') $rssFeeds[$_POST['data2']]->compactDisplay=true; else $rssFeeds[$_POST['data2']]->compactDisplay=false;
	    	$rssFeeds[$_POST['data2']]->asyncUpdateHTML(true);
	    	$needSave=true;
		}
	}

	// Change feed's max displayed items
	elseif($action=='setFeedsDisplay' && isset($_POST['data2'])){
		if(in_array($_POST['data2'],array('all','day','last','number'))){
			cfRSetVar('feedsDisplay',$_POST['data2']);
			foreach ($rssFeeds as $id=>$feed) if($feed->tab==cfRGetVar('selectedTab')) $feed->asyncUpdateHTML(false);
			echo cfAsyncXMLJSaction('setImages()');
		}
	}
	elseif($action=='changeMaxItems' && isset($_POST['data4'])){
		if(isset($rssFeeds[$_POST['data2']])){
			if(is_numeric($_POST['data4']) && $_POST['data4']>=0){
		    	$rssFeeds[$_POST['data2']]->maxItems=$_POST['data4'];
		    	$needSave=true;
		    	$rssFeeds[$_POST['data2']]->asyncUpdateHTML(true);
		    }
		}
	}
	// Change feed's URL
	elseif($action=='changeURL' && isset($_POST['data4']) && isset($rssFeeds[$_POST['data2']])){
		$rssFeeds[$_POST['data2']]->url=$_POST['data4'];
		if($rssFeeds[$_POST['data2']]->updateXML()=='OK'){
			$rssFeeds[$_POST['data2']]->asyncUpdateHTML(true);
			cfAsyncXMLJSaction('hideOptionsForm()');
			$needSave=true;
		}
		else echo cfAsyncXMLJSaction('dgi("optionURL").style.color="red"');
		echo cfAsyncXMLJSaction('dgi("optionIcon").src="'.outIcon('gears').'"');
	}
	// Tooltip
	elseif ($action=='getFeedInfo'){
		list($feed,$itemNb)=@explode('_',$_POST['data2']);

		if(isset($rssFeeds[$feed]) && is_numeric($itemNb)) {
			if($info=$rssFeeds[$feed]->tooltipArray($itemNb)){
				require_once(INCLUDE_DIR.'fileInfoFunctions.php');
				echo cfAsyncXMLJSaction('tooltipSetProperties('.fiPHPArrayToJSArray($info).')');
			}
			else echo cfAsyncXMLJSaction('tooltipDestroy()');
		}
	}

	/**
	 * TABS
	 */

	// Sort
	elseif($action=='tabSort' && isset($_POST['data4'])){
		foreach (array_flip(explode(',',$_POST['data4'])) as $id=>$index) $tabs[$id]->index=$index;
	    // Sort indexes
	    uasort($tabs,'tabsSort');
		$needSave=true;
	}

	// Select tab
	elseif($action=='tabSelect' && isset($_POST['data4']) && isset($tabs[$_POST['data4']])){
		cfRSetVar('selectedTab', $_POST['data4']);
		foreach ($tabs as $id=>$tab) $tabs[$id]->selected=(($id==$_POST['data4'])?true:false);
		// Clear previous feeds and display tab's feeds
		insertFeedsAsyncHTML();

		echo cfAsyncXMLJSaction("rssUpdateCookie('".cfRGetVar('selectedTab')."',".time().")");
		$needSave=true;
	}

	// Create new tab
	elseif($action=='tabCreate'){
		foreach ($tabs as $id=>$tab) $tabs[$id]->selected=false;
		$newTab=new tab();
		$newTab->name=cfCaption('genNewTab');
		$newTab->selected=true;
		cfRSetVar('selectedTab',$newTab->id);
		$tabs[$newTab->id]=$newTab;

		// 2 feeds : reload whole page
		if(count($tabs)==2){
			echo cfAsyncXMLJSaction('document.location.reload()');
		}
		else{
			// Refresh tabs
			insertTabsAsyncHTML();

			// Clear previous feeds and display tab's feeds
			insertFeedsAsyncHTML();
		}

		$needSave=true;
	}

	// Delete tab
	elseif($action=='tabDel' && isset($_POST['data4']) && isset($tabs[$_POST['data4']]) && count($tabs)>1 && $editAllowed){
		// Destroy tab
		unset($tabs[$_POST['data4']]);
		// Select first tab
		$i=0;
		foreach ($tabs as $id=>$tab) {
			if(!$i) {cfRSetVar('selectedTab',$id);	$tabs[$id]->selected=true;}
			else $tabs[$id]->selected=false;
			$i++;
		}

		// Refresh tabs
		insertTabsAsyncHTML();

		// Clear previous feeds and display tab's feeds
		insertFeedsAsyncHTML();

		$needSave=true;
	}

	// Edit tab caption
	elseif($action=='tabEditCaption' && isset($_POST['data4']) && isset($_POST['data5']) && isset($tabs[$_POST['data4']]) && $editAllowed){
		$tabs[$_POST['data4']]->name=$_POST['data5'];

		// Refresh tabs
		insertTabsAsyncHTML();

		$needSave=true;
	}
	// Set tab's icon
	elseif($action=='tabChangeIcon' && isset($_POST['data4']) && isset($_POST['data5']) && isset($tabs[$_POST['data4']]) && $editAllowed){
		$tabs[$_POST['data4']]->setIcon('/gfx/'.basename($_POST['data5']));

		// Refresh tabs
		insertTabsAsyncHTML();

		$needSave=true;
	}

	// Import Feeds
	elseif($action=='addImportedFeeds'){
		if(substr($_POST['data2'],0,4)=='FILE')	$_POST['data2']='FILE'.strtolower(substr($_POST['data2'],4));

		if((isset($importedFeeds[$_POST['data2']]) xor substr($_POST['data2'],0,4)=='FILE') && !$importedFeeds[$_POST['data2']]){
			// Create new tab for imported feeds
			$newTab=new tab();
			$newTab->selected=true;
			$newTab->imported=$_POST['data2'];
			foreach ($tabs as $key=>$tab) $tabs[$key]->selected=false;
			$tabs[$newTab->id]=$newTab;
			cfRSetVar('selectedTab',$newTab->id);

			// Set tab name and icon, and import feeds
			if(!updateImportedFeeds($newTab, true)){
				echo cfAsyncXMLJSaction('alert("'.cfCaption('genError').cfCaption('genSeparator').cfCaption('rssImportError').'")');
			}
			else{
				// Refresh tabs
				insertTabsAsyncHTML();

				// Clear previous feeds and display tab's feeds
				insertFeedsAsyncHTML();

				$needSave=true;
			}
		}
	}

	// Unrecognized command
	else{}

    if($needSave) saveFeeds(true);
    die(cfAsyncFooter());
}

/**
 * @desc sort $tabs array on index property
 *
 */
function tabsSort($a,$b){
	if($a->index > $b->index) return 1;
	if($a->index < $b->index) return -1;
	return 0;
}

/**
 * @desc initialize resource data
 *
 */
function initResourceData(){

	cfRInitUserRightsVar('editionAllowed');

	if(cfRGetVar('parametersLoaded')) return;
	if(cfRGetVar('updatesCheckFrequency')===false) cfRSetVar('updatesCheckFrequency',120);
	if(!cfRGetVar('linkImageWidth')) cfRSetVar('linkImageWidth',70);
	if(!cfRGetVar('linkImageHeight')) cfRSetVar('linkImageHeight',50);
	if(cfRGetVar('descriptionLength')===false) cfRSetVar('descriptionLength',140);
	cfRInitVar('feedsDisplay','number');
	cfRInitVar('maxItems',10);

	if(!cfRGetVar('updateTimeout')) cfRSetVar('updateTimeout',7);

	cfSetSavedRVar('feedsDisplay');

	cfRSetVar('parametersLoaded',true);
}

/**
 * @desc load RSS feeds from resourceDataDir / feeds.ini file into RSSFeeds resource var
 * 		 and generate columns<->feeds array
 * 		 Load tabs (identified by an Id starting by "tab_"
 *
 * @return integer : number of feeds
 */
function loadFeeds(){
    global $rssFeeds;
    global $tabs;
    global $importedFeeds;
    global $cols;

    $needSave=0;
	cfRSetVar('spreadCols',0);
    $dom = new DOMDocument();
    $rssFeeds=array();

    /**
     * Read serialized data
     */
    if(file_exists(cfAppResourceDir().'/'.FEEDS_SERIALIZED_FILENAME)){
    	list($rssFeeds,$tabs,$selectedTab)=unserialize(file_get_contents(cfAppResourceDir().'/'.FEEDS_SERIALIZED_FILENAME));
    	foreach ($rssFeeds as $id=>$feed) if($id!=$feed->id){
    		$rssFeeds[$feed->id]=$feed;
    		unset($rssFeeds[$id]);
    		$needSave=1;
    	}
    }
    /**
     * If serialized data doesn't (yet) exist, parse from XML file
     */
    else{
    	$feeds=array();
    	$tabs=array();
    	$selectedTab=0;
    }

    // Create 1 tab if no existing tab
    if(count($tabs)==0)	{
    	$tabs['tab_0']=new tab('tab_0');
	    $tabs['tab_0']->name=cfCaption('rssFeeds');
	    $tabs['tab_0']->index=0;
	    $tabs['tab_0']->selected=true;
	    $needSave++;
    }
    // Select 1st tab if no selected tab
    if(!$selectedTab) foreach ($tabs as $id=>$tab){
    	$tabs[$id]->selected=true; $selectedTab=$id;
    	break;
    }
    cfRSetVar('selectedTab',$selectedTab);

    // Update imported feeds
    if(!cfRGetVar('importedTabsUpdated')){
    	foreach ($tabs as $id=>$tab)
        	 if($tabs[$id]->imported && updateImportedFeeds($tabs[$id])) $needSave++;

    	cfRSetVar('importedTabsUpdated',true);
	}

	// Save feeds if updated from imported feeds
	if($needSave || cfRGetVar('needSave')) saveFeeds();

	// Generate sorted by columns feeds array
    $cols=generateColsArray();
    unset($dom);

    return count($rssFeeds);
}

/**
 * @desc Generate cols array that sort feed items per column. Format :
 * 	$cols format : array(
 * 		array( // column 0
 * 			0=>top feed id, 1=>bottom feed id, ...),
 * 		array( // column 1
 * 			0=>top feed id, 1=>bottom feed id, ...)
 * 		...)
 * @param string $tabId : (optional) id of tab. Default : selected tab (cfRGetVar('selectedTab'))
 */
function generateColsArray($tabId=false){
    global $rssFeeds;

    if($tabId==false) $tabId=cfRGetVar('selectedTab');

    // Reset yIndex(es) and generate columns<->feeds array
    $columns=array();
    for($o=0;$o<cfRGetVar('nbColumns');$o++){
        $col=array();
        $columns[$o]=array();
        foreach ($rssFeeds as $key=>$value) if(@$value->tab==$tabId && $value->col==$o) {
            $col[$value->id]=$value->yIndex;
        }
        $yIndex=0;
        while(count($col)){
            $min=1001;
            $minId=false;
            foreach ($col as $key=>$value){
                if($value<$min) {$min=$value;$minId=$key;}
            }
            $rssFeeds[$minId]->yIndex=$yIndex;
            $columns[$o][$yIndex]=$minId;
            $yIndex++;
            unset($col[$minId]);
        }
    }
    return $columns;
}

/**
 * @desc save feeds and tabs to resourceDataDir / FEEDS_SERIALIZED_FILENAME file
 *
 * @return boolean result
 */
function saveFeeds(){
	global $rssFeeds;
	global $tabs;

	/**
	 * Save to file using serialization
	 */
	file_put_contents(cfAppResourceDir().'/'.FEEDS_SERIALIZED_FILENAME,serialize(
		array(
			$rssFeeds,
			$tabs,
			cfRGetVar('selectedTab')
		)
	));
	cfRSetVar('needSave',false);
	return true;
}

/**
 * @desc Clean a link that will be included in a JS function
 *
 * @param string $url
 */
function linkSanitize($url){
	return addslashes(str_replace(array("\n","\r","\0"),'',trim($url)));
}

/**
 * @desc Insert javascript
 *
 */
function insertJavascript(){
	global $editAllowed;
	global $importedFeeds;
	echo cfScriptLink('RSS.js');
?>
<script type="text/javascript">
<?php echo cfJSCaptionVar('bookmarksInvalidURL');?>
<?php echo cfJSCaptionVar('genRefresh');?>
<?php echo cfJSCaptionVar('genConfirmSuppress');?>
<?php echo cfJSRVar('nbColumns');?>
<?php echo cfJSRVar('updatesCheckFrequency');?>
<?php echo cfJSRVar('maxItems');?>
<?php echo cfJSRVar('linkImageWidth');?>
<?php echo cfJSRVar('linkImageHeight');?>
<?php echo cfJSIconURL('loading');?>
<?php echo cfJSIconURL('change');?>
<?php echo cfJSIconURL('alert');?>
<?php echo 'var '.colJSArray();?>;
wl.rVar.noPopup=<?php echo (int)cfHGetVar('noPopup');?>;
var newFeedBtTemplate='<?php echo outBlockItem('foo',outImageIcon('add').'&nbsp;CONTENT','','min-width:100%;width:100%;',array('onclick'=>'ACTION'))?>';
var editAllowed=<?php echo (int)$editAllowed;?>;
<?php if(cfGetBrowser()=='gecko') echo 'W.onkeypress=checkKey;'; ?>
var tabIcons=[<?php
if(!cfRGetVar('tabIcons')){$ti='';	foreach (glob(cfAppDocRoot().'/gfx/*.png') as $key=>$value) $ti.=',"'.cfFileWithoutExtension(basename($value)).'"';	cfRSetVar('tabIcons',substr($ti,1));}
echo cfRGetVar('tabIcons');
?>];

/**
 * General options
 */
var feedsDisplay="<?php echo cfRGetVar('feedsDisplay');?>";
function feedsDisplayPopupMenu(){
	var ico="<?php echo outIcon('ok16');?>",items=[
		ContextMenuItem((feedsDisplay=='number')?ico:'',"<?php echo cfRGetVar('maxItems');?>","setFeedsDisplay('number')"),
		ContextMenuItem((feedsDisplay=='day')?ico:'',"<?php echo cfCaption('genToday');?>","setFeedsDisplay('day')"),
		ContextMenuItem((feedsDisplay=='last')?ico:'',"<?php echo cfCaption('genNotRead');?>","setFeedsDisplay('last')"),
		ContextMenuItem((feedsDisplay=='all')?ico:'',"<?php echo cfCaption('genAll');?>","setFeedsDisplay('all')"),
		ContextMenuItem('sep'),
		ContextMenuItem('','<?php echo outImage(outIcon('displayList'));?>',"setAllItemsDisplay(1)"),
		ContextMenuItem('','<?php echo outImage(outIcon('displayFull'));?>',"setAllItemsDisplay(0)")
	];
	new ContextMenu(items);
}

function optionsPopupMenu(){
	var items=[],i=0;
<?php
	foreach ($importedFeeds+array('opml'=>false) as $applicationName=>$isImported) if(substr($applicationName,0,4)!='FILE') {
		switch ($applicationName){
			case 'Firefox': $icon=outIcon('iconFF'); $caption='Firefox';$action='addImportedFeeds(\''.$applicationName.'\')';break;
			case 'Internet Explorer 7': $icon=outIcon('iconIE'); $caption='Internet Explorer';$action='addImportedFeeds(\''.$applicationName.'\')';break;
			case 'opml': $icon=outIcon('opml'); $caption=cfCaption('genFile').' - OPML'; $action='showFilePathForm()';break;
		}
?>
	items[i++]=ContextMenuItem("<?php echo $icon;?>","<?php echo cfCaption('genImport').' - '.$caption;?>","<?php echo $action;?>");
<?php
	}
?>
	new ContextMenu(items);
}
</script>
<?php
}

/**
 * @desc Create JS col array from PHP $cols array
 * 		col format : array(
 * 						array( // column 0
 * 								top feed id, bottom feed id, ...),
 * 						array( // column 1
 * 								top feed id, bottom feed id, ...)
 * 						...)
 *
 * @return string HTML code
 */
function colJSArray(){
	global $cols;
	$output='col=[';
	foreach ($cols as $colNb=>$colItems){
		if($colNb) $output.=',';
		$output.='[';
		foreach ($colItems as $index=>$itemId) $output.=(($index)?',':'').'"feedItem'.$itemId.'"';
		$output.=']';
	}
	$output.="];\n";
	return $output;
}

/**
 * @desc generate tabs row
 *
 * @param string $rowId
 * @param array $tabs
 * @param string $onSelectFunction : name of javascript function called on tab selection
 * @param boolean $draggable : true if tabs can be dragged and repositioned
 * @return unknown
 */
function outTabRow($rowId,$tabs,$onSelectFunction,$draggable=true, $editable=false, $extraHTML=false, $selTabExtraInnerHTML=false,$asyncUpdate=false){
	if($asyncUpdate) $output=''; else $output='<span class="tabRow" id="'.$rowId.'" '.$extraHTML.'>';
	$output.='<span style="display:none"><span id="tabOSF_'.$rowId.'">'.$onSelectFunction.'</span>';
	if($selTabExtraInnerHTML) $output.='<span id="tabSH_'.$rowId.'">'.str_replace('ROWID',$rowId,$selTabExtraInnerHTML).'</span>';
	$output.='</span>';

	// New tab tab
	if(cfRGetVar('editionAllowed')){
		$tab=new tab('plus');
		$tab->setIcon(outIcon('newTab'));
		$tab->tooltip(cfCaption('genNewTab'));
		$tab->onclick('tabCreate(\'topTabs\')');
		$tab->setStatic();
		$tabs['plus']=$tab;
	}

	// Tabs HTML
	foreach ($tabs as $tabId=>$tab){
		if($tabId===0) $tabId='0';
		$output.='<a name="tAb_'.$rowId.'" id="'.$rowId.'_'.$tabId.'" ';
		if($tab->selected) $output.=' class="tabSel"'; else $output.=' class="tab"';

		// Action
		if($tab->onclickFunction)
			$output.='href="javascript:'.$tab->onclickFunction.'"';
		else
			$output.='href="javascript:if(tabSel(\''.$tabId.'\',\''.$rowId.'\')) '.$onSelectFunction.'(\''.$tabId.'\',\''.$rowId.'\')"';

		// Tooltip
		if($tab->tooltipCaption) $output.=outToolTip($tab->tooltipCaption);

		// Drag behaviour
		if($draggable && !$tab->static) {
			$output.=' ondrop="tabDrop(\''.$tabId.'\',\''.$rowId.'\',false)"';
			$output.=' onpick="tabPick(\''.$tabId.'\',\''.$rowId.'\')"';
			cfDragAddItem($rowId.'_'.$tabId,'+HORIZONTAL+INERTIA');
		}
		$output.='>';

		// icon
		if($tab->icon()) $output.=outImage($tab->icon(),false,'class="ico" id="'.$rowId.'_'.$tabId.'_icon"',(($tab->name)?'margin-right:0.5em':''));
		else $output.=outImage(outIcon('v'),false,'id="'.$rowId.'_'.$tabId.'_icon"','vertical-align:middle');
		// title
		if($tab->name) {
			$output.='<span id="'.$rowId.'_'.$tabId.'_c">'.$tab->name.'</span>&nbsp;';
		}
		// extraInnerHTML
		$output.='<span id="'.$rowId.'_'.$tabId.'_sh">'.(($tab->selected)?str_replace('ROWID',$rowId,$selTabExtraInnerHTML):'').'</span>';
		$output.="</a>\n";
	}
	if(!$asyncUpdate) $output.='</span>';

	if(!$asyncUpdate && !isset($_ENV['outTabScripts'])){
		$_ENV['outTabScripts']=true; // Probably useless as scripts have been moved to RSS.js
	}
	return $output;
}

/**
 * @desc update $rssFeeds array from imported feeds
 * @return boolean true if feeds updated, false if no feed updated
 *
 */
function updateImportedFeeds($tab, $newTab=false){
	global $rssFeeds;
	global $tabs;

	require_once(INCLUDE_DIR.'bookmarkFunctions.php');
	$impf=array();

	$updated=false;
	// Firefox feeds
	if($tab->imported=='Firefox' && file_exists($file=browserBookmarksLocation('firefox').'/bookmarks.html')){
		if($tab->name==false) $tabs[$tab->id]->name='Firefox';
		if($tab->iconURL===0) $tabs[$tab->id]->iconURL='/gfx/iconFF.png';

		$dom = new DOMDocument();
		@$dom->loadHTMLFile($file);
		foreach (@$dom->getElementsByTagName('a') as $id=>$node) {
			if($node->hasAttribute('feedurl')) $impf[$node->getAttribute('feedurl')]=$node->nodeValue;
		}
	}

	// IE7 feeds
	elseif($tab->imported=='Internet Explorer 7' && is_dir($folder=cfGGetVar('specialFolderLocalAppData').'/Microsoft/Feeds')){
		if($tab->name==false) $tabs[$tab->id]->name='Internet Explorer';
		if($tab->iconURL===0) $tabs[$tab->id]->iconURL='/gfx/iconIE.png';
		importIE7Feeds($folder,$impf);
	}
	// OPML feeds file
	elseif (substr($tab->imported,0,4)=='FILE'){
		if(!file_exists($filename=substr($tab->imported,4))) return false;
		// Open and parse OPML file
		$dom = new DOMDocument();
		if(!@$dom->load($filename)) return false;
		// Tab title
		if($tab->name==false){
			foreach (@$dom->getElementsByTagName('title') as $node) {
				$tabs[$tab->id]->name=$node->nodeValue;
				break;
			}
			if($tab->name==false) $tabs[$tab->id]->name=cfFileWithoutExtension(basename($filename));
		}
		// Tab icon
		if($tab->iconURL===0) $tabs[$tab->id]->iconURL='/gfx/opml.png';

		// Read feeds
		foreach (@$dom->getElementsByTagName('outline') as $node) {
			if($node->hasAttribute('xmlUrl')) $impf[$node->getAttribute('xmlUrl')]=((($node->hasAttribute('text'))?($node->getAttribute('text')):(($node->hasAttribute('title'))?($node->getAttribute('title')):false)));
			elseif($node->hasAttribute('url')) $impf[$node->getAttribute('url')]=((($node->hasAttribute('text'))?($node->getAttribute('text')):(($node->hasAttribute('title'))?($node->getAttribute('title')):false)));
		}
	}

	// Application not found
	else return false;

	// Update existing feeds titles
	foreach ($rssFeeds as $id=>$feed) if($feed->tab==$tab->id){
		foreach ($impf as $url=>$title){
			if(strtolower($feed->url)==strtolower($url)) {
				if(false && $rssFeeds[$id]->title!=$title){ // Title update disabled
					$rssFeeds[$id]->title==$title;
					$updated=true;
				}
				// Remove already-present feeds
				unset($impf[$url]);
			}
		}
	}

	// Create new feeds
	$columns=generateColsArray($tab->id);
	foreach ($impf as $url=>$title){
		$feed=new RSSFeed($url);
	    $feed->title=$title;
	    $feed->tab=$tab->id;
	    if($newTab) $feed->folded=true;

	    // Set feed's column / yIndex
	    $minCol=0;
	    $minNb=9999;
	    foreach ($columns as $colNb=>$colItems) if(count($colItems)<$minNb) {$minCol=$colNb; $minNb=count($colItems);}
	    $feed->col=$minCol;   $feed->yIndex=$minNb;

	    // Update XML
	    if(!$newTab) $feed->updateXML();

	    $columns[$minCol][]=$feed->id;

	    // Add feed to feeds list
	    $rssFeeds[$feed->id]=$feed;

	    unset($feed);$updated=true;
	}
	return $updated;
}

/**
 * @desc Echo Async HTML and JS code that rebuilds tabs
 *
 */
function insertTabsAsyncHTML(){
	global $rssFeeds;
	global $cols;
	global $tabsExtraHTML;
	global $tabs;

	// Insert new tab's HTML
	echo cfAsyncXMLInnerHTMLbyId(outTabRow('topTabs',$tabs,'selectTab',true,true,false,$tabsExtraHTML,true),'topTabs');
	// Rebuild drag items
	echo cfAsyncXMLJSaction('tabRebuildDragElts("topTabs")');
	// Set selected tab
	foreach ($tabs as $id=>$tab) if($tab->selected){echo cfAsyncXMLJSaction('tabSelected["topTabs"]="'.$id.'";');break;}
}

/**
 * @desc Echo Async HTML and JS code that rebuilds selected tab's feeds
 *
 */
function insertFeedsAsyncHTML(){
	global $rssFeeds;
	global $cols;
	// Clear displayed feeds
	echo cfAsyncXMLJSaction('clearAllFeeds(true);hideLIP()');
	// Regenerate cols array
	$cols=generateColsArray();
	// Insert tab's feeds
	$innerHTML='';
	foreach ($rssFeeds as $feed) if($feed->tab==cfRGetVar('selectedTab')) $innerHTML.=$feed->HTML(true,'visibility:hidden');
	if($innerHTML=='') $innerHTML=' ';

	echo cfAsyncXMLInnerHTMLbyId($innerHTML,'feedsDiv',false); // Insert feeds'HTML
	echo cfAsyncXMLJSaction(colJSArray()); // Set new "col" array
	echo cfAsyncXMLJSaction('rebuildDragElts(true)'); // Build draggable elements, reset size, positions...
	echo cfAsyncXMLJSaction('setSelectedTabIcon()'); // Reset selected tab's icon if replaced by "loading" icon
	echo cfAsyncXMLJSaction('checkUpdates(false)'); // Check if feeds must be updated
	echo cfAsyncXMLJSaction('setImages()'); // Set icons and images from values stored into alt
}

/**
 * @desc List feeds, see if favicon has been retreived. If so, populate ->favicon, else try to retreive it
 *
 */
function updateFavicons(){
	global $rssFeeds;
	$rssImage=outIcon('rss');
	$needSave=0;

	// 1st time function is called: reset links to favicons to check if they still exist
	if(!cfRGetVar('faviconChecked')){
		foreach ($rssFeeds as $k=>$v){
			if($rssFeeds[$k]->favicon && substr($rssFeeds[$k]->favicon,0,4)!='/gfx' && !file_exists(mfFaviconFile($rssFeeds[$k]->favicon))){
				$rssFeeds[$k]->favicon=false;
			}

		}
		cfRSetVar('faviconChecked',true);
	}

	$urls=array();
	// Search for missing favicons
	foreach ($rssFeeds as $k=>$v){
		if(!($v->favicon)) {
			if($v->redirectedUrl && mfFaviconFile($v->redirectedUrl)) $rssFeeds[$k]->favicon=$v->redirectedUrl;
			elseif (mfFaviconFile($v->url))
				$rssFeeds[$k]->favicon=(mfFaviconFile($v->url)!=='noIcon')?$v->url:$rssImage;
			// Favicon not retreived yet, add to list
			else{
				$urls[]=$v->url;
				if($v->redirectedUrl) $urls[]=$v->redirectedUrl;
			}
			$needSave=1;
		}
	}

	// Retreive missing favicons
	if(count($urls)){
		require_once(INCLUDE_DIR.'outputFunctions.php');

		if(!cfIsAsync()) outSyncProgress(false,false,count($urls));

		mfGetFavicons($urls,array('toFile'=>cfAppResourceDir(),'HTMLParsing'=>false));
		foreach ($rssFeeds as $k=>$v){
			if(!($v->favicon)) {
				if($v->redirectedUrl && ($ff=mfFaviconFile($v->redirectedUrl)) && $ff!='noIcon')
					$rssFeeds[$k]->favicon=$v->redirectedUrl;
				elseif(($ff=mfFaviconFile($v->url)) && $ff!='noIcon')
					$rssFeeds[$k]->favicon=$v->url;
				else
					$rssFeeds[$k]->favicon=$rssImage;
			}
		}

		// Reload page when done
		if(!cfIsAsync()) outSyncProgressUpdate('done');
	}
	if($needSave) saveFeeds();
}

/**
 * @desc Call this function in feed's html display to know if item should be displayed as new, as read, or not displayed. Call the function with init for $item and $maxItem to initialize for a feed
 * @param feed item $item: item to display (or not)
 * @param int $maxItems: max number of items to display for that feed
 * @return 'new', 'old' or false (if should not be displayed
 */
function rssItemsFilter($item,$maxItems=false){
	static $maxImportTime, $maxItems, $maxPubDate, $displayed, $displayed,$lastUpdate;

	if($item==='init'){
        $displayed=0;
        $maxItems=0;
        $maxPubDate=0;
        $maxImportTime=0;
        $displayed=0;
        $lastUpdate=0;

        // Get last display update from cookie
        if(isset($_COOKIE['weezoRSS'.WEnv::res('id')])) {
	    	$cookie=@$_COOKIE['weezoRSS'.WEnv::res('id')];
	    	$pairs=explode(',',$cookie);
	    	$tab=cfRGetVar('selectedTab').'=';
	    	foreach ($pairs as $pair){
	    		if(cfCmpLeft($pair,$tab)){
		        	$lastUpdate=1*substr($pair,strlen($tab));
		        	break;
	    		}
	    	}
        }
        if(cfRGetVar('feedsDisplay')=='number') $maxItems=($maxItems)?$$maxItems:cfRGetVar('maxItems');
        elseif(cfRGetVar('feedsDisplay')=='day') $maxPubDate=mktime(0,0,1,date('n'),date('j'),date('Y'));
        elseif(cfRGetVar('feedsDisplay')=='last' && $lastUpdate) $maxImportTime=$lastUpdate-(int)cfConstant('RSS_READ_TIME_MARGIN')*60;

		return;
	}

	if($maxItems && $displayed>=$maxItems) return false; // Limit number of items
	if($maxPubDate && $item['pubdate']<$maxPubDate) return false; // Day view
	if($maxImportTime && (int)$item['importTime']<$maxImportTime) return false; // Last time view
	$displayed++;

	return ($lastUpdate && $item['importTime']<$lastUpdate)?'old':'new';
}

function rrsSortFunction($itemA, $itemB){
	if($itemA['pubdate']==$itemB['pubdate']) return 0;
	return ($itemA['pubdate']<$itemB['pubdate'])?1:-1;
}

/*
 ***************************************************************************************************************************
 * START
 ***************************************************************************************************************************
 */
require_once(INCLUDE_DIR.'outputFunctions.php');
require_once(INCLUDE_DIR.'XML_RSS/RSS.php');
require_once(INCLUDE_DIR.'miscFunctions.php');

$importedFeeds=array('Firefox'=>false, 'Internet Explorer 7'=>false, 'FILE'=>false);

// Init resource
initResourceData();

// Edition/modification right
if(cfRGetVar('editionAllowed')) $editAllowed=true; else $editAllowed=false;

// Load feeds and tabs
loadFeeds();

// Retreive and populate favicons
updateFavicons();

// Tab's options button
if($editAllowed) $tabsExtraHTML='<img style="vertical-align:middle;cursor:pointer" src="/res/bookmarks/RSS/tabOpt.gif" onclick="tabOptionsShow(\'ROWID\',this)"/>';
else $tabsExtraHTML='';

//if(!cfIsAsync()) foreach ($rssFeeds as $key=>$value) cfDbg($key.'>"'.$value->title.'">'.$value->tab.' - '.$value->col,'');

/**
 * Mobiles
 */
if(cfIsMobile()) {require(dirname($_SERVER['SCRIPT_FILENAME']).'/mobileIndex.php');exit;}



rssProcessPost();

/*
 ***************************************************************************************************************************
 * Insert HEAD, javascripts and comForm
 ***************************************************************************************************************************
 */
cfInsertHEAD(false);
?>
<style type="text/css">.feedHeaderT{width:100%;table-layout:fixed;margin:0;padding:0}</style>
</head>
<body onload="init()">
<?php echo cfScriptLink('wz_dragdrop.js');?>
<div class="topDiv">
<?php

// Frame 1
echo outDivFrame('frame1','id="mainTable"','display:block;position:relative;height:100%');
$l='';

/**
 ***************************************************************************************************************************
 * TABS
 ***************************************************************************************************************************
 */
$l.=outTabRow('topTabs',$tabs,'selectTab',true,true,false,$tabsExtraHTML);

/**
 * Right-top controls
 */
$r='';
if(cfRGetVar('editionAllowed')) $r.=outButton(cfCaption('rssNewFeed'),'javascript:showNewFeedForm()',outIcon('newFeed'),cfCaption('rssNewFeed'),false,'style="margin-right:2em"');
$r.=outButton(cfCaption('genDisplay'),'javascript:feedsDisplayPopupMenu()',outIcon('calendar'),false,false,'style="margin-left:2em"');
$r.=outButton(cfCaption('genRefresh'),'javascript:checkUpdates(1)',outIcon('reload2'),false,false);
$r.=outButton('', 'javascript:unfoldAll(false)',outIcon('plusX2'),false,'unfoldAllBt','style="margin-left:2em"');
$r.=outButton('', 'javascript:collapseAll(false)',outIcon('minusX2'),false,'collapseAllBt');

if($editAllowed) $r.=outButton(false,'javascript:optionsPopupMenu()',outIcon('import'),cfCaption('genImport'),false,'style="margin-left:2em"');

echo outFrameHeaderTable('frame1Header',$l,$r,'id="mainTableHeader"');


// Com form
outInsertStandardComForm($_SERVER['PHP_SELF'],false,false,false,false,false,true,true);

/*
 ***************************************************************************************************************************
 * Items
 ***************************************************************************************************************************
 */

// insert separator
echo '<div class="dropTargetOver" style="position:relative;height:0px;visibility:hidden" id="separator"></div>';
cfDragAddItem("separator",'');

// Loading-in-progress div
echo '<table id="lip" style="width:100%;height:90%;text-align:center;'.((cfGetBrowser()=='opera')?'padding-top:30%;padding-bottom:30%;':'').'"><tr><td>';
echo outLoadingIcon(false,true);
echo '</td></td></table>';
flush();

// Feeds div
echo '<div id="feedsDiv">';
// Insert feeds
foreach ($rssFeeds as $feed) if(@$feed->tab===cfRGetVar('selectedTab')) echo $feed->HTML(true,'visibility:hidden');

?>
</div>
</div>
</div>
<?php



/*
 ***************************************************************************************************************************
 * New feed popup
 ***************************************************************************************************************************
 */
echo '<div id="newFeedForm" style="position:absolute;top:-500px">';
outShadowBefore();
echo '<div id="newFeedFormDiv" class="popup" style="position:relative;margin:0px;text-align:left;vertical-align:top;width:500px">';
echo outFrameHeaderTable("popupHeader",outImage(outIcon('newFeed'),false,false,'margin-right:1em;vertical-align:middle').cfCaption('rssNewFeed'),outButton(false,'javascript:hideNewFeedForm()',outIcon('close')),false,'cursor:move');
echo '<br/><b> &nbsp; &nbsp; '.cfCaption('genLink').'&nbsp;/&nbsp;'.cfCaption('genSite').'&nbsp;&nbsp;&nbsp;&nbsp;</b><input type="text" id="newFeedURL" onchange="checkFeedURL()" style="width:300px" onkeypress="checkKey()">';
echo outButton(cfCaption('genAdd'),'javascript:addFeed()',outIcon('add'),false,'newFeedFormBt').'<br>';
echo '<div id="newFeedURLError" class="warning" style="margin-left:1em">&nbsp;</div>';
echo '<div id="newFeedLinks" style="margin-left:1em;margin-right:1em">&nbsp;</div>';
echo "</div>";
outShadowAfter();
echo "</div>\n";
cfDragAddItem("newFeedForm",'+NO_SCROLL+INERTIA');

/*
 ***************************************************************************************************************************
 * Delete feed form
 ***************************************************************************************************************************
 */
echo '<div id="deleteFeedForm" style="position:absolute;top:-500px;">';
outShadowBefore('');
echo '<div class="popup" style="position:relative;margin:0px;text-align:left;vertical-align:top;width:400px">';
echo outFrameHeaderTable("popupHeader",outImage(outIcon('warning'),false,false,'margin-right:1em;vertical-align:middle').cfCaption('genDelete'),'',false,'cursor:move');
echo '<center><br/>'.cfCaption('genConfirmSuppress','<span id="suppressFeedName"></span><span id="suppressFeedId" style="display:none"></span>').'<br/><br/>';
echo outButton(cfCaption('genDelete'),'javascript:deleteFeed()',outIcon('del'),false,false,'style="margin-right:2em"');
echo outButton(cfCaption('genCancel'),'javascript:hideDeleteFeedForm()',outIcon('back')).'<br/>&nbsp;</center>';
echo "</div>";
outShadowAfter();
echo "</div>\n";
cfDragAddItem("deleteFeedForm",'+NO_SCROLL+INERTIA');

/*
 ***************************************************************************************************************************
 * Feed Options form
 ***************************************************************************************************************************
 */
?>
<div id="feedOptionsForm" style="position:absolute;top:-500px;">
<div class="helpFrame helpBg" style="margin:0px;text-align:left;vertical-align:top; width:auto; height:auto; padding:1em">
<div nowrap="nowrap" style="display:block;">
<?php echo outThemeImage(outIcon('gears'),false,'id="optionIcon"','position:absolute;left:8px;top:9px'); ?>
<h1><?php echo cfCaption('mainConfigButton');?></h1></div>
<span id="optionsFeedId" style="display:none"></span>

<div style="position:absolute;right:1em;top:0.5em;overflow:hidden;" nowrap="nowrap"><?php echo outButton(cfCaption('genRefresh'),'javascript:optionRefreshFeed(\'feedItem\')',outIcon('change'));?></div><br><br>

<form onsubmit="W.setTimeout('optionChangeURL()',1);return false;"><b><?php echo cfCaption('genURL');?></b><input id="optionURL" onkeydown="this.style.color=''" onchange="this.style.color=''" type="text" size="1" id="optionMaxItems" style="width:11em;margin-left:1em;"><?php if($editAllowed) echo outButton(false,'javascript:optionChangeURL()',outIcon('ok16'));?></form>

<div nowrap="nowrap" style="display:none"><?php echo cfCaption('rssNbLines'); ?>
<input type="text" size="1" id="optionMaxItems" style="margin-left:1em;margin-right:1em" onchange="optionMaxItems()" onkeyup="optionMaxItems()">
<input type="checkbox" id="optionMaxItemsDefault" style="margin-right:1em" onclick="optionMaxItemsDefault()"><?php echo cfCaption('genDefaultValue');?>
</div><br>

<div nowrap="nowrap">
<?php echo cfCaption('genDisplay'); ?>&nbsp;&nbsp;
<input onclick="optionCompactDisplay()" type = "radio" name="optionCompactDisplay" value="true"<?php if(true) echo ' checked'; ?>><?php echo outImage(outIcon('displayList'),false,false,'vertical-align:middle;margin:1em');?>
<input onclick="optionCompactDisplay()" type = "radio" name="optionCompactDisplay" value="false"<?php if(false) echo ' checked'; ?>><?php echo outImage(outIcon('displayFull'),false,false,'vertical-align:middle;margin:1em');?></div>

<div style="position:absolute;right:1em" nowrap="nowrap"><?php if($editAllowed) echo outButton(cfCaption('genDelete'),'javascript:optionDeleteFeed(optionFeedId)',outIcon('del')); ?></div><br/><br/>
</div></div>
<?php
cfDragAddItem("feedOptionsForm",'+INERTIA');

/*
 ***************************************************************************************************************************
 * Tab Options form
 ***************************************************************************************************************************
 */
?>
<div id="tabOptionsForm" style="position:absolute;top:-500px;">
<div class="helpFrame helpBg" style="margin:0px;text-align:left;vertical-align:top; width:45em; height:auto; padding:1em">
<div nowrap="nowrap" style="display:block;">

<h1><?php echo outThemeImage(outIcon('tab'),false,'id="tabOptionsFormIcon"','position:relative;left:-20px;vertical-align:middle'); ?><input type="text" class="input" id="tabOptionsTitle" onfocus="tabEditCaption()" onblur="tabDoneEditCaption()" onkeydown="tabOptionsTitleKD()">&nbsp;<?php echo outButton(false,'javascript:tabDoneEditCaption()',outIcon('ok16'));?></h1></div>
<div style="position:absolute;right:1em;top:0.7em"><?php echo outButton(cfCaption('genRefresh'),'javascript:checkUpdates(true)',outIcon('change'),false,false,'style="margin-right:1em"').outButton(cfCaption('genDelete'),'javascript:tabDel()',outIcon('del'));?></div><br>

<?php
// Add feed button
echo outButton(cfCaption('rssNewFeed'),'javascript:tabOptionsHide();showNewFeedForm()',outIcon('newFeed'), false, false,'style=""');
?>
<br/><br/>
<b><?php echo cfCaption('userIcon');?></b><br/>
<span id="tabIcons"></span>
<br/><br/>
</div></div>
<?php
cfDragAddItem("tabOptionsForm",'+INERTIA');

/*
 ***************************************************************************************************************************
 * File path
 ***************************************************************************************************************************
 */
echo '<div id="filePathForm" style="position:absolute;top:-500px;">';
echo outShadowBefore('500px').'<div class="popup" style="margin:0px;text-align:left;vertical-align:top">';
echo outFrameHeaderTable("popupHeader",outImage(outIcon('opml'),false,false,'margin-right:1em;vertical-align:middle').cfCaption('genImport'),outButton(false,'javascript:hideFilePathForm()',outIcon('close')),false,'cursor:move');
echo '<br/> &nbsp; &nbsp; '.cfCaption('genFilePath','OPML').' &nbsp; &nbsp; <input type="text" id="filePath" style="width:300px"/>';
echo outButton(cfCaption('genAdd'),'javascript:addImportedFeeds(\'FILE\'+dgi(\'filePath\').value);hideFilePathForm();',outIcon('add'),false,'newFeedFormBt').'<br><div id="newFeedURLError" class="warning" style="margin-left:1em">&nbsp;</div>';
echo "</div>".outShadowAfter()."</div>\n";
cfDragAddItem("filePathForm",'+NO_SCROLL+INERTIA');

// Screen mask
echo '<div id="screenMask" style="position:absolute; top:-200%; left:0px; width:100%; height:100%; background:black;" onmouseover="screenMaskOver()"></div>';
cfDragAddItem("screenMask",'+NO_DRAG');

// Javascript stuff
insertJavascript();
echo cfDragRegisterItems();
?>
<script type="text/javascript">
rssUpdateCookie('<?php echo cfRGetVar('selectedTab');?>',<?php echo time();?>);
</script>
</body>
</html>